@@ -1,7 +1,7 @@
 #!/bin/sh
 # Run this to set up the build system: configure, makefiles, etc.
 
-# $Id: autogen.sh,v 1.3 2002/04/23 12:41:55 easysw Exp $
+# $Id: autogen.sh,v 1.3.6.1 2004/03/22 01:39:27 giles Exp $
 
 package="ghostscript"
 
@@ -21,8 +21,9 @@ cd "$srcdir"
 echo "Generating configuration files for $package, please wait...."
 
 echo "  creating links to configuration sources"
-ln -sf src/configure.ac .
-ln -sf src/Makefile.in .
+rm -f configure.ac Makefile.in
+ln -s src/configure.ac .
+ln -s src/Makefile.in .
 
 #echo "  aclocal $ACLOCAL_FLAGS"
 #aclocal $ACLOCAL_FLAGS
