@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: bfont.h,v 1.4 2003/07/13 04:43:17 easysw Exp $ */
+/* $Id: bfont.h,v 1.9 2003/03/21 19:14:24 igor Exp $ */
 /* Interpreter internal routines and data needed for building fonts */
 /* Requires gxfont.h */
 
@@ -24,8 +24,8 @@
 #include "ifont.h"
 
 /* In zfont.c */
-int add_FID(P4(i_ctx_t *i_ctx_p, ref *pfdict, gs_font *pfont,
-	       gs_ref_memory_t *imem));
+int add_FID(i_ctx_t *i_ctx_p, ref *pfdict, gs_font *pfont,
+	    gs_ref_memory_t *imem);
 
 font_proc_make_font(zdefault_make_font);
 font_proc_make_font(zbase_make_font);
@@ -49,33 +49,36 @@ typedef enum {
 } build_font_options_t;
 
 /* In zbfont.c */
-int build_proc_name_refs(P3(build_proc_refs * pbuild,
-			    const char *bcstr,
-			    const char *bgstr));
-int build_gs_font_procs(P2(os_ptr, build_proc_refs *));
+int build_proc_name_refs(build_proc_refs * pbuild,
+			 const char *bcstr, const char *bgstr);
+int build_gs_font_procs(os_ptr, build_proc_refs *);
 #define BUILD_BASE_FONT_PROC(proc)\
-  int proc(P7(i_ctx_t *, os_ptr, gs_font_base **, font_type,\
-	      gs_memory_type_ptr_t, const build_proc_refs *,\
-	      build_font_options_t))
+  int proc(i_ctx_t *, os_ptr, gs_font_base **, font_type,\
+	   gs_memory_type_ptr_t, const build_proc_refs *,\
+	   build_font_options_t)
 typedef BUILD_BASE_FONT_PROC((*build_base_font_proc_t));
 BUILD_BASE_FONT_PROC(build_gs_primitive_font);
-int build_gs_FDArray_font(P6(i_ctx_t *, /*const*/ ref *, gs_font_base **,
-			     font_type, gs_memory_type_ptr_t,
-			     const build_proc_refs *));
-int build_gs_outline_font(P8(i_ctx_t *, os_ptr, gs_font_base **, font_type,
-			     gs_memory_type_ptr_t, const build_proc_refs *,
-			     build_font_options_t, build_base_font_proc_t));
+int build_gs_FDArray_font(i_ctx_t *, /*const*/ ref *, gs_font_base **,
+			  font_type, gs_memory_type_ptr_t,
+			  const build_proc_refs *);
+int build_gs_outline_font(i_ctx_t *, os_ptr, gs_font_base **, font_type,
+			  gs_memory_type_ptr_t, const build_proc_refs *,
+			  build_font_options_t, build_base_font_proc_t);
 BUILD_BASE_FONT_PROC(build_gs_simple_font);
-void init_gs_simple_font(P3(gs_font_base *pfont, const double bbox[4],
-			    const gs_uid *puid));
-void lookup_gs_simple_font_encoding(P1(gs_font_base *));
-int build_gs_font(P7(i_ctx_t *, os_ptr, gs_font **, font_type,
-		     gs_memory_type_ptr_t, const build_proc_refs *,
-		     build_font_options_t));
-int build_gs_sub_font(P8(i_ctx_t *, const ref *, gs_font **,
-			 font_type, gs_memory_type_ptr_t,
-			 const build_proc_refs *, const ref *, ref *));
-int define_gs_font(P1(gs_font *));
-gs_glyph zfont_encode_char(P3(gs_font *pfont, gs_char chr, gs_glyph_space_t ignored));
+void init_gs_simple_font(gs_font_base *pfont, const double bbox[4],
+			 const gs_uid *puid);
+void lookup_gs_simple_font_encoding(gs_font_base *);
+int build_gs_font(i_ctx_t *, os_ptr, gs_font **, font_type,
+		  gs_memory_type_ptr_t, const build_proc_refs *,
+		  build_font_options_t);
+int build_gs_sub_font(i_ctx_t *, const ref *, gs_font **,
+		      font_type, gs_memory_type_ptr_t,
+		      const build_proc_refs *, const ref *, ref *);
+int define_gs_font(gs_font *);
+void get_font_name(ref * pfname, const ref * op);
+void copy_font_name(gs_font_name * pfstr, const ref * pfname);
+gs_glyph zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t ignored);
+gs_char gs_font_map_glyph_to_unicode(gs_font *font, gs_glyph glyph);
+const ref *zfont_get_to_unicode_map(gs_font_dir *dir);
 
 #endif /* bfont_INCLUDED */
