@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: dirent_.h,v 1.3 2002/04/23 11:58:38 easysw Exp $ */
+/* $Id: dirent_.h,v 1.4 2002/02/21 22:24:51 giles Exp $ */
 /* Generic substitute for Unix dirent.h */
 
 #ifndef dirent__INCLUDED
@@ -23,13 +23,6 @@
 /* We must include std.h before any file that includes sys/types.h. */
 #include "std.h"
 
-/*
- * The location (or existence) of certain system headers is
- * environment-dependent. We detect this in the makefile
- * and conditionally define switches in gconfig_.h.
- */
-#include "gconfig_.h"
-
 /*
  * Directory entries may be defined in quite a number of different
  * header files.  The following switches are defined in gconfig_.h.
