@@ -15,7 +15,7 @@
 */
 
 
-/*$Id: dpmain.c,v 1.4 2003/07/13 04:43:17 easysw Exp $ */
+/* $Id: dpmain.c,v 1.8.2.1 2003/12/12 22:07:59 giles Exp $ */
 /* Ghostscript DLL loader for OS/2 */
 /* For WINDOWCOMPAT (console mode) application */
 
@@ -40,7 +40,7 @@
 #include <sys/select.h>
 #include "gscdefs.h"
 #define GS_REVISION gs_revision
-#include "errors.h"
+#include "ierrors.h"
 #include "iapi.h"
 #include "gdevdsp.h"
 
@@ -985,7 +985,7 @@ display_callback display = {
 int
 main(int argc, char *argv[])
 {
-    int code;
+    int code, code1;
     int exit_code;
     int exit_status;
     int nargc;
@@ -1051,7 +1051,9 @@ main(int argc, char *argv[])
 	code = gsdll.init_with_args(instance, nargc, nargv);
 	if (code == 0) 
 	    code = gsdll.run_string(instance, start_string, 0, &exit_code);
-	gsdll.exit(instance);
+	code1 = gsdll.exit(instance);
+	if (code == 0 || (code == e_Quit && code1 != 0))
+	    code = code1;
 
 	gsdll.delete_instance(instance);
     }
