@@ -15,7 +15,7 @@
 */
 
 
-/* $Id: dwtext.c,v 1.3 2002/04/23 12:43:43 easysw Exp $ */
+/* $Id: dwtext.c,v 1.8.2.1 2005/03/04 10:30:04 ghostgum Exp $ */
 
 /* Microsoft Windows text window for Ghostscript.
 
@@ -446,7 +446,8 @@ int n;
     while (cnt>0) {
 	p = tw->ScreenBuffer + tw->CursorPos.y*tw->ScreenSize.x + tw->CursorPos.x;
 	limit = tw->ScreenSize.x - tw->CursorPos.x;
-	for (count=0; (count < limit) && (cnt>0) && (isprint(*str) || *str=='\t'); count++) {
+	for (count=0; (count < limit) && (cnt>0) && 
+	    (isprint((unsigned char)(*str)) || *str=='\t'); count++) {
 	    if (*str=='\t') {
 		for (n = 8 - ((tw->CursorPos.x+count) % 8); (count < limit) & (n>0); n--, count++ )
 		    *p++ = ' ';
@@ -467,7 +468,7 @@ int n;
 		str++;
 		cnt--;
 	    }
-	    else if (!isprint(*str) && *str!='\t') {
+	    else if (!isprint((unsigned char)(*str)) && *str!='\t') {
 		text_putch(tw, *str++);
 		cnt--;
 	    }
@@ -906,58 +907,31 @@ WndTextProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 	    }
 	    return(0);
 	case WM_KEYDOWN:
-	    if (GetKeyState(VK_SHIFT) < 0) {
-	      switch(wParam) {
-		case VK_HOME:
-			SendMessage(hwnd, WM_VSCROLL, SB_TOP, (LPARAM)0);
-			break;
-		case VK_END:
-			SendMessage(hwnd, WM_VSCROLL, SB_BOTTOM, (LPARAM)0);
-			break;
-		case VK_PRIOR:
-			SendMessage(hwnd, WM_VSCROLL, SB_PAGEUP, (LPARAM)0);
-			break;
-		case VK_NEXT:
-			SendMessage(hwnd, WM_VSCROLL, SB_PAGEDOWN, (LPARAM)0);
-			break;
-		case VK_UP:
-			SendMessage(hwnd, WM_VSCROLL, SB_LINEUP, (LPARAM)0);
-			break;
-		case VK_DOWN:
-			SendMessage(hwnd, WM_VSCROLL, SB_LINEDOWN, (LPARAM)0);
-			break;
-		case VK_LEFT:
-			SendMessage(hwnd, WM_HSCROLL, SB_LINEUP, (LPARAM)0);
-			break;
-		case VK_RIGHT:
-			SendMessage(hwnd, WM_HSCROLL, SB_LINEDOWN, (LPARAM)0);
-			break;
-	      }
-	    }
-	    else {
-	        switch(wParam) {
-		    case VK_HOME:
-		    case VK_END:
-		    case VK_PRIOR:
-		    case VK_NEXT:
-		    case VK_UP:
-		    case VK_DOWN:
-		    case VK_LEFT:
-		    case VK_RIGHT:
-		    case VK_DELETE:
-		    { /* store key in circular buffer */
-			long count = tw->KeyBufIn - tw->KeyBufOut;
-			if (count < 0) count += tw->KeyBufSize;
-			if (count < tw->KeyBufSize-2) {
-			    *tw->KeyBufIn++ = 0;
-			    if (tw->KeyBufIn - tw->KeyBuf >= tw->KeyBufSize)
-				tw->KeyBufIn = tw->KeyBuf; /* wrap around */
-			    *tw->KeyBufIn++ = HIWORD(lParam) & 0xff;
-			    if (tw->KeyBufIn - tw->KeyBuf >= tw->KeyBufSize)
-				tw->KeyBufIn = tw->KeyBuf; /* wrap around */
-			}
-		    }
-	        }
+	    switch(wParam) {
+	      case VK_HOME:
+		      SendMessage(hwnd, WM_VSCROLL, SB_TOP, (LPARAM)0);
+		      break;
+	      case VK_END:
+		      SendMessage(hwnd, WM_VSCROLL, SB_BOTTOM, (LPARAM)0);
+		      break;
+	      case VK_PRIOR:
+		      SendMessage(hwnd, WM_VSCROLL, SB_PAGEUP, (LPARAM)0);
+		      break;
+	      case VK_NEXT:
+		      SendMessage(hwnd, WM_VSCROLL, SB_PAGEDOWN, (LPARAM)0);
+		      break;
+	      case VK_UP:
+		      SendMessage(hwnd, WM_VSCROLL, SB_LINEUP, (LPARAM)0);
+		      break;
+	      case VK_DOWN:
+		      SendMessage(hwnd, WM_VSCROLL, SB_LINEDOWN, (LPARAM)0);
+		      break;
+	      case VK_LEFT:
+		      SendMessage(hwnd, WM_HSCROLL, SB_LINEUP, (LPARAM)0);
+		      break;
+	      case VK_RIGHT:
+		      SendMessage(hwnd, WM_HSCROLL, SB_LINEDOWN, (LPARAM)0);
+		      break;
 	    }
 	    break;
 	case WM_CHAR:
