@@ -1,4 +1,4 @@
-/* Copyright (C) 1999, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 1999-2005, Ghostgum Software Pty Ltd.  All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-// $Id: dwuninst.cpp,v 1.3 2002/04/23 11:58:40 easysw Exp $
+// $Id: dwuninst.cpp,v 1.4.2.2 2005/03/04 23:51:06 ghostgum Exp $
 
 #define STRICT
 #include <windows.h>
@@ -38,6 +38,13 @@
 #define MAXSTR 256
 #define UNINSTALLKEY TEXT("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall")
 
+#ifdef _WIN64
+#define DLGRETURN INT_PTR
+#else
+#define DLGRETURN BOOL
+#endif
+
+
 HWND hDlgModeless;
 HWND hText1;
 HWND hText2;
@@ -673,7 +680,7 @@ BOOL shell_old(void)
 #ifdef __BORLANDC__
 #pragma argsused
 #endif
-BOOL CALLBACK _export
+DLGRETURN CALLBACK _export
 RemoveDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
   switch(message) {
@@ -785,7 +792,7 @@ init(void)
 	}
 	if (strlen(szLogFile) == 0) {
 		MessageBox(HWND_DESKTOP, "Usage: uninstgs logfile.txt", 
-			"GNU Ghostscript Uninstall", MB_OK);
+			"GPL Ghostscript Uninstall", MB_OK);
 		return FALSE;
 	}
 	
