@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevbjc.h,v 1.4 2003/08/19 15:24:41 tillkamppeter Exp $*/
+/* $Id: gdevbjc.h,v 1.5 2002/10/07 08:28:56 ghostgum Exp $*/
 
 #ifndef gdevbjc_INCLUDED
 #  define gdevbjc_INCLUDED
@@ -93,11 +93,14 @@
  */
 
 #define BJC_MARGINS_LETTER \
-    (6.5 / 25.4), BJC_LOWER_LIMIT, (6.5 / 25.4), BJC_PRINT_LIMIT
+    (float)(6.5 / 25.4), (float)BJC_LOWER_LIMIT, \
+    (float)(6.5 / 25.4), (float)BJC_PRINT_LIMIT
 #define BJC_MARGINS_A4 \
-    (3.4 / 25.4), BJC_LOWER_LIMIT, (3.4 / 25.4), BJC_PRINT_LIMIT
+    (float)(3.4 / 25.4), (float)BJC_LOWER_LIMIT, \
+    (float)(3.4 / 25.4), (float)BJC_PRINT_LIMIT
 #define BJC_MARGINS_A3 \
-    (4.0 / 25.4), BJC_LOWER_LIMIT, (4.0 / 25.4), BJC_A3_PRINT_LIMIT
+    (float)(4.0 / 25.4), (float)BJC_LOWER_LIMIT, \
+    (float)(4.0 / 25.4), (float)BJC_A3_PRINT_LIMIT
 
 /*
  * Drivers options names.
