@@ -1,4 +1,4 @@
-/* Copyright (C) 1995, 1996, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1995-2003 artofcode LLC.  All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,12 +14,13 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevcgm.c,v 1.4 2002/04/23 11:58:42 easysw Exp $ */
+/* $Id: gdevcgm.c,v 1.6.2.2 2004/08/27 21:36:36 dan Exp $ */
 /* CGM (Computer Graphics Metafile) driver */
 #include "memory_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gxdevice.h"
+#include "gp.h"
 #include "gsparam.h"
 #include "gdevcgml.h"
 #include "gdevpccm.h"
@@ -30,11 +31,9 @@
 	  do masked copy_mono with cell array if possible
  ****************/
 
-#define fname_size 80
-
 typedef struct gx_device_cgm_s {
     gx_device_common;
-    char fname[fname_size + 1];
+    char fname[gp_file_name_sizeof];
     FILE *file;
     cgm_state *st;
     bool in_picture;
@@ -95,7 +94,7 @@ cgm_device("cgmmono", 1, 1, 2,
 	   gx_default_map_rgb_color, gx_default_w_b_map_color_rgb);
 
 gx_device_cgm gs_cgm8_device =
-cgm_device("cgm8", 8, 6, 7,
+cgm_device("cgm8", 8, 5, 6,
 	   pc_8bit_map_rgb_color, pc_8bit_map_color_rgb);
 
 gx_device_cgm gs_cgm24_device =
@@ -149,10 +148,13 @@ cgm_open(gx_device * dev)
     {-1, 1};
     cgm_metafile_elements meta;
     cgm_result result;
-
+#if 0
     cdev->file = fopen(cdev->fname, "wb");
+#else
+    (void)gx_device_open_output_file(dev, cdev->fname, 1, 0, &cdev->file);
     if (cdev->file == 0)
 	return_error(gs_error_ioerror);
+#endif
     cal.private_data = cdev;
     cal.alloc = cgm_gs_alloc;
     cal.free = cgm_gs_free;
@@ -216,8 +218,15 @@ cgm_close(gx_device * dev)
     result = cgm_terminate(cdev->st);
     check_result(result);
     cdev->st = 0;
+#if 0
     fclose(cdev->file);
     cdev->file = 0;
+#else
+    code = gx_device_close_output_file(dev, cdev->fname, cdev->file);
+    cdev->file = 0;
+    if (code < 0)
+	return code;
+#endif
     return 0;
 }
 
@@ -228,7 +237,6 @@ cgm_get_params(gx_device * dev, gs_param_list * plist)
     gx_device_cgm *cdev = (gx_device_cgm *) dev;
     int code = gx_default_get_params(dev, plist);
     gs_param_string ofns;
-
     if (code < 0)
 	return code;
     ofns.data = (const byte *)cdev->fname,
@@ -249,11 +257,13 @@ cgm_put_params(gx_device * dev, gs_param_list * plist)
 
     switch (code = param_read_string(plist, (param_name = "OutputFile"), &ofs)) {
 	case 0:
-	    if (dev->LockSafetyParams) {
+	    if (dev->LockSafetyParams &&
+		    bytes_compare(ofs.data, ofs.size,
+			(const byte *)cdev->fname, strlen(cdev->fname))) {
 	        ecode = gs_note_error(gs_error_invalidaccess);
 		goto ofe;
 	    }
-	    if (ofs.size > fname_size)
+	    if (ofs.size >= gp_file_name_sizeof)
 		ecode = gs_error_limitcheck;
 	    else
 		break;
@@ -271,15 +281,25 @@ cgm_put_params(gx_device * dev, gs_param_list * plist)
     code = gx_default_put_params(dev, plist);
     if (code < 0)
 	return code;
-
     if (ofs.data != 0) {	/* Close the file if it's open. */
 	if (cdev->file != 0) {
+#if 0
 	    fclose(cdev->file);
 	    cdev->file = 0;
+#else
+	    code = gx_device_close_output_file(dev, cdev->fname, cdev->file);
+	    cdev->file = 0;
+	    if (code < 0)
+		return code;
+#endif
 	}
 	memcpy(cdev->fname, ofs.data, ofs.size);
 	cdev->fname[ofs.size] = 0;
+#if 0
 	cdev->file = fopen(cdev->fname, "wb");
+#else
+        (void)gx_device_open_output_file(dev, cdev->fname, 1, 0, &cdev->file);
+#endif
 	if (cdev->file == 0)
 	    return_error(gs_error_ioerror);
     }
@@ -388,13 +408,12 @@ cgm_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     cgm_color fill_color;
     cgm_point points[2];
     cgm_result result;
-
+    
     fit_fill(dev, x, y, w, h);
     if (!cdev->in_picture) {	/* Check for erasepage. */
-	if (color == (*dev_proc(dev, map_rgb_color)) (dev,
-				     gx_max_color_value, gx_max_color_value,
-						      gx_max_color_value)
-	    )
+	gx_color_value blank[3] = {gx_max_color_value, gx_max_color_value, 
+				   gx_max_color_value};
+	if (color == (*dev_proc(dev, encode_color)) (dev, blank))
 	    return 0;
 	cgm_begin_picture(cdev);
     }
