@@ -1,39 +1,20 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 2000 artofcode LLC. All rights reserved.
 
-   This program may be distributed as part of AFPL Ghostscript, under the
-   terms of the Aladdin Free Public License (the "License").
+  This software is provided AS-IS with no warranty, either express or
+  implied.
+   
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
+   
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+*/
 
-   AFPL Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No
-   author or distributor accepts any responsibility for the consequences of
-   using it, or for whether it serves any particular purpose or works at
-   all, unless he or she says so in writing.  Refer to the License for full
-   details.
-
-   Every copy of AFPL Ghostscript must include a copy of the License,
-   normally in a plain ASCII text file named PUBLIC.  The License grants you
-   the right to copy, modify and redistribute AFPL Ghostscript, but only
-   under certain conditions described in the License.  Among other things,
-   the License requires that the copyright notice and this notice be
-   preserved on all copies.
-
-   This program may also be distributed and/or modified under the terms of
-   the GNU General Public License as published by the Free Software
-   Foundation (the "GPL"); either version 2 of the GPL, or (at your option)
-   any later version.
-
-   When distributed under the terms of the GPL, this program is distributed
-   in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-   PURPOSE.  See the GPL for more details.
-
-   If this program is being distributed under the terms of the GPL, you
-   should have received a copy of the GPL along with this program, normally
-   in a plain ASCII text file named COPYING; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111
-   U.S.A.
- */
-
-/*$Id: gdevdjet.c,v 1.5 2002/07/20 21:03:21 tillkamppeter Exp $ */
+/* $Id: gdevdjet.c,v 1.10.2.1 2004/01/30 00:24:37 giles Exp $ */
 /* HP LaserJet/DeskJet driver for Ghostscript */
 #include "gdevprn.h"
 #include "gdevdljm.h"
@@ -104,14 +85,14 @@
  * inexact paperlength which is set to 117 10ths.
  * Somebody should check for letter sized paper. I left it at 0.07".
  */
-#define DESKJET_MARGINS_LETTER  0.2, 0.45, 0.3, 0.05
-#define DESKJET_MARGINS_A4	0.125, 0.5, 0.143, 0.09
+#define DESKJET_MARGINS_LETTER  (float)0.2, (float)0.45, (float)0.3, (float)0.05
+#define DESKJET_MARGINS_A4	(float)0.125, (float)0.5, (float)0.143, (float)0.09
 /* Similar margins for the LaserJet. */
 /* These are defined in the PCL 5 Technical Reference Manual. */
 /* Note that for PCL 5 printers, we get the printer to translate the */
 /* coordinate system: the margins only define the unprintable area. */
-#define LASERJET_MARGINS_A4	0.167, 0.167, 0.167, 0.167
-#define LASERJET_MARGINS_LETTER	0.167, 0.167, 0.167, 0.167
+#define LASERJET_MARGINS_A4	(float)0.167, (float)0.167, (float)0.167, (float)0.167
+#define LASERJET_MARGINS_LETTER	(float)0.167, (float)0.167, (float)0.167, (float)0.167
 
 /* See gdevdljm.h for the definitions of the PCL_ features. */
 
@@ -283,9 +264,13 @@ hpjet_close(gx_device * pdev)
 
     if (code < 0)
 	return code;
-    if (ppdev->Duplex_set >= 0 && ppdev->Duplex)
-	fputs("\033&l0H", ppdev->file);
-    fputs("\033E", ppdev->file);
+    if (ppdev->PageCount > 0) {
+	if (ppdev->Duplex_set >= 0 && ppdev->Duplex)
+	    fputs("\033&l0H", ppdev->file);
+
+	fputs("\033E", ppdev->file);
+    }
+
     return gdev_prn_close(pdev);
 }
 
