@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
  
-/*$Id: gdevimgn.c,v 1.3 2002/04/23 11:58:42 easysw Exp $*/
+/* $Id: gdevimgn.c,v 1.6 2002/02/21 22:24:51 giles Exp $*/
 /*
  * Imagen ImPRESS printer driver - version 1.4
  *
@@ -71,6 +71,7 @@
  
 /* -------------------------------------------------------- */ 
 #include "gdevprn.h" 
+#include "gxobj.h" 
 /* #include <stdio.h> should not be used in drivers */
 #include <stdlib.h>
  
@@ -354,9 +355,9 @@ private int
 imagen_print_page(gx_device_printer *pdev, FILE *prn_stream) 
 { 
   int line_size = gdev_mem_bytes_per_scan_line((gx_device *)pdev); 
+  const int align_size = obj_align_round((line_size/BIGSIZE)+1);
   /* input buffer: one line of bytes rasterized by gs */
-  byte *in = (byte *)gs_malloc(BIGSIZE, line_size / BIGSIZE + 1,
-	"imagen_print_page(in)"); 
+  byte *in = (byte *)gs_malloc(BIGSIZE, align_size, "imagen_print_page(in)"); 
   /* output buffer: 32 lines, interleaved into imPress swatches */
   byte *out; 
   /* working pointer into output buffer */	
@@ -444,8 +445,9 @@ imagen_print_page(gx_device_printer *pdev, FILE *prn_stream)
     /* ------------------------------------------------------- */
     /* get 32 lines and interleave into a row of swatches */ 
     for (swatchLine = 0 ; swatchLine <= lastLine; swatchLine++) { 
+
       /* blank out end of buffer for BIGSIZE overlap */
-      for (temp = in + line_size; temp < in + line_size + BIGSIZE;temp++){
+      for (temp = in + line_size; temp < in + align_size*BIGSIZE; temp++){
 	*temp = 0;
       } /* for temp */
 
@@ -553,10 +555,9 @@ imagen_print_page(gx_device_printer *pdev, FILE *prn_stream)
  
   fflush(prn_stream); 
  
-  gs_free((char *)swatchMap, BIGSIZE, swatchCount / BIGSIZE + 1,
-	"imagen_print_page(swatchMap)" );
   gs_free((char *)out, TotalBytesPerSw, swatchCount+1, "imagen_print_page(out)"); 
-  gs_free((char *)in, BIGSIZE, line_size / BIGSIZE + 1, "imagen_print_page(in)"); 
+  gs_free((char *)swatchMap, BIGSIZE, swatchCount / BIGSIZE + 1, "imagen_print_page(swatchMap)" );
+  gs_free((char *)in, BIGSIZE, align_size, "imagen_print_page(in)"); 
   /* ----------------------------------------- */
 
   DebugMsg(1,"Debug: Grey: %d \n",totalGreySwatches);
