@@ -1,21 +1,26 @@
-/* Copyright (C) 1994-1997 artofcode LLC. All rights reserved.
+/* Copyright (C) 1994-2003 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
+  This file is part of Aladdin Ghostscript.
   
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
+  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
+  or distributor accepts any responsibility for the consequences of using it,
+  or for whether it serves any particular purpose or works at all, unless he
+  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
+  License (the "License") for full details.
   
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  Every copy of Aladdin Ghostscript must include a copy of the License,
+  normally in a plain ASCII text file named PUBLIC.  The License grants you
+  the right to copy, modify and redistribute Aladdin Ghostscript, but only
+  under certain conditions described in the License.  Among other things, the
+  License requires that the copyright notice and this notice be preserved on
+  all copies.
 */
 
-/* $Id: gdevmac.c,v 1.5 2003/07/13 04:43:17 easysw Exp $ */
-
+/* $Id: gdevmac.c,v 1.8 2003/04/08 12:17:17 giles Exp $ */
+/* MacOS bitmap output device. This code is superceeded by
+   the newer gsapi_* interface and the DISPLAY device. Please
+   use that instead. See doc/API.htm for more information */
+   
 #include "gdevmac.h"
 #include "gsparam.h"
 #include "gsdll.h"
@@ -626,7 +631,9 @@ mac_put_params(gx_device *dev, gs_param_list *plist)
 		return code;
 	} else if (code == 0) {
 		
-		if (dev->LockSafetyParams) {
+		if (dev->LockSafetyParams &&
+			bytes_compare(outputFile.data, outputFile.size,
+			    (const byte *)mdev->outputFileName, strlen(mdev->outputFileName))) {
 			param_signal_error(plist, "OutputFile", gs_error_invalidaccess);
 			return gs_error_invalidaccess;
 		}
