@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevpdfr.c,v 1.3 2002/04/23 11:58:42 easysw Exp $ */
+/* $Id: gdevpdfr.c,v 1.8.2.1 2005/04/25 12:38:44 igor Exp $ */
 /* Named object pdfmark processing */
 #include "memory_.h"
 #include "string_.h"
@@ -51,7 +51,11 @@ pdf_find_named(gx_device_pdf * pdev, const gs_param_string * pname,
 
     if (!pdf_objname_is_valid(pname->data, pname->size))
 	return_error(gs_error_rangecheck);
-    if ((pvalue = cos_dict_find(pdev->named_objects, pname->data, pname->size)) != 0) {
+    if ((pvalue = cos_dict_find(pdev->local_named_objects, pname->data,
+				pname->size)) != 0 ||
+	(pvalue = cos_dict_find(pdev->global_named_objects, pname->data,
+				pname->size)) != 0
+	) {
 	*ppco = pvalue->contents.object;
 	return 0;
     }
@@ -59,8 +63,9 @@ pdf_find_named(gx_device_pdf * pdev, const gs_param_string * pname,
 }
 
 /*
- * Create a named object.  id = -1L means do not assign an id.  pname = 0
- * means just create the object, do not name it.
+ * Create a (local) named object.  id = -1L means do not assign an id.
+ * pname = 0 means just create the object, do not name it.  Note that
+ * during initialization, local_named_objects == global_named_objects.
  */
 int
 pdf_create_named(gx_device_pdf *pdev, const gs_param_string *pname,
@@ -75,7 +80,7 @@ pdf_create_named(gx_device_pdf *pdev, const gs_param_string *pname,
     pco->id =
 	(id == -1 ? 0L : id == 0 ? pdf_obj_ref(pdev) : id);
     if (pname) {
-	int code = cos_dict_put(pdev->named_objects, pname->data,
+	int code = cos_dict_put(pdev->local_named_objects, pname->data,
 				pname->size, cos_object_value(&value, pco));
 
 	if (code < 0)
@@ -206,6 +211,54 @@ pdf_get_named(gx_device_pdf * pdev, const gs_param_string * pname,
     return code;
 }
 
+/*
+ * Push the current local namespace onto the namespace stack, and reset it
+ * to an empty namespace.
+ */
+int
+pdf_push_namespace(gx_device_pdf *pdev)
+{
+    int code = cos_array_add_object(pdev->Namespace_stack,
+				    COS_OBJECT(pdev->local_named_objects));
+    cos_dict_t *pcd =
+	cos_dict_alloc(pdev, "pdf_push_namespace(local_named_objects)");
+    cos_array_t *pca =
+	cos_array_alloc(pdev, "pdf_push_namespace(NI_stack)");
+
+    if (code < 0 ||
+	(code = cos_array_add_object(pdev->Namespace_stack,
+				     COS_OBJECT(pdev->NI_stack))) < 0
+	)
+	return code;
+    if (pcd == 0 || pca == 0)
+	return_error(gs_error_VMerror);
+    pdev->local_named_objects = pcd;
+    pdev->NI_stack = pca;
+    return 0;
+}
+
+/*
+ * Pop the top local namespace from the namespace stack.  Return an error if
+ * the stack is empty.
+ */
+int
+pdf_pop_namespace(gx_device_pdf *pdev)
+{
+    cos_value_t nis_value, lno_value;
+    int code = cos_array_unadd(pdev->Namespace_stack, &nis_value);
+    
+    if (code < 0 ||
+	(code = cos_array_unadd(pdev->Namespace_stack, &lno_value)) < 0
+	)
+	return code;
+    COS_FREE(pdev->local_named_objects,
+	     "pdf_pop_namespace(local_named_objects)");
+    pdev->local_named_objects = (cos_dict_t *)lno_value.contents.object;
+    COS_FREE(pdev->NI_stack, "pdf_pop_namespace(NI_stack)");
+    pdev->NI_stack = (cos_array_t *)nis_value.contents.object;
+    return 0;
+}
+
 /*
  * Scan a token from a string.  <<, >>, [, and ] are treated as tokens.
  * Return 1 if a token was scanned, 0 if we reached the end of the string,
@@ -213,17 +266,32 @@ pdf_get_named(gx_device_pdf * pdev, const gs_param_string * pname,
  * to but not including *pscan.
  *
  * Note that this scanner expects a subset of PostScript syntax, not PDF
- * syntax.  In particular, it doesn't understand ASCII85 strings, and it
- * doesn't process the PDF #-escape syntax within names.  Note also that
- * it does only minimal syntax checking.
+ * syntax.  In particular, it doesn't understand ASCII85 strings,
+ * doesn't process the PDF #-escape syntax within names, and does only
+ * minimal syntax checking.  It also recognizes one extension to PostScript
+ * syntax, to allow gs_pdfwr.ps to pass names that include non-regular
+ * characters: If a name is immediately preceded by two null characters,
+ * the name includes everything up to a following null character.  The only
+ * place that currently generates this convention is the PostScript code
+ * that pre-processes the arguments for pdfmarks, in lib/gs_pdfwr.ps.
  */
 int
 pdf_scan_token(const byte **pscan, const byte * end, const byte **ptoken)
 {
     const byte *p = *pscan;
 
-    while (p < end && scan_char_decoder[*p] == ctype_space)
+    while (p < end && scan_char_decoder[*p] == ctype_space) {
 	++p;
+	if (p[-1] == 0 && p + 1 < end && *p == 0 && p[1] == '/') {
+	/* Special handling for names delimited by a null character. */
+	    *ptoken = ++p;
+	    while (*p != 0)
+		if (++p >= end)
+		    return_error(gs_error_syntaxerror);	/* no terminator */
+	    *pscan = p;
+	    return 1;
+	}
+    }
     *ptoken = p;
     if (p >= end) {
 	*pscan = p;
@@ -241,7 +309,8 @@ pdf_scan_token(const byte **pscan, const byte * end, const byte **ptoken)
 	stream_PSSD_state ss;
 	int status;
 
-	s_PSSD_init_inline(&ss);
+	s_PSSD_partially_init_inline(&ss);
+	ss.from_string = true;
 	r.ptr = p;		/* skip the '(' */
 	r.limit = end - 1;
 	w.limit = buf + sizeof(buf) - 1;
