@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevpdfu.c,v 1.3 2002/04/23 11:58:42 easysw Exp $ */
+/* $Id: gdevpdfu.c,v 1.40.2.11 2005/01/17 15:04:26 igor Exp $ */
 /* Output utilities for PDF-writing driver */
 #include "memory_.h"
 #include "jpeglib_.h"		/* for sdct.h */
@@ -24,8 +24,10 @@
 #include "gscdefs.h"
 #include "gsdsrc.h"
 #include "gsfunc.h"
+#include "gsfunc3.h"
 #include "gdevpdfx.h"
 #include "gdevpdfo.h"
+#include "gdevpdfg.h"
 #include "scanchar.h"
 #include "strimpl.h"
 #include "sa85x.h"
@@ -34,6 +36,8 @@
 #include "slzwx.h"
 #include "spngpx.h"
 #include "srlx.h"
+#include "sarc4.h"
+#include "smd5.h"
 #include "sstring.h"
 #include "szlibx.h"
 
@@ -47,7 +51,6 @@
 #  define compression_filter_template s_zlibE_template
 #  define compression_filter_state stream_zlib_state
 #else
-#  include "slzwx.h"
 #  define compression_filter_name "LZWDecode"
 #  define compression_filter_template s_LZWE_template
 #  define compression_filter_state stream_LZW_state
@@ -61,11 +64,13 @@ extern stream_state_proc_get_params(s_CF_get_params, stream_CF_state);
   BEGIN if ((code = (expr)) < 0) return code; END
 
 /* GC descriptors */
-extern_st(st_pdf_font);
+extern_st(st_pdf_color_space);
+extern_st(st_pdf_font_resource);
 extern_st(st_pdf_char_proc);
 extern_st(st_pdf_font_descriptor);
 public_st_pdf_resource();
 private_st_pdf_x_object();
+private_st_pdf_pattern();
 
 /* ---------------- Utilities ---------------- */
 
@@ -89,17 +94,14 @@ pdf_open_document(gx_device_pdf * pdev)
      * It also isn't clear whether the compression method can now be
      * changed in the course of the document.
      *
-     * The following algorithm is per an update to TN # 5151 by
-     * Adobe Developer Support.
+     * Flate compression is available starting in PDF 1.2.  Since we no
+     * longer support any older PDF versions, we ignore UseFlateCompression
+     * and always use Flate compression.
      */
     if (!pdev->params.CompressPages)
 	pdev->compression = pdf_compress_none;
-    else if (pdev->CompatibilityLevel < 1.2)
-	pdev->compression = pdf_compress_LZW;
-    else if (pdev->params.UseFlateCompression)
-	pdev->compression = pdf_compress_Flate;
     else
-	pdev->compression = pdf_compress_LZW;
+	pdev->compression = pdf_compress_Flate;
 }
 
 /* ------ Objects ------ */
@@ -179,10 +181,10 @@ pdf_end_obj(gx_device_pdf * pdev)
 
 /* Handle transitions between contexts. */
 private int
-    none_to_stream(P1(gx_device_pdf *)), stream_to_text(P1(gx_device_pdf *)),
-    string_to_text(P1(gx_device_pdf *)), text_to_stream(P1(gx_device_pdf *)),
-    stream_to_none(P1(gx_device_pdf *));
-typedef int (*context_proc) (P1(gx_device_pdf *));
+    none_to_stream(gx_device_pdf *), stream_to_text(gx_device_pdf *),
+    string_to_text(gx_device_pdf *), text_to_stream(gx_device_pdf *),
+    stream_to_none(gx_device_pdf *);
+typedef int (*context_proc) (gx_device_pdf *);
 private const context_proc context_procs[4][4] =
 {
     {0, none_to_stream, none_to_stream, none_to_stream},
@@ -191,14 +193,90 @@ private const context_proc context_procs[4][4] =
     {string_to_text, string_to_text, string_to_text, 0}
 };
 
+/* Compute an object encryption key. */
+private int
+pdf_object_key(const gx_device_pdf * pdev, gs_id object_id, byte key[16])
+{
+    md5_state_t md5;
+    md5_byte_t zero[2] = {0, 0}, t;
+    int KeySize = pdev->KeyLength / 8;
+
+    md5_init(&md5);
+    md5_append(&md5, pdev->EncryptionKey, KeySize);
+    t = (byte)(object_id >>  0);  md5_append(&md5, &t, 1);
+    t = (byte)(object_id >>  8);  md5_append(&md5, &t, 1);
+    t = (byte)(object_id >> 16);  md5_append(&md5, &t, 1);
+    md5_append(&md5, zero, 2);
+    md5_finish(&md5, key);
+    return min(KeySize + 5, 16);
+}
+
+/* Initialize encryption. */
+int
+pdf_encrypt_init(const gx_device_pdf * pdev, gs_id object_id, stream_arcfour_state *psarc4)
+{
+    byte key[16];
+
+    return s_arcfour_set_key(psarc4, key, pdf_object_key(pdev, object_id, key));
+}
+
+
+/* Add the encryption filter. */
+int
+pdf_begin_encrypt(gx_device_pdf * pdev, stream **s, gs_id object_id)
+{
+    gs_memory_t *mem = pdev->v_memory;
+    stream_arcfour_state *ss;
+    md5_byte_t key[16];
+    int code, keylength;
+
+    if (!pdev->KeyLength)
+	return 0;
+    keylength = pdf_object_key(pdev, object_id, key);
+    ss = gs_alloc_struct(mem, stream_arcfour_state, 
+		    s_arcfour_template.stype, "psdf_encrypt");
+    if (ss == NULL)
+	return_error(gs_error_VMerror);
+    code = s_arcfour_set_key(ss, key, keylength);
+    if (code < 0)
+	return code;
+    if (s_add_filter(s, &s_arcfour_template, (stream_state *)ss, mem) == 0)
+	return_error(gs_error_VMerror);
+    return 0;
+    /* IMPORTANT NOTE :
+       We don't encrypt streams written into temporary files,
+       because they can be used for comparizon
+       (for example, for merging equal images).
+       Instead that the encryption is applied in pdf_copy_data,
+       when the stream is copied to the output file.
+     */
+}
+
+/* Remove the encryption filter. */
+void
+pdf_end_encrypt(gx_device_pdf * pdev)
+{
+    if (pdev->KeyLength) {
+	stream *s = pdev->strm;
+	stream *fs = s->strm;
+
+	sclose(s);
+	gs_free_object(pdev->pdf_memory, s->cbuf, "encrypt buffer");
+	gs_free_object(pdev->pdf_memory, s, "encrypt stream");
+	pdev->strm = fs;
+    }
+}
+
 /* Enter stream context. */
 private int
 none_to_stream(gx_device_pdf * pdev)
 {
     stream *s;
+    int code;
 
     if (pdev->contents_id != 0)
 	return_error(gs_error_Fatal);	/* only 1 contents per page */
+    pdev->compression_at_page_start = pdev->compression;
     pdev->contents_id = pdf_begin_obj(pdev);
     pdev->contents_length_id = pdf_obj_ref(pdev);
     s = pdev->strm;
@@ -207,6 +285,10 @@ none_to_stream(gx_device_pdf * pdev)
 	pprints1(s, "/Filter /%s", compression_filter_name);
     stream_puts(s, ">>\nstream\n");
     pdev->contents_pos = pdf_stell(pdev);
+    code = pdf_begin_encrypt(pdev, &s, pdev->contents_id);
+    if (code < 0)
+	return code;
+    pdev->strm = s;
     if (pdev->compression == pdf_compress_Flate) {	/* Set up the Flate filter. */
 	const stream_template *template = &compression_filter_template;
 	stream *es = s_alloc(pdev->pdf_memory, "PDF compression stream");
@@ -245,14 +327,16 @@ none_to_stream(gx_device_pdf * pdev)
 		     ri_names[(int)pdev->params.DefaultRenderingIntent]);
 	}
     }
-    /* Do a level of gsave for the clipping path. */
-    stream_puts(s, "q\n");
+    pdev->vgstack_depth = 0;
+    pdev->AR4_save_bug = false;
     return PDF_IN_STREAM;
 }
 /* Enter text context from stream context. */
 private int
 stream_to_text(gx_device_pdf * pdev)
 {
+    int code;
+
     /*
      * Bizarrely enough, Acrobat Reader cares how the final font size is
      * obtained -- the CTM (cm), text matrix (Tm), and font size (Tf)
@@ -261,29 +345,33 @@ stream_to_text(gx_device_pdf * pdev)
      * anti-alias characters.  Therefore, we have to temporarily patch
      * the CTM so that the scale factors are unity.  What a nuisance!
      */
-    pprintg2(pdev->strm, "q %g 0 0 %g 0 0 cm BT\n",
+    code = pdf_save_viewer_state(pdev, pdev->strm);
+    if (code < 0)
+	return 0;
+    pprintg2(pdev->strm, "%g 0 0 %g 0 0 cm BT\n",
 	     pdev->HWResolution[0] / 72.0, pdev->HWResolution[1] / 72.0);
     pdev->procsets |= Text;
-    gs_make_identity(&pdev->text.matrix);
-    pdev->text.line_start.x = pdev->text.line_start.y = 0;
-    pdev->text.buffer_count = 0;
-    return PDF_IN_TEXT;
+    code = pdf_from_stream_to_text(pdev);
+    return (code < 0 ? code : PDF_IN_TEXT);
 }
 /* Exit string context to text context. */
 private int
 string_to_text(gx_device_pdf * pdev)
 {
-    pdf_put_string(pdev, pdev->text.buffer, pdev->text.buffer_count);
-    stream_puts(pdev->strm, (pdev->text.use_leading ? "'\n" : "Tj\n"));
-    pdev->text.use_leading = false;
-    pdev->text.buffer_count = 0;
-    return PDF_IN_TEXT;
+    int code = pdf_from_string_to_text(pdev);
+
+    return (code < 0 ? code : PDF_IN_TEXT);
 }
 /* Exit text context to stream context. */
 private int
 text_to_stream(gx_device_pdf * pdev)
 {
-    stream_puts(pdev->strm, "ET Q\n");
+    int code;
+
+    stream_puts(pdev->strm, "ET\n");
+    code = pdf_restore_viewer_state(pdev, pdev->strm);
+    if (code < 0)
+	return 0;
     pdf_reset_text(pdev);	/* because of Q */
     return PDF_IN_STREAM;
 }
@@ -294,9 +382,9 @@ stream_to_none(gx_device_pdf * pdev)
     stream *s = pdev->strm;
     long length;
 
-    /* Close the extra q/Q for poorly designed PDF tools. */
-    stream_puts(s, "Q\n");
-    if (pdev->compression == pdf_compress_Flate) {	/* Terminate the Flate filter. */
+    if (pdev->vgstack_depth)
+	pdf_restore_viewer_state(pdev, s);
+    if (pdev->compression_at_page_start == pdf_compress_Flate) {	/* Terminate the Flate filter. */
 	stream *fs = s->strm;
 
 	sclose(s);
@@ -304,6 +392,8 @@ stream_to_none(gx_device_pdf * pdev)
 	gs_free_object(pdev->pdf_memory, s, "zlib stream");
 	pdev->strm = s = fs;
     }
+    pdf_end_encrypt(pdev);
+    s = pdev->strm;
     length = pdf_stell(pdev) - pdev->contents_pos;
     stream_puts(s, "endstream\n");
     pdf_end_obj(pdev);
@@ -317,7 +407,7 @@ stream_to_none(gx_device_pdf * pdev)
 int
 pdf_open_contents(gx_device_pdf * pdev, pdf_context_t context)
 {
-    int (*proc) (P1(gx_device_pdf *));
+    int (*proc) (gx_device_pdf *);
 
     while ((proc = context_procs[pdev->context][context]) != 0) {
 	int code = (*proc) (pdev);
@@ -338,8 +428,8 @@ pdf_close_contents(gx_device_pdf * pdev, bool last)
 	return 0;
     if (last) {			/* Exit from the clipping path gsave. */
 	pdf_open_contents(pdev, PDF_IN_STREAM);
-	stream_puts(pdev->strm, "Q\n");
-	pdev->text.font = 0;
+	stream_puts(pdev->strm, "Q\n");	/* See none_to_stream. */
+	pdf_close_text_contents(pdev);
     }
     return pdf_open_contents(pdev, PDF_IN_NONE);
 }
@@ -354,6 +444,51 @@ const gs_memory_struct_type_t *const pdf_resource_type_structs[] = {
     PDF_RESOURCE_TYPE_STRUCTS
 };
 
+/* Cancel a resource (do not write it into PDF). */
+int
+pdf_cancel_resource(gx_device_pdf * pdev, pdf_resource_t *pres, pdf_resource_type_t rtype)
+{
+    /* fixme : remove *pres from resource chain. */
+    pres->where_used = 0;
+    pres->object->written = true;
+    if (rtype == resourceXObject || rtype == resourceCharProc) {
+	int code = cos_stream_release_pieces((cos_stream_t *)pres->object);
+
+	if (code < 0)
+	    return code;
+    }
+    cos_release(pres->object, "pdf_cancel_resource");
+    return 0;
+}
+
+/* Remove a resource. */
+void
+pdf_forget_resource(gx_device_pdf * pdev, pdf_resource_t *pres1, pdf_resource_type_t rtype)
+{   /* fixme : optimize. */
+    pdf_resource_t **pchain = pdev->resources[rtype].chains;
+    pdf_resource_t *pres;
+    pdf_resource_t **pprev = &pdev->last_resource;
+    int i;
+
+    for (; (pres = *pprev) != 0; pprev = &pres->prev)
+	if (pres == pres1) {
+	    *pprev = pres->prev;
+	    break;
+	}
+    for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
+	pprev = pchain + i;
+	for (; (pres = *pprev) != 0; pprev = &pres->next)
+	    if (pres == pres1) {
+		*pprev = pres->next;
+		COS_RELEASE(pres->object, "pdf_forget_resource");
+		gs_free_object(pdev->pdf_memory, pres->object, "pdf_forget_resource");
+		gs_free_object(pdev->pdf_memory, pres, "pdf_forget_resource");
+		break;
+	    }
+    }
+}
+
+
 /* Find a resource of a given type by gs_id. */
 pdf_resource_t *
 pdf_find_resource_by_gs_id(gx_device_pdf * pdev, pdf_resource_type_t rtype,
@@ -375,6 +510,110 @@ pdf_find_resource_by_gs_id(gx_device_pdf * pdev, pdf_resource_type_t rtype,
     return 0;
 }
 
+/* Find resource by resource id. */
+pdf_resource_t *
+pdf_find_resource_by_resource_id(gx_device_pdf * pdev, pdf_resource_type_t rtype, gs_id id)
+{
+    pdf_resource_t **pchain = pdev->resources[rtype].chains;
+    pdf_resource_t *pres;
+    int i;
+    
+    for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
+	for (pres = pchain[i]; pres != 0; pres = pres->next) {
+	    if (pres->object->id == id)
+		return pres;
+	}
+    }
+    return 0;
+}
+
+
+/* Find same resource. */
+int
+pdf_find_same_resource(gx_device_pdf * pdev, pdf_resource_type_t rtype, pdf_resource_t **ppres,
+	int (*eq)(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1))
+{
+    pdf_resource_t **pchain = pdev->resources[rtype].chains;
+    pdf_resource_t *pres;
+    cos_object_t *pco0 = (*ppres)->object;
+    int i;
+    
+    for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
+	for (pres = pchain[i]; pres != 0; pres = pres->next) {
+	    if (!pres->named && *ppres != pres) {
+		cos_object_t *pco1 = pres->object;
+		int code = pco0->cos_procs->equal(pco0, pco1, pdev);
+
+		if (code < 0)
+		    return code;
+		if (code > 0) {
+ 		    code = eq(pdev, *ppres, pres);
+ 		    if (code < 0)
+ 			return code;
+ 		    if (code > 0) {
+		        *ppres = pres;
+		        return 1;
+		    }
+		}
+	    }
+	}
+    }
+    return 0;
+}
+
+/* Drop resources by a condition. */
+void
+pdf_drop_resources(gx_device_pdf * pdev, pdf_resource_type_t rtype, 
+	int (*cond)(gx_device_pdf * pdev, pdf_resource_t *pres))
+{
+    pdf_resource_t **pchain = pdev->resources[rtype].chains;
+    pdf_resource_t **pprev;
+    pdf_resource_t *pres;
+    int i;
+
+    for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
+	pprev = pchain + i;
+	for (; (pres = *pprev) != 0; ) {
+	    if (cond(pdev, pres)) {
+		*pprev = pres->next;
+		pres->next = pres; /* A temporary mark - see below */
+	    } else
+		pprev = &pres->next;
+	}
+    }
+    pprev = &pdev->last_resource;
+    for (; (pres = *pprev) != 0; )
+	if (pres->next == pres) {
+	    *pprev = pres->prev;
+	    COS_RELEASE(pres->object, "pdf_drop_resources");
+	    gs_free_object(pdev->pdf_memory, pres->object, "pdf_drop_resources");
+	    gs_free_object(pdev->pdf_memory, pres, "pdf_drop_resources");
+	} else
+	    pprev = &pres->prev;
+}
+
+/* Print resource statistics. */
+void
+pdf_print_resource_statistics(gx_device_pdf * pdev)
+{
+
+    int rtype;
+
+    for (rtype = 0; rtype < NUM_RESOURCE_TYPES; rtype++) {
+	pdf_resource_t **pchain = pdev->resources[rtype].chains;
+	pdf_resource_t *pres;
+	const char *name = pdf_resource_type_names[rtype];
+	int i, n = 0;
+    
+	for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
+	    for (pres = pchain[i]; pres != 0; pres = pres->next, n++);
+	}
+	dprintf3("Resource type %d (%s) has %d instances.\n", rtype, 
+		(name ? name : ""), n);
+    }
+}
+
+
 /* Begin an object logically separate from the contents. */
 long
 pdf_open_separate(gx_device_pdf * pdev, long id)
@@ -390,8 +629,15 @@ pdf_begin_separate(gx_device_pdf * pdev)
     return pdf_open_separate(pdev, 0L);
 }
 
+void
+pdf_reserve_object_id(gx_device_pdf * pdev, pdf_resource_t *pres, long id)
+{
+    pres->object->id = (id == 0 ? pdf_obj_ref(pdev) : id);
+    sprintf(pres->rname, "R%ld", pres->object->id);
+}
+
 /* Begin an aside (resource, annotation, ...). */
-private int
+int
 pdf_alloc_aside(gx_device_pdf * pdev, pdf_resource_t ** plist,
 		const gs_memory_struct_type_t * pst, pdf_resource_t **ppres,
 		long id)
@@ -403,24 +649,24 @@ pdf_alloc_aside(gx_device_pdf * pdev, pdf_resource_t ** plist,
 	pst = &st_pdf_resource;
     pres = gs_alloc_struct(pdev->pdf_memory, pdf_resource_t, pst,
 			   "pdf_alloc_aside(resource)");
-    object = cos_object_alloc(pdev, "pdf_alloc_aside(object)");
-    if (pres == 0 || object == 0) {
+    if (pres == 0)
 	return_error(gs_error_VMerror);
-    }
+    object = cos_object_alloc(pdev, "pdf_alloc_aside(object)");
+    if (object == 0)
+	return_error(gs_error_VMerror);
+    memset(pres + 1, 0, pst->ssize - sizeof(*pres));
+    pres->object = object;
     if (id < 0) {
 	object->id = -1L;
 	pres->rname[0] = 0;
-    } else {
-	object->id = (id == 0 ? pdf_obj_ref(pdev) : id);
-	sprintf(pres->rname, "R%ld", object->id);
-    }
+    } else
+	pdf_reserve_object_id(pdev, pres, id);
     pres->next = *plist;
     *plist = pres;
     pres->prev = pdev->last_resource;
     pdev->last_resource = pres;
     pres->named = false;
     pres->where_used = pdev->used_mask;
-    pres->object = object;
     *ppres = pres;
     return 0;
 }
@@ -506,16 +752,15 @@ pdf_end_resource(gx_device_pdf * pdev)
 }
 
 /*
- * Write and release the Cos objects for resources local to a content stream.
- * We must write all the objects before freeing any of them, because
- * they might refer to each other.
+ * Write the Cos objects for resources local to a content stream.  Formerly,
+ * this procedure also freed such objects, but this doesn't work, because
+ * resources of one type might refer to resources of another type.
  */
 int
 pdf_write_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
 {
     int j;
 
-    /* Write objects. */
     for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
 	pdf_resource_t *pres = pdev->resources[rtype].chains[j];
 
@@ -523,8 +768,18 @@ pdf_write_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
 	    if (!pres->named && !pres->object->written)
 		cos_write_object(pres->object, pdev);
     }
+    return 0;
+}
+
+/*
+ * Free unnamed Cos objects for resources local to a content stream,
+ * since they can't be used again.
+ */
+int
+pdf_free_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
+{
+    int j;
 
-    /* Free unnamed objects, which can't be used again. */
     for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
 	pdf_resource_t **prev = &pdev->resources[rtype].chains[j];
 	pdf_resource_t *pres;
@@ -533,13 +788,12 @@ pdf_write_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
 	    if (pres->named) {	/* named, don't free */
 		prev = &pres->next;
 	    } else {
-		cos_free(pres->object, "pdf_write_resource_objects");
+		cos_free(pres->object, "pdf_free_resource_objects");
 		pres->object = 0;
 		*prev = pres->next;
 	    }
 	}
     }
-
     return 0;
 }
 
@@ -550,51 +804,47 @@ pdf_write_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
 int
 pdf_store_page_resources(gx_device_pdf *pdev, pdf_page_t *page)
 {
+    int i;
 
-    /* Write out any resource dictionaries. */
+    /* Write any resource dictionaries. */
 
-    {
-	int i;
+    for (i = 0; i <= resourceFont; ++i) {
+	stream *s = 0;
+	int j;
 
-	for (i = 0; i <= resourceFont; ++i) {
-	    stream *s = 0;
-	    int j;
+	page->resource_ids[i] = 0;
+	for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
+	    pdf_resource_t *pres = pdev->resources[i].chains[j];
 
-	    page->resource_ids[i] = 0;
-	    for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
-		pdf_resource_t *pres = pdev->resources[i].chains[j];
+	    for (; pres != 0; pres = pres->next) {
+		if (pres->where_used & pdev->used_mask) {
+		    long id = pres->object->id;
 
-		for (; pres != 0; pres = pres->next) {
-		    if (pres->where_used & pdev->used_mask) {
-			long id = pres->object->id;
-
-			if (s == 0) {
-			    page->resource_ids[i] = pdf_begin_separate(pdev);
-			    s = pdev->strm;
-			    stream_puts(s, "<<");
-			}
-			pprints1(s, "/%s\n", pres->rname);
-			pprintld1(s, "%ld 0 R", id);
-			pres->where_used -= pdev->used_mask;
+		    if (s == 0) {
+			page->resource_ids[i] = pdf_begin_separate(pdev);
+			s = pdev->strm;
+			stream_puts(s, "<<");
 		    }
+		    pprints1(s, "/%s\n", pres->rname);
+		    pprintld1(s, "%ld 0 R", id);
+		    pres->where_used -= pdev->used_mask;
 		}
 	    }
-	    if (s) {
-		stream_puts(s, ">>\n");
-		pdf_end_separate(pdev);
-		if (i != resourceFont)
-		    pdf_write_resource_objects(pdev, i);
-	    }
+	}
+	if (s) {
+	    stream_puts(s, ">>\n");
+	    pdf_end_separate(pdev);
+	    if (i != resourceFont)
+		pdf_write_resource_objects(pdev, i);
 	}
     }
-
     page->procsets = pdev->procsets;
     return 0;
 }
 
 /* Copy data from a temporary file to a stream. */
 void
-pdf_copy_data(stream *s, FILE *file, long count)
+pdf_copy_data(stream *s, FILE *file, long count, stream_arcfour_state *ss)
 {
     long left = count;
     byte buf[sbuf_size];
@@ -602,12 +852,36 @@ pdf_copy_data(stream *s, FILE *file, long count)
     while (left > 0) {
 	uint copy = min(left, sbuf_size);
 
-	fread(buf, 1, sbuf_size, file);
+	fread(buf, 1, copy, file);
+	if (ss)
+	    s_arcfour_process_buffer(ss, buf, copy);
 	stream_write(s, buf, copy);
 	left -= copy;
     }
 }
 
+
+/* Copy data from a temporary file to a stream, 
+   which may be targetted to the same file. */
+void
+pdf_copy_data_safe(stream *s, FILE *file, long position, long count)
+{   
+    long left = count;
+
+    while (left > 0) {
+	byte buf[sbuf_size];
+	long copy = min(left, (long)sbuf_size);
+	long end_pos = ftell(file);
+
+	fseek(file, position + count - left, SEEK_SET);
+	fread(buf, 1, copy, file);
+	fseek(file, end_pos, SEEK_SET);
+	stream_write(s, buf, copy);
+	sflush(s);
+	left -= copy;
+    }
+}
+
 /* ------ Pages ------ */
 
 /* Get or assign the ID for a page. */
@@ -683,6 +957,31 @@ pdf_open_page(gx_device_pdf * pdev, pdf_context_t context)
     return pdf_open_contents(pdev, context);
 }
 
+
+/*  Go to the unclipped stream context. */
+int
+pdf_unclip(gx_device_pdf * pdev)
+{
+    if (pdev->sbstack_depth == 0) {
+	int code = pdf_open_page(pdev, PDF_IN_STREAM);
+
+    	if (code < 0)
+	    return code;
+    }
+    if (pdev->vgstack_depth > pdev->vgstack_bottom) {
+	int code = pdf_restore_viewer_state(pdev, pdev->strm);
+
+	if (code < 0)
+	    return code;
+	code = pdf_remember_clip_path(pdev, NULL);
+	if (code < 0)
+	    return code;
+	pdev->clip_path_id = pdev->no_clip_path_id;
+    }
+    return 0;
+}
+
+
 /* ------ Miscellaneous output ------ */
 
 /* Generate the default Producer string. */
@@ -709,38 +1008,12 @@ pdf_put_matrix(gx_device_pdf * pdev, const char *before,
 }
 
 /*
- * Write a name, with escapes for unusual characters.  In PDF 1.1, we have
- * no choice but to replace these characters with '?'; in PDF 1.2, we can
- * use an escape sequence for anything except a null <00>.
+ * Write a name, with escapes for unusual characters.  Since we only support
+ * PDF 1.2 and above, we can use an escape sequence for anything except a
+ * null <00>, and the machinery for selecting the put_name_chars procedure
+ * depending on CompatibilityLevel is no longer needed.
  */
 private int
-pdf_put_name_chars_1_1(stream *s, const byte *nstr, uint size)
-{
-    uint i;
-
-    for (i = 0; i < size; ++i) {
-	uint c = nstr[i];
-
-	switch (c) {
-	    default:
-		if (c >= 0x21 && c <= 0x7e) {
-		    stream_putc(s, c);
-		    break;
-		}
-		/* falls through */
-	    case '%':
-	    case '(': case ')':
-	    case '<': case '>':
-	    case '[': case ']':
-	    case '{': case '}':
-	    case '/':
-	    case 0:
-		stream_putc(s, '?');
-	}
-    }
-    return 0;
-}
-private int
 pdf_put_name_chars_1_2(stream *s, const byte *nstr, uint size)
 {
     uint i;
@@ -752,7 +1025,7 @@ pdf_put_name_chars_1_2(stream *s, const byte *nstr, uint size)
 	switch (c) {
 	    default:
 		if (c >= 0x21 && c <= 0x7e) {
-		    stream_putc(s, c);
+		    stream_putc(s, (byte)c);
 		    break;
 		}
 		/* falls through */
@@ -775,8 +1048,7 @@ pdf_put_name_chars_1_2(stream *s, const byte *nstr, uint size)
 pdf_put_name_chars_proc_t
 pdf_put_name_chars_proc(const gx_device_pdf *pdev)
 {
-    return (pdev->CompatibilityLevel >= 1.2 ? pdf_put_name_chars_1_2 :
-	    pdf_put_name_chars_1_1);
+    return &pdf_put_name_chars_1_2;
 }
 void
 pdf_put_name_chars(const gx_device_pdf *pdev, const byte *nstr, uint size)
@@ -790,6 +1062,106 @@ pdf_put_name(const gx_device_pdf *pdev, const byte *nstr, uint size)
     pdf_put_name_chars(pdev, nstr, size);
 }
 
+/* Write an encoded string with encryption. */
+private int
+pdf_encrypt_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size, gs_id object_id)
+{
+    stream sinp, sstr, sout;
+    stream_PSSD_state st;
+    stream_state so;
+    byte buf[100], bufo[100];
+    stream_arcfour_state sarc4;
+    if (pdf_encrypt_init(pdev, object_id, &sarc4) < 0) {
+	/* The interface can't pass an error. */
+	stream_write(pdev->strm, str, size);
+	return size;
+    }
+    sread_string(&sinp, str + 1, size);
+    s_init(&sstr, NULL);
+    s_init_state((stream_state *)&st, &s_PSSD_template, NULL);
+    s_init_filter(&sstr, (stream_state *)&st, buf, sizeof(buf), &sinp);
+    s_init(&sout, NULL);
+    s_init_state(&so, &s_PSSE_template, NULL);
+    s_init_filter(&sout, &so, bufo, sizeof(bufo), pdev->strm);
+    stream_putc(pdev->strm, '(');
+    for (;;) {
+	uint n;
+	int code = sgets(&sstr, buf, sizeof(buf), &n);
+
+	if (n > 0) {
+	    s_arcfour_process_buffer(&sarc4, buf, n);
+	    stream_write(&sout, buf, n);
+	}
+	if (code == EOFC)
+	    break;
+	if (code < 0 || n < sizeof(buf)) {
+	    /* The interface can't pass an error. */
+	    break;
+	}
+    }
+    sclose(&sout); /* Writes ')'. */
+    return stell(&sinp) + 1;
+}
+
+/* Write an encoded string with possible encryption. */
+private void
+pdf_put_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size, gs_id object_id)
+{
+    if (!pdev->KeyLength || object_id == (gs_id)-1)
+	stream_write(pdev->strm, str, size);
+    else
+	DISCARD(pdf_encrypt_encoded_string(pdev, str, size, object_id));
+}
+
+/*  Scan an item in a serialized array or dictionary.
+    This is a very simplified Postscript lexical scanner.
+    It assumes the serialization with pdf===only defined in gs/lib/gs_pdfwr.ps .
+    We only need to select strings and encrypt them.
+    Other items are passed identically.
+    Note we don't reconstruct the nesting of arrays|dictionaries.
+*/
+private int
+pdf_scan_item(const gx_device_pdf * pdev, const byte * p, uint l, gs_id object_id)
+{
+    const byte *q = p;
+    int n = l; 
+
+    if (*q == ' ' || *q == 't' || *q == '\r' || *q == '\n')
+	return (l > 0 ? 1 : 0);
+    for (q++, n--; n; q++, n--) {
+	if (*q == ' ' || *q == 't' || *q == '\r' || *q == '\n')
+	    return q - p;
+	if (*q == '/' || *q == '[' || *q == ']' || *q == '{' || *q == '}' || *q == '(' || *q == '<')
+	    return q - p;
+	/* Note : immediate names are not allowed in PDF. */
+    }
+    return l;
+}
+
+/* Write a serialized array or dictionary with possible encryption. */
+private void
+pdf_put_composite(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_id object_id)
+{
+    if (!pdev->KeyLength || object_id == (gs_id)-1) {
+	stream_write(pdev->strm, vstr, size);
+	return;
+    } else {
+	const byte *p = vstr;
+	int l = size, n;
+
+	for (;l > 0 ;) {
+	    if (*p == '(')
+		n = pdf_encrypt_encoded_string(pdev, p, l, object_id);
+	    else {
+		n = pdf_scan_item(pdev, p, l, object_id);
+		stream_write(pdev->strm, p, n);
+	    }
+	    l -= n;
+	    p += n;
+	}
+    }
+}
+
 /*
  * Write a string in its shortest form ( () or <> ).  Note that
  * this form is different depending on whether binary data are allowed.
@@ -804,10 +1176,18 @@ pdf_put_string(const gx_device_pdf * pdev, const byte * str, uint size)
 
 /* Write a value, treating names specially. */
 void
-pdf_write_value(const gx_device_pdf * pdev, const byte * vstr, uint size)
+pdf_write_value(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_id object_id)
 {
     if (size > 0 && vstr[0] == '/')
 	pdf_put_name(pdev, vstr + 1, size - 1);
+    else if (size > 3 && vstr[0] == 0 && vstr[1] == 0 && vstr[size - 1] == 0)
+	pdf_put_name(pdev, vstr + 3, size - 4);
+    else if (size > 1 && vstr[0] == '(')
+	pdf_put_encoded_string(pdev, vstr, size, object_id);
+    else if (size > 1 && (vstr[0] == '[' || vstr[0] == '{'))
+	pdf_put_composite(pdev, vstr, size, object_id);
+    else if (size > 2 && vstr[0] == '<' && vstr[1] == '<')
+	pdf_put_composite(pdev, vstr, size, object_id);
     else
 	stream_write(pdev->strm, vstr, size);
 }
@@ -842,7 +1222,7 @@ pdf_put_filters(cos_dict_t *pcd, gx_device_pdf *pdev, stream *s,
 		return_error(gs_error_VMerror);
 	    CHECK(cos_param_list_writer_init(&writer, decode_parms, 0));
 	    /*
-	     * If EndOfBlock is true, we mustn't write out a Rows value.
+	     * If EndOfBlock is true, we mustn't write a Rows value.
 	     * This is a hack....
 	     */
 	    cfs = *(const stream_CF_state *)st;
@@ -932,11 +1312,17 @@ pdf_flate_binary(gx_device_pdf *pdev, psdf_binary_writer *pbw)
  * the << and any desired dictionary keys.
  */
 int
-pdf_begin_data_binary(gx_device_pdf *pdev, pdf_data_writer_t *pdw,
-		      bool data_is_binary)
+pdf_begin_data(gx_device_pdf *pdev, pdf_data_writer_t *pdw)
+{
+    return pdf_begin_data_stream(pdev, pdw,
+				 DATA_STREAM_BINARY | DATA_STREAM_COMPRESS, 0);
+}
+int
+pdf_begin_data_stream(gx_device_pdf *pdev, pdf_data_writer_t *pdw,
+		      int orig_options, gs_id object_id)
 {
-    long length_id = pdf_obj_ref(pdev);
     stream *s = pdev->strm;
+    int options = orig_options;
 #define USE_ASCII85 1
 #define USE_FLATE 2
     static const char *const fnames[4] = {
@@ -946,19 +1332,31 @@ pdf_begin_data_binary(gx_device_pdf *pdev, pdf_data_writer_t *pdw,
     int filters = 0;
     int code;
 
-    if (pdev->CompatibilityLevel >= 1.2) {
+    if (options & DATA_STREAM_COMPRESS) {
 	filters |= USE_FLATE;
-	data_is_binary = true;
+	options |= DATA_STREAM_BINARY;
     }
-    if (data_is_binary && !pdev->binary_ok)
+    if ((options & DATA_STREAM_BINARY) && !pdev->binary_ok)
 	filters |= USE_ASCII85;
-    stream_puts(s, fnames[filters]);
-    pprintld1(s, "/Length %ld 0 R>>stream\n", length_id);
+    if (!(options & DATA_STREAM_NOLENGTH)) {
+	long length_id = pdf_obj_ref(pdev);
+		
+	stream_puts(s, fnames[filters]);
+	pprintld1(s, "/Length %ld 0 R>>stream\n", length_id);
+	pdw->length_id = length_id;
+    }
+    if (options & DATA_STREAM_ENCRYPT) {
+	code = pdf_begin_encrypt(pdev, &s, object_id);
+	if (code < 0)
+	    return code;
+	pdev->strm = s;
+	pdw->encrypted = true;
+    } else
+    	pdw->encrypted = false;
     code = psdf_begin_binary((gx_device_psdf *)pdev, &pdw->binary);
     if (code < 0)
 	return code;
     pdw->start = stell(s);
-    pdw->length_id = length_id;
     if (filters & USE_FLATE)
 	code = pdf_flate_binary(pdev, &pdw->binary);
     return code;
@@ -976,6 +1374,8 @@ pdf_end_data(pdf_data_writer_t *pdw)
 
     if (code < 0)
 	return code;
+    if (pdw->encrypted)
+	pdf_end_encrypt(pdev);
     stream_puts(pdev->strm, "\nendstream\n");
     pdf_end_separate(pdev);
     pdf_open_separate(pdev, pdw->length_id);
@@ -984,6 +1384,46 @@ pdf_end_data(pdf_data_writer_t *pdw)
 }
 
 /* Create a Function object. */
+private int pdf_function_array(gx_device_pdf *pdev, cos_array_t *pca,
+			       const gs_function_info_t *pinfo);
+int
+pdf_function_scaled(gx_device_pdf *pdev, const gs_function_t *pfn,
+		    const gs_range_t *pranges, cos_value_t *pvalue)
+{
+    if (pranges == NULL)
+	return pdf_function(pdev, pfn, pvalue);
+    {
+	/*
+	 * Create a temporary scaled function.  Note that the ranges
+	 * represent the inverse scaling from what gs_function_make_scaled
+	 * expects.
+	 */
+	gs_memory_t *mem = pdev->pdf_memory;
+	gs_function_t *psfn;
+	gs_range_t *ranges = (gs_range_t *)
+	    gs_alloc_byte_array(mem, pfn->params.n, sizeof(gs_range_t),
+				"pdf_function_scaled");
+	int i, code;
+
+	if (ranges == 0)
+	    return_error(gs_error_VMerror);
+	for (i = 0; i < pfn->params.n; ++i) {
+	    double rbase = pranges[i].rmin;
+	    double rdiff = pranges[i].rmax - rbase;
+	    double invbase = -rbase / rdiff;
+
+	    ranges[i].rmin = invbase;
+	    ranges[i].rmax = invbase + 1.0 / rdiff;
+	}
+	code = gs_function_make_scaled(pfn, &psfn, ranges, mem);
+	if (code >= 0) {
+	    code = pdf_function(pdev, psfn, pvalue);
+	    gs_function_free(psfn, true, mem);
+	}
+	gs_free_object(mem, ranges, "pdf_function_scaled");
+	return code;
+    }
+}
 int
 pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
 	     cos_value_t *pvalue)
@@ -993,13 +1433,28 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
     pdf_resource_t *pres;
     cos_object_t *pcfn;
     cos_dict_t *pcd;
-    cos_value_t v;
     int code = pdf_alloc_resource(pdev, resourceFunction, gs_no_id, &pres, 0L);
 
     if (code < 0)
 	return code;
     pcfn = pres->object;
     gs_function_get_info(pfn, &info);
+    if (FunctionType(pfn) == function_type_ArrayedOutput) {
+	/*
+	 * Arrayed Output Functions are used internally to represent
+	 * Shading Function entries that are arrays of Functions.
+	 * They require special handling.
+	 */
+	cos_array_t *pca;
+
+	cos_become(pcfn, cos_type_array);
+	pca = (cos_array_t *)pcfn;
+	code = pdf_function_array(pdev, pca, &info);
+	if (code < 0)
+	    return code;
+	COS_OBJECT_VALUE(pvalue, pca);
+	return 0;
+    }
     if (info.DataSource != 0) {
 	psdf_binary_writer writer;
 	stream *save = pdev->strm;
@@ -1014,8 +1469,7 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
 	    return_error(gs_error_VMerror);
 	pdev->strm = s;
 	code = psdf_begin_binary((gx_device_psdf *)pdev, &writer);
-	if (code >= 0 && info.data_size > 30 &&	/* 30 is arbitrary */
-	    pdev->CompatibilityLevel >= 1.2
+	if (code >= 0 && info.data_size > 30	/* 30 is arbitrary */
 	    )
 	    code = pdf_flate_binary(pdev, &writer);
 	if (code >= 0) {
@@ -1048,23 +1502,16 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
 	pcd = (cos_dict_t *)pcfn;
     }
     if (info.Functions != 0) {
-	int i;
 	cos_array_t *functions =
 	    cos_array_alloc(pdev, "pdf_function(Functions)");
+	cos_value_t v;
 
 	if (functions == 0)
 	    return_error(gs_error_VMerror);
-	for (i = 0; i < info.num_Functions; ++i) {
-	    if ((code = pdf_function(pdev, info.Functions[i], &v)) < 0 ||
-		(code = cos_array_add(functions, &v)) < 0
-		) {
-		COS_FREE(functions, "pdf_function(Functions)");
-		return code;
-	    }
-	}
-	code = cos_dict_put_c_key(pcd, "/Functions",
-				  COS_OBJECT_VALUE(&v, functions));
-	if (code < 0) {
+	if ((code = pdf_function_array(pdev, functions, &info)) < 0 ||
+	    (code = cos_dict_put_c_key(pcd, "/Functions",
+				       COS_OBJECT_VALUE(&v, functions))) < 0
+	    ) {
 	    COS_FREE(functions, "pdf_function(Functions)");
 	    return code;
 	}
@@ -1078,6 +1525,22 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
     COS_OBJECT_VALUE(pvalue, pcd);
     return 0;
 }
+private int pdf_function_array(gx_device_pdf *pdev, cos_array_t *pca,
+			       const gs_function_info_t *pinfo)
+{
+    int i, code = 0;
+    cos_value_t v;
+
+    for (i = 0; i < pinfo->num_Functions; ++i) {
+	if ((code = pdf_function(pdev, pinfo->Functions[i], &v)) < 0 ||
+	    (code = cos_array_add(pca, &v)) < 0
+	    ) {
+	    break;
+	}
+    }
+    return code;
+}
+
 
 /* Write a Function object. */
 int
@@ -1091,3 +1554,22 @@ pdf_write_function(gx_device_pdf *pdev, const gs_function_t *pfn, long *pid)
     *pid = value.contents.object->id;
     return 0;
 }
+
+/* Write a FontBBox dictionary element. */
+int
+pdf_write_font_bbox(gx_device_pdf *pdev, const gs_int_rect *pbox)
+{
+    stream *s = pdev->strm;
+    /*
+     * AR 4 doesn't like fonts with empty FontBBox, which
+     * happens when the font contains only space characters.
+     * Small bbox causes AR 4 to display a hairline. So we use
+     * the full BBox.
+     */ 
+    int x = pbox->q.x + ((pbox->p.x == pbox->q.x) ? 1000 : 0);
+    int y = pbox->q.y + ((pbox->p.y == pbox->q.y) ? 1000 : 0);
+
+    pprintd4(s, "/FontBBox[%d %d %d %d]",
+	     pbox->p.x, pbox->p.y, x, y);
+    return 0;
+}
