@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevprn.c,v 1.5 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gdevprn.c,v 1.13.2.1 2003/12/12 22:08:01 giles Exp $ */
 /* Generic printer driver support */
 #include "ctype_.h"
 #include "gdevprn.h"
@@ -54,9 +54,9 @@ const gx_device_procs prn_std_procs =
     prn_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close);
 
 /* Forward references */
-int gdev_prn_maybe_realloc_memory(P4(gx_device_printer *pdev, 
-				     gdev_prn_space_params *old_space,
-				     int old_width, int old_height));
+int gdev_prn_maybe_realloc_memory(gx_device_printer *pdev, 
+				  gdev_prn_space_params *old_space,
+				  int old_width, int old_height);
 
 /* ------ Open/close ------ */
 
@@ -220,9 +220,10 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
     gx_device_memory * const pmemdev = (gx_device_memory *)pdev;
     byte *the_memory = 0;
     gdev_prn_space_params save_params;
-    int save_width, save_height;
-    bool is_command_list;
-    bool save_is_command_list;
+    int save_width = 0x0badf00d; /* Quiet compiler */
+    int save_height = 0x0badf00d; /* Quiet compiler */
+    bool is_command_list = false; /* Quiet compiler */
+    bool save_is_command_list = false; /* Quiet compiler */
     int ecode = 0;
     int pass;
     gs_memory_t *buffer_memory =
@@ -385,6 +386,10 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	COPY_PROC(get_clipping_box);
 	COPY_PROC(map_color_rgb_alpha);
 	COPY_PROC(get_hardware_params);
+	COPY_PROC(get_color_mapping_procs);
+	COPY_PROC(get_color_comp_index);
+	COPY_PROC(encode_color);
+	COPY_PROC(decode_color);
 #undef COPY_PROC
 	/* If using a command list, already opened the device. */
 	if (is_command_list)
@@ -588,8 +593,11 @@ label:\
 
     switch (code = param_read_string(plist, (param_name = "OutputFile"), &ofs)) {
 	case 0:
-	    if (pdev->LockSafetyParams)
+	    if (pdev->LockSafetyParams &&
+		    bytes_compare(ofs.data, ofs.size,
+			(const byte *)ppdev->fname, strlen(ppdev->fname))) {
 	        code = gs_error_invalidaccess;
+	    }
 	    else
 		code = validate_output_file(&ofs);
 	    if (code >= 0)
@@ -975,6 +983,7 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
     if (target == (gx_device *)mdev) {
 	/* The following is a special hack for setting up printer devices. */
 	assign_dev_procs(mdev, mdproto);
+	gx_device_fill_in_procs((gx_device *)mdev);
     } else
 	gs_make_mem_device(mdev, mdproto, mem, (for_band ? 1 : 0),
 			   (target == (gx_device *)mdev ? NULL : target));
