@@ -14,27 +14,32 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevpsu.c,v 1.4 2002/11/26 22:22:04 tillkamppeter Exp $ */
+/* $Id: gdevpsu.c,v 1.15.2.2 2004/03/24 08:38:53 giles Exp $ */
 /* PostScript-writing utilities */
 #include "math_.h"
 #include "time_.h"
+#include "stat_.h"
+#include "unistd_.h"
 #include "gx.h"
 #include "gscdefs.h"
 #include "gxdevice.h"
 #include "gdevpsu.h"
 #include "spprint.h"
 #include "stream.h"
+#include "gserrors.h"
 
 /* ---------------- Low level ---------------- */
 
 /* Write a 0-terminated array of strings as lines. */
-void
+int
 psw_print_lines(FILE *f, const char *const lines[])
 {
     int i;
-
-    for (i = 0; lines[i] != 0; ++i)
-	fprintf(f, "%s\n", lines[i]);
+    for (i = 0; lines[i] != 0; ++i) {
+	if (fprintf(f, "%s\n", lines[i]) < 0)
+	    return_error(gs_error_ioerror);
+    }
+    return 0;
 }
 
 /* Write the ProcSet name. */
@@ -111,7 +116,7 @@ private const char *const psw_ps_procset[] = {
          "{ PageSize dup  1",  /* x y /a4 [  ] [  ] 1   */
            "5 -1 roll put 0 "  /* x /a4 [ y] 0          */
            "4 -1 roll put "    /* /a4                   */
-           "dup null eq {false} {dup where} ifelse"        
+           "dup null eq {false} {dup where} ifelse"
              "{ exch get exec" /* -                     */
              "}",
              "{ pop"           /* -                     */
@@ -134,18 +139,31 @@ private const char *const psw_ps_procset[] = {
 };
 
 private const char *const psw_end_prolog[] = {
-    "end readonly def",
+    "end def",
     "%%EndResource",		/* ProcSet */
     "/pagesave null def",	/* establish binding */
     "%%EndProlog",
     0
 };
 
+/* Return true when the file is seekable.
+ * On Windows NT ftell() returns some non-EOF value when used on pipes.
+ */
+private bool
+is_seekable(FILE *f)
+{
+    struct stat buf;
+
+    if(fstat(fileno(f), &buf))
+      return 0;
+    return S_ISREG(buf.st_mode);
+}
+
 /*
  * Write the file header, up through the BeginProlog.  This must write to a
  * file, not a stream, because it may be called during finalization.
  */
-void
+int
 psw_begin_file_header(FILE *f, const gx_device *dev, const gs_rect *pbbox,
 		      gx_device_pswrite_common_t *pdpc, bool ascii)
 {
@@ -153,7 +171,7 @@ psw_begin_file_header(FILE *f, const gx_device *dev, const gs_rect *pbbox,
     if (pbbox) {
 	psw_print_bbox(f, pbbox);
 	pdpc->bbox_position = 0;
-    } else if (ftell(f) < 0) {	/* File is not seekable. */
+    } else if (!is_seekable(f)) {	/* File is not seekable. */
 	pdpc->bbox_position = -1;
 	fputs("%%BoundingBox: (atend)\n", f);
 	fputs("%%HiResBoundingBox: (atend)\n", f);
@@ -183,46 +201,59 @@ psw_begin_file_header(FILE *f, const gx_device *dev, const gs_rect *pbbox,
     psw_print_lines(f, psw_begin_prolog);
     fprintf(f, "%% %s\n", gs_copyright);
     fputs("%%BeginResource: procset ", f);
+    fflush(f);
     psw_print_procset_name(f, dev, pdpc);
     fputs("\n/", f);
+    fflush(f);
     psw_print_procset_name(f, dev, pdpc);
     fputs(" 80 dict dup begin\n", f);
     psw_print_lines(f, psw_ps_procset);
+    fflush(f);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
+    return 0;
 }
 
 /*
  * End the file header.
  */
-void
+int
 psw_end_file_header(FILE *f)
 {
-    psw_print_lines(f, psw_end_prolog);
+    return psw_print_lines(f, psw_end_prolog);
 }
 
 /*
  * End the file.
  */
-void
+int
 psw_end_file(FILE *f, const gx_device *dev,
 	     const gx_device_pswrite_common_t *pdpc, const gs_rect *pbbox,
-             int page_count)
+             int /* should be long */ page_count)
 {
     if (f == NULL)
-        return;		/* clients should be more careful */
-    fprintf(f, "%%%%Trailer\n%%%%Pages: %ld\n", page_count);
+        return 0;	/* clients should be more careful */
+    fprintf(f, "%%%%Trailer\n%%%%Pages: %ld\n", (long)page_count);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
     if (dev->PageCount > 0 && pdpc->bbox_position != 0) {
 	if (pdpc->bbox_position >= 0) {
 	    long save_pos = ftell(f);
 
 	    fseek(f, pdpc->bbox_position, SEEK_SET);
 	    psw_print_bbox(f, pbbox);
-	    fputc('%', f);
-	    fseek(f, save_pos, SEEK_SET);
+            fputc('%', f);
+            if (ferror(f))
+                return_error(gs_error_ioerror);
+            fseek(f, save_pos, SEEK_SET);
 	} else
 	    psw_print_bbox(f, pbbox);
     }
     if (!pdpc->ProduceEPS)
 	fputs("%%EOF\n", f);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
+    return 0;
 }
 
 /* ---------------- Page level ---------------- */
@@ -230,7 +261,7 @@ psw_end_file(FILE *f, const gx_device *dev,
 /*
  * Write the page header.
  */
-void
+int
 psw_write_page_header(stream *s, const gx_device *dev,
                       const gx_device_pswrite_common_t *pdpc,
                       bool do_scale, long page_ord, int dictsize)
@@ -271,6 +302,8 @@ psw_write_page_header(stream *s, const gx_device *dev,
 	while (p->size_name[0] == '/' &&
 	       (p->width != width || p->height != height))
 	    ++p;
+	/* If no match then decrement pointer back to /letter */
+	if (p->width == 0 && p->height == 0) --p;
 	pprintd2(s, "%d %d ", width, height);
 	pprints1(s, "%s setpagesize\n", p->size_name);
     }
@@ -279,17 +312,24 @@ psw_write_page_header(stream *s, const gx_device *dev,
 	pprintg2(s, "%g %g scale\n",
 		 72.0 / dev->HWResolution[0], 72.0 / dev->HWResolution[1]);
     stream_puts(s, "%%EndPageSetup\ngsave mark\n");
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
+    return 0;
 }
 
 /*
  * Write the page trailer.  We do this directly to the file, rather than to
  * the stream, because we may have to do it during finalization.
  */
-void
+int
 psw_write_page_trailer(FILE *f, int num_copies, int flush)
 {
     if (num_copies != 1)
 	fprintf(f, "userdict /#copies %d put\n", num_copies);
     fprintf(f, "cleartomark end end pagesave restore %s\n%%%%PageTrailer\n",
 	    (flush ? "showpage" : "copypage"));
+    fflush(f);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
+    return 0;
 }
