@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevpsu.h,v 1.4 2002/11/26 22:22:04 tillkamppeter Exp $ */
+/* $Id: gdevpsu.h,v 1.9 2003/03/20 20:27:46 alexcher Exp $ */
 /* Interface to PostScript-writing utilities */
 
 #ifndef gdevpsu_INCLUDED
@@ -34,7 +34,7 @@ typedef struct gx_device_pswrite_common_s {
 /* ---------------- Low level ---------------- */
 
 /* Write a 0-terminated array of strings as lines. */
-void psw_print_lines(P2(FILE *f, const char *const lines[]));
+int psw_print_lines(FILE *f, const char *const lines[]);
 
 /* ---------------- File level ---------------- */
 
@@ -42,31 +42,31 @@ void psw_print_lines(P2(FILE *f, const char *const lines[]));
  * Write the file header, up through the BeginProlog.  This must write to a
  * file, not a stream, because it may be called during finalization.
  */
-void psw_begin_file_header(P5(FILE *f, const gx_device *dev,
-			      const gs_rect *pbbox,
-			      gx_device_pswrite_common_t *pdpc, bool ascii));
+int psw_begin_file_header(FILE *f, const gx_device *dev,
+			   const gs_rect *pbbox,
+			   gx_device_pswrite_common_t *pdpc, bool ascii);
 
 /* End the file header.*/
-void psw_end_file_header(P1(FILE *f));
+int psw_end_file_header(FILE *f);
 
 /* End the file. */
-void psw_end_file(P5(FILE *f, const gx_device *dev,
-		     const gx_device_pswrite_common_t *pdpc,
-		     const gs_rect *pbbox, int page_count));
+int psw_end_file(FILE *f, const gx_device *dev,
+		  const gx_device_pswrite_common_t *pdpc,
+		  const gs_rect *pbbox, int page_count);
 
 /* ---------------- Page level ---------------- */
 
 /*
  * Write the page header.
  */
-void psw_write_page_header(P6(stream *s, const gx_device *dev,
-			      const gx_device_pswrite_common_t *pdpc,
-			      bool do_scale, long page_ord, int dictsize));
+int psw_write_page_header(stream *s, const gx_device *dev,
+			   const gx_device_pswrite_common_t *pdpc,
+			   bool do_scale, long page_ord,  int dictsize);
 /*
  * Write the page trailer.  We do this directly to the file, rather than to
  * the stream, because we may have to do it during finalization.
  */
-void psw_write_page_trailer(P3(FILE *f, int num_copies, int flush));
+int psw_write_page_trailer(FILE *f, int num_copies, int flush);
 
 #endif /* gdevpsu_INCLUDED */
 
