@@ -14,13 +14,13 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevtfax.h,v 1.3 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gdevtfax.h,v 1.7 2002/06/16 07:25:26 lpd Exp $ */
 /* Entry points to the TIFF/fax writing driver */
 
 #ifndef gdevtfax_INCLUDED
 #  define gdevtfax_INCLUDED
 
-int gdev_fax_print_page_stripped(P4(gx_device_printer *pdev, FILE *prn_stream,
-				    stream_CFE_state *ss, long rows_per_strip));
+int gdev_fax_print_page_stripped(gx_device_printer *pdev, FILE *prn_stream,
+				 stream_CFE_state *ss, long rows_per_strip);
 
 #endif /* gdevtfax_INCLUDED */
