@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevwddb.c,v 1.3 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gdevwddb.c,v 1.6 2002/06/16 05:48:55 lpd Exp $ */
 /*
  * Microsoft Windows 3.n driver for Ghostscript,
  * using device-dependent bitmap.
@@ -33,9 +33,9 @@ typedef struct gx_device_win_ddb_s gx_device_win_ddb;
 #define wdev ((gx_device_win_ddb *)dev)
 
 /* Forward references */
-private void near win_addtool(P2(gx_device_win_ddb *, int));
-private void near win_maketools(P2(gx_device_win_ddb *, HDC));
-private void near win_destroytools(P1(gx_device_win_ddb *));
+private void near win_addtool(gx_device_win_ddb *, int);
+private void near win_maketools(gx_device_win_ddb *, HDC);
+private void near win_destroytools(gx_device_win_ddb *);
 
 /* Device procedures */
 
