@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevwpr2.c,v 1.3 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gdevwpr2.c,v 1.15.2.1 2004/08/27 21:36:36 dan Exp $ */
 /*
  * Microsoft Windows 3.n printer driver for Ghostscript.
  * Original version by Russell Lang and
@@ -229,6 +229,7 @@ gx_device_win_pr2 far_data gs_mswinpr2_device =
     NULL,			/* win32_hdevnames */
     NULL,			/* lpfnAbortProc */
     NULL,			/* lpfnCancelProc */
+    NULL,			/* hDlgModeless */
     false,			/* use_old_spool_name */
     NULL			/* original_device */
 };
@@ -594,7 +595,6 @@ win_pr2_print_page(gx_device_printer * pdev, FILE * file)
 	    ShowWindow(wdev->hDlgModeless, SW_HIDE);
     }
 
-  bmp_done:
     GlobalUnlock(hrow);
     GlobalFree(hrow);
 
@@ -608,19 +608,21 @@ win_pr2_print_page(gx_device_printer * pdev, FILE * file)
 
 /* Map a r-g-b color to a color index. */
 private gx_color_index
-win_pr2_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		      gx_color_value b)
+win_pr2_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
+    gx_color_value r = cv[0];
+    gx_color_value g = cv[1];
+    gx_color_value b = cv[2];
     switch (dev->color_info.depth) {
 	case 1:
-	    return gdev_prn_map_rgb_color(dev, r, g, b);
+	    return gdev_prn_map_rgb_color(dev, cv);
 	case 4:
 	    /* use only 8 colors */
 	    return (r > (gx_max_color_value / 2 + 1) ? 4 : 0) +
 		(g > (gx_max_color_value / 2 + 1) ? 2 : 0) +
 		(b > (gx_max_color_value / 2 + 1) ? 1 : 0);
 	case 8:
-	    return pc_8bit_map_rgb_color(dev, r, g, b);
+	    return pc_8bit_map_rgb_color(dev, cv);
 	case 24:
 	    return gx_color_value_to_byte(r) +
 		((uint) gx_color_value_to_byte(g) << 8) +
@@ -687,6 +689,22 @@ win_pr2_set_bpp(gx_device * dev, int depth)
     }
     
     ((gx_device_win_pr2 *)dev)->selected_bpp = depth;
+
+    /* copy encode/decode procedures */
+    dev->procs.encode_color = dev->procs.map_rgb_color;
+    dev->procs.decode_color = dev->procs.map_color_rgb;
+    if (depth == 1) {
+	dev->procs.get_color_mapping_procs = 
+	    gx_default_DevGray_get_color_mapping_procs;
+	dev->procs.get_color_comp_index = 
+	    gx_default_DevGray_get_color_comp_index;
+    }
+    else {
+	dev->procs.get_color_mapping_procs = 
+	    gx_default_DevRGB_get_color_mapping_procs;
+	dev->procs.get_color_comp_index = 
+	    gx_default_DevRGB_get_color_comp_index;
+    }
 }
 
 /********************************************************************************/
@@ -703,7 +721,7 @@ win_pr2_get_params(gx_device * pdev, gs_param_list * plist)
 	code = param_write_bool(plist, "NoCancel",
 				&(wdev->nocancel));
     if (code >= 0)
-	code = param_write_bool(plist, "QueryUser",
+	code = param_write_int(plist, "QueryUser",
 				&(wdev->query_user));
     if (code >= 0)
 	code = win_pr2_write_user_settings(wdev, plist);
@@ -1078,7 +1096,7 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     }
     
     if (wdev->win32_hdevmode) {
-	LPDEVMODE pdevmode = (LPDEVMODE) GlobalLock(GlobalLock(wdev->win32_hdevmode));
+	LPDEVMODE pdevmode = (LPDEVMODE) GlobalLock(wdev->win32_hdevmode);
 	if (pdevmode) {
 	    memcpy(pdevmode, podevmode, sizeof(DEVMODE));
 	    GlobalUnlock(wdev->win32_hdevmode);
