@@ -14,13 +14,16 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevwprn.c,v 1.3 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gdevwprn.c,v 1.8 2002/10/31 10:06:01 ghostgum Exp $ */
 /*
  * Microsoft Windows 3.n printer driver for Ghostscript.
  *
  * Original version by Russell Lang and
  * L. Peter Deutsch, Aladdin Enterprises.
  */
+
+/* This driver is very slow and as of 2002-09-14 it does not work. */
+
 #include "gdevmswn.h"
 #include "gp.h"
 #include "gpcheck.h"
@@ -45,9 +48,9 @@ typedef struct gx_device_win_prn_s gx_device_win_prn;
 #define wdev ((gx_device_win_prn *)dev)
 
 /* Forward references */
-private void near win_prn_addtool(P2(gx_device_win_prn *, int));
-private void near win_prn_maketools(P2(gx_device_win_prn *, HDC));
-private void near win_prn_destroytools(P1(gx_device_win_prn *));
+private void near win_prn_addtool(gx_device_win_prn *, int);
+private void near win_prn_maketools(gx_device_win_prn *, HDC);
+private void near win_prn_destroytools(gx_device_win_prn *);
 BOOL CALLBACK _export AbortProc(HDC, int);
 
 /* Device procedures */
@@ -205,6 +208,22 @@ win_prn_open(gx_device * dev)
 	wdev->nColors = 2;
     }
 
+    /* copy encode/decode procedures */
+    wdev->procs.encode_color = wdev->procs.map_rgb_color;
+    wdev->procs.decode_color = wdev->procs.map_color_rgb;
+    if (dev->color_info.depth == 1) {
+	wdev->procs.get_color_mapping_procs = 
+	    gx_default_DevGray_get_color_mapping_procs;
+	wdev->procs.get_color_comp_index = 
+	    gx_default_DevGray_get_color_comp_index;
+    }
+    else {
+	wdev->procs.get_color_mapping_procs = 
+	    gx_default_DevRGB_get_color_mapping_procs;
+	wdev->procs.get_color_comp_index = 
+	    gx_default_DevRGB_get_color_comp_index;
+    }
+
     /* create palette for display */
     if ((wdev->limgpalette = win_makepalette((gx_device_win *) dev))
 	== (LPLOGPALETTE) NULL) {
@@ -303,11 +322,10 @@ win_prn_output_page(gx_device * dev, int num_copies, int flush)
 
 /* Map a r-g-b color to the colors available under Windows */
 private gx_color_index
-win_prn_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		      gx_color_value b)
+win_prn_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
     int i = wdev->nColors;
-    gx_color_index color = win_map_rgb_color(dev, r, g, b);
+    gx_color_index color = win_map_rgb_color(dev, cv);
 
     if (color != i)
 	return color;
