@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1995, 1996, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989-2004 artofcode LLC. All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,16 +14,18 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: genarch.c,v 1.5 2003/08/19 15:32:25 tillkamppeter Exp $ */
-/* Generate a header file (arch.h) with parameters */
-/* reflecting the machine architecture and compiler characteristics. */
+/* $Id: genarch.c,v 1.5.2.2 2004/05/06 07:47:25 giles Exp $ */
+/*
+ * Generate a header file (arch.h) with parameters
+ * reflecting the machine architecture and compiler characteristics.
+ */
 
 #include "stdpre.h"
 #include <ctype.h>
 #include <stdio.h>
 /*
- * In theory, not all systems provide <string.h>, or declare memset in
- * that header file, but at this point I don't think we care about any
+ * In theory, not all systems provide <string.h> or <setjmp.h>, or declare
+ * memset in <string.h>, but at this point I don't think we care about any
  * that don't.
  */
 #include <string.h>
@@ -62,14 +64,30 @@ define_int(FILE *f, const char *str, int value)
     fprintf(f, "#define %s %d\n", str, value);
 }
 
-const char ff_str[] = "ffffffffffffffff";	/* 8 bytes */
+private void
+print_ffs(FILE *f, int nbytes)
+{
+    int i;
+
+    for (i = 0; i < nbytes; ++i)
+	fprintf(f, "ff");
+}
+
+private int
+ilog2(int n)
+{
+    int i = 0, m = n;
+
+    while (m > 1)
+	++i, m = (m + 1) >> 1;
+    return i;
+}
 
 int
 main(int argc, char *argv[])
 {
     char *fname = argv[1];
     long one = 1;
-    int ff_strlen = sizeof(ff_str) - 1;
     struct {
 	char c;
 	short s;
@@ -94,12 +112,11 @@ main(int argc, char *argv[])
 	char c;
 	double d;
     } sd;
+    /* Some architectures have special alignment requirements for jmpbuf. */
     struct {
 	char c;
-	jmp_buf jb;
-    } sjb;
-    static int log2s[17] =
-    {0, 0, 1, 0, 2, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 4};
+	jmp_buf j;
+    } sj;
     long lm1 = -1;
     long lr1 = lm1 >> 1, lr2 = lm1 >> 2;
     unsigned long um1 = ~(unsigned long)0;
@@ -115,7 +132,7 @@ main(int argc, char *argv[])
 	float f;
 	int i;
 	long l;
-    } f0 , f1, fm1;
+    } f0, f1, fm1;
     int floats_are_IEEE;
     FILE *f = fopen(fname, "w");
 
@@ -123,7 +140,9 @@ main(int argc, char *argv[])
 	fprintf(stderr, "genarch.c: can't open %s for writing\n", fname);
 	return exit_FAILED;
     }
-    fprintf(f, "/* Parameters derived from machine and compiler architecture */\n");
+    fprintf(f, "/* Parameters derived from machine and compiler architecture. */\n");
+    fprintf(f, "/* This file is generated mechanically by genarch.c. */\n");
+
     /* We have to test the size dynamically here, */
     /* because the preprocessor can't evaluate sizeof. */
     f0.f = 0.0, f1.f = 1.0, fm1.f = -1.0;
@@ -141,14 +160,15 @@ main(int argc, char *argv[])
     define_int(f, "ARCH_ALIGN_PTR_MOD", OFFSET_IN(sp, p));
     define_int(f, "ARCH_ALIGN_FLOAT_MOD", OFFSET_IN(sf, f));
     define_int(f, "ARCH_ALIGN_DOUBLE_MOD", OFFSET_IN(sd, d));
-    define_int(f, "ARCH_ALIGN_JMP_BUF_MOD", OFFSET_IN(sjb, jb));
+    define_int(f, "ARCH_ALIGN_STRUCT_MOD", OFFSET_IN(sj, j));
 #undef OFFSET_IN
 
     section(f, "Scalar sizes");
 
-    define_int(f, "ARCH_LOG2_SIZEOF_SHORT", log2s[size_of(short)]);
-    define_int(f, "ARCH_LOG2_SIZEOF_INT", log2s[size_of(int)]);
-    define_int(f, "ARCH_LOG2_SIZEOF_LONG", log2s[size_of(long)]);
+    define_int(f, "ARCH_LOG2_SIZEOF_CHAR", ilog2(size_of(char)));
+    define_int(f, "ARCH_LOG2_SIZEOF_SHORT", ilog2(size_of(short)));
+    define_int(f, "ARCH_LOG2_SIZEOF_INT", ilog2(size_of(int)));
+    define_int(f, "ARCH_LOG2_SIZEOF_LONG", ilog2(size_of(long)));
     define_int(f, "ARCH_SIZEOF_PTR", size_of(char *));
     define_int(f, "ARCH_SIZEOF_FLOAT", size_of(float));
     define_int(f, "ARCH_SIZEOF_DOUBLE", size_of(double));
@@ -168,10 +188,18 @@ main(int argc, char *argv[])
 
     section(f, "Unsigned max values");
 
+    /*
+     * We can't use fprintf with a numeric value for PRINT_MAX, because
+     * too many compilers produce warnings or do the wrong thing for
+     * complementing or widening unsigned types.
+     */
 #define PRINT_MAX(str, typ, tstr, l)\
-  define(f, str);\
-  fprintf(f, "((%s)0x%s%s + (%s)0)\n",\
-    tstr, ff_str + ff_strlen - size_of(typ) * 2, l, tstr)
+  BEGIN\
+    define(f, str);\
+    fprintf(f, "((%s)0x", tstr);\
+    print_ffs(f, sizeof(typ));\
+    fprintf(f, "%s + (%s)0)\n", l, tstr);\
+  END
     PRINT_MAX("ARCH_MAX_UCHAR", unsigned char, "unsigned char", "");
     PRINT_MAX("ARCH_MAX_USHORT", unsigned short, "unsigned short", "");
     /*
@@ -191,7 +219,8 @@ main(int argc, char *argv[])
      * non-linearity in the time required to fill blocks with memset.
      */
     {
-#define MAX_BLOCK (1 << 20)
+#define MAX_BLOCK (1 << 22)	/* max 4M cache */
+#define MAX_NREPS (1 << 10)	/* limit the number of reps we try */
 	static char buf[MAX_BLOCK];
 	int bsize = 1 << 10;
 	int nreps = 1;
@@ -203,10 +232,10 @@ main(int argc, char *argv[])
 	 * long enough to exceed the likely uncertainty.
 	 */
 
-	while ((t = time_clear(buf, bsize, nreps)) == 0)
+	while (nreps < MAX_NREPS && (t = time_clear(buf, bsize, nreps)) == 0)
 	    nreps <<= 1;
 	t_eps = t;
-	while ((t = time_clear(buf, bsize, nreps)) < t_eps * 10)
+	while (nreps < MAX_NREPS && (t = time_clear(buf, bsize, nreps)) < t_eps * 10)
 	    nreps <<= 1;
 
 	/*
