@@ -180,24 +180,20 @@ rl(FILE * in, char *str, int len)
 private void
 wsc(FILE * out, const byte *str, int len)
 {
-    int n = 0;
-    const byte *p = str;
-    int i;
+    fputs("\n\"", out);
+    while (*str) {
+        int c = *str++;
 
-    for (i = 0; i < len; ++i) {
-	int c = str[i];
-
-	fprintf(out,
-		(c < 32 || c >= 127 ? "%d," :
-		 c == '\'' || c == '\\' ? "'\\%c'," : "'%c',"),
-		c);
-	if (++n == 15) {
-	    fputs("\n", out);
-	    n = 0;
-	}
+        if (c == '\n') {
+            fputs("\\n", out);
+            if (*str) fputs("\"\n\"", out);
+        } else if (c == '\\' || c == '\"') {
+            putc('\\', out);
+            putc(c, out);
+        } else if (c < 32 || c >= 127) fprintf(out, "\\%03o", c);
+        else putc(c, out);
     }
-    if (n != 0)
-	fputc('\n', out);
+    putc('\"', out);
 }
 private void
 ws(FILE * out, const byte *str, int len, bool to_c)
@@ -460,9 +456,9 @@ merge_to_c(const char *prefix, const char *inname, FILE * in, FILE * config,
     fputs("\n", out);
     fputs("/* Pre-compiled interpreter initialization string. */\n", out);
     fputs("\n", out);
-    fputs("const unsigned char gs_init_string[] = {\n", out);
+    fputs("const unsigned char gs_init_string[] =", out);
     mergefile(prefix, inname, in, config, out, true, false);
-    fputs("10};\n", out);
+    fputs(";\n", out);
     fputs("const unsigned int gs_init_string_sizeof = sizeof(gs_init_string);\n", out);
 }
 
