@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gp_getnv.c,v 1.3 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gp_getnv.c,v 1.5 2002/06/16 05:48:55 lpd Exp $ */
 /* Standard implementation of gp_getenv */
 #include "stdio_.h"
 #include "string_.h"
@@ -23,7 +23,7 @@
 #include "gp.h"
 
 /* Import the C getenv function. */
-extern char *getenv(P1(const char *));
+extern char *getenv(const char *);
 
 /* Get the value of an environment variable.  See gp.h for details. */
 int
