@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gp_iwatc.c,v 1.4 2003/07/13 04:43:17 easysw Exp $ */
+/* $Id: gp_iwatc.c,v 1.13.2.1 2003/12/12 22:08:02 giles Exp $ */
 /* Intel processor, Watcom C-specific routines for Ghostscript */
 #include "dos_.h"
 #include <fcntl.h>
@@ -27,13 +27,13 @@
 #include "gpmisc.h"
 
 /* Library routines not declared in a standard header */
-extern char *mktemp(P1(char *));	/* in gp_mktmp.c */
+extern char *mktemp(char *);	/* in gp_mktmp.c */
 
 /* Define a substitute for stdprn (see below). */
 private FILE *gs_stdprn;
 
 /* Forward declarations */
-private void handle_FPE(P1(int));
+private void handle_FPE(int);
 
 /* Do platform-dependent initialization. */
 void
@@ -63,6 +63,7 @@ gp_exit(int exit_status, int code)
 void
 gp_do_exit(int exit_status)
 {
+    exit(exit_status);
 }
 
 /* ------ Printer accessing ------ */
@@ -70,7 +71,7 @@ gp_do_exit(int exit_status)
 /* Open a connection to a printer.  A null file name means use the */
 /* standard printer connected to the machine, if any. */
 /* Return NULL if the connection could not be opened. */
-extern void gp_set_file_binary(P2(int, int));
+extern void gp_set_file_binary(int, int);
 FILE *
 gp_open_printer(char fname[gp_file_name_sizeof], int binary_mode)
 {
@@ -125,8 +126,9 @@ gp_open_scratch_file(const char *prefix, char *fname, const char *mode)
 {	      /* The -7 is for XXXXXXX */
     int prefix_length = strlen(prefix);
     int len = gp_file_name_sizeof - prefix_length - 7;
+    FILE *f;
 
-    if (gp_pathstring_not_bare(prefix, prefix_length) ||
+    if (gp_file_name_is_absolute(prefix, prefix_length) ||
 	gp_gettmpdir(fname, &len) != 0
 	)
 	*fname = 0;
@@ -144,7 +146,10 @@ gp_open_scratch_file(const char *prefix, char *fname, const char *mode)
     strcat(fname, prefix);
     strcat(fname, "XXXXXX");
     mktemp(fname);
-    return gp_fopentemp(fname, mode);
+    f = gp_fopentemp(fname, mode);
+    if (f == NULL)
+	eprintf1("**** Could not open temporary file %s\n", fname);
+    return f;
 }
 
 
@@ -154,3 +159,24 @@ gp_fopen(const char *fname, const char *mode)
 {
     return fopen(fname, mode);
 }
+
+/* ------ Font enumeration ------ */
+ 
+ /* This is used to query the native os for a list of font names and
+  * corresponding paths. The general idea is to save the hassle of
+  * building a custom fontmap file.
+  */
+ 
+void *gp_enumerate_fonts_init(gs_memory_t *mem)
+{
+    return NULL;
+}
+         
+int gp_enumerate_fonts_next(void *enum_state, char **fontname, char **path)
+{
+    return 0;
+}
+                         
+void gp_enumerate_fonts_free(void *enum_state)
+{
+}           
