@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gp_strdl.c,v 1.3 2002/04/23 11:58:43 easysw Exp $ */
+/* $Id: gp_strdl.c,v 1.5 2002/06/16 05:48:55 lpd Exp $ */
 /* Default, stream-based readline implementation */
 #include "std.h"
 #include "gstypes.h"
@@ -31,7 +31,7 @@ int
 gp_readline(stream *s_in, stream *s_out, void *readline_data,
 	    gs_const_string *prompt, gs_string * buf,
 	    gs_memory_t * bufmem, uint * pcount, bool *pin_eol,
-	    bool (*is_stdin)(P1(const stream *)))
+	    bool (*is_stdin)(const stream *))
 {
     return sreadline(s_in, s_out, readline_data, prompt, buf, bufmem, pcount,
 		     pin_eol, is_stdin);
