@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsalpha.h,v 1.3 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gsalpha.h,v 1.5 2002/06/16 08:45:42 lpd Exp $ */
 /* API for alpha value in graphics state */
 
 #ifndef gsalpha_INCLUDED
@@ -27,7 +27,7 @@
  */
 
 /* Set/read alpha value. */
-int gs_setalpha(P2(gs_state *, floatp));
-float gs_currentalpha(P1(const gs_state *));
+int gs_setalpha(gs_state *, floatp);
+float gs_currentalpha(const gs_state *);
 
 #endif /* gsalpha_INCLUDED */
