@@ -1,4 +1,4 @@
-/* Copyright (C) 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1996-2004 artofcode LLC. All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gscdef.c,v 1.8 2003/07/13 04:43:17 easysw Exp $ */
+/* $Id: gscdef.c,v 1.33.2.4 2004/08/17 19:52:31 giles Exp $ */
 /* Configuration scalars */
 #include "std.h"
 #include "gscdefs.h"		/* interface */
@@ -33,7 +33,7 @@ CONFIG_CONST long gs_buildtime = GS_BUILDTIME;
 
 #ifndef GS_COPYRIGHT
 #  define GS_COPYRIGHT\
-	"Copyright 2003 artofcode LLC and Easy Software Products, all rights reserved."
+	"Copyright (C) 2004 artofcode LLC, Benicia, CA.  All rights reserved."
 #endif
 const char *CONFIG_CONST gs_copyright = GS_COPYRIGHT;
 
@@ -45,7 +45,7 @@ const char *CONFIG_CONST gs_productfamily = GS_PRODUCTFAMILY;
 
 #ifndef GS_PRODUCT
 #  define GS_PRODUCT\
-	"ESP Ghostscript"
+	GS_PRODUCTFAMILY
 #endif
 const char *CONFIG_CONST gs_product = GS_PRODUCT;
 
@@ -64,9 +64,6 @@ gs_revision_number(void)
     return gs_revision;
 }
 
-/* GS_REVISION must be defined in the makefile. */
-CONFIG_CONST long gs_revisionpatch = GS_REVISIONPATCH;
-
 /* GS_REVISIONDATE must be defined in the makefile. */
 CONFIG_CONST long gs_revisiondate = GS_REVISIONDATE;
 
