@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gscpixel.h,v 1.3 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gscpixel.h,v 1.5 2002/06/16 08:45:42 lpd Exp $ */
 /* Interface to DevicePixel color space */
 /* Requires gscspace.h */
 
@@ -22,6 +22,6 @@
 #  define gscpixel_INCLUDED
 
 /* Initialize a DevicePixel color space. */
-int gs_cspace_init_DevicePixel(P2(gs_color_space * pcs, int depth));
+int gs_cspace_init_DevicePixel(gs_color_space * pcs, int depth);
 
 #endif /* gscpixel_INCLUDED */
