@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsdevice.c,v 1.3 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gsdevice.c,v 1.19.2.1 2003/12/20 04:11:34 ray Exp $ */
 /* Device operators for Ghostscript library */
 #include "ctype_.h"
 #include "memory_.h"		/* for memchr, memcpy */
@@ -34,6 +34,7 @@
 #include "gxdevice.h"
 #include "gxdevmem.h"
 #include "gxiodev.h"
+#include "gxcspace.h"
 
 /* Include the extern for the device list. */
 extern_gs_lib_device_list();
@@ -292,9 +293,12 @@ gs_copydevice2(gx_device ** pnew_dev, const gx_device * dev, bool keep_open,
     fill_dev_proc(new_dev, finish_copydevice, gx_default_finish_copydevice);
     code = dev_proc(new_dev, finish_copydevice)(new_dev, dev);
     if (code < 0) {
-	gs_free_object(mem, new_dev, "gs_copydevice(device)");
-	if (a_std)
+	if (a_std) {
 	    gs_free_object(&gs_memory_default, a_std, "gs_copydevice(stype)");
+	    new_dev->stype = NULL;
+	    new_dev->stype_is_dynamic = false;
+	}
+	gs_free_object(mem, new_dev, "gs_copydevice(device)");
 	return code;
     }
     *pnew_dev = new_dev;
@@ -399,10 +403,25 @@ gs_setdevice_no_init(gs_state * pgs, gx_device * dev)
     /*
      * Just set the device, possibly changing color space but no other
      * device parameters.
+     * 
+     * Make sure we don't close the device if dev == pgs->device
+     * This could be done by allowing the rc_assign to close the
+     * old 'dev' if the rc goes to 0 (via the device structure's
+     * finalization procedure), but then the 'code' from the dev
+     * closedevice would not be propagated up. We want to allow
+     * the code to be handled, particularly for the pdfwrite
+     * device.
      */
+    if (pgs->device != NULL && pgs->device->rc.ref_count == 1 &&
+	pgs->device != dev) {
+	int code = gs_closedevice(pgs->device);
+
+	if (code < 0)
+	    return code;
+    }
     rc_assign(pgs->device, dev, "gs_setdevice_no_init");
     gs_state_update_device(pgs);
-    return 0;
+    return pgs->overprint ? gs_do_set_overprint(pgs) : 0;
 }
 
 /* Initialize a just-allocated device. */
@@ -470,9 +489,9 @@ gs_closedevice(gx_device * dev)
 
     if (dev->is_open) {
 	code = (*dev_proc(dev, close_device))(dev);
+	dev->is_open = false;
 	if (code < 0)
 	    return_error(code);
-	dev->is_open = false;
     }
     return code;
 }
@@ -550,8 +569,8 @@ gx_device_set_resolution(gx_device * dev, floatp x_dpi, floatp y_dpi)
 {
     dev->HWResolution[0] = x_dpi;
     dev->HWResolution[1] = y_dpi;
-    dev->width = dev->MediaSize[0] * x_dpi / 72.0 + 0.5;
-    dev->height = dev->MediaSize[1] * y_dpi / 72.0 + 0.5;
+    dev->width = (int)(dev->MediaSize[0] * x_dpi / 72.0 + 0.5);
+    dev->height = (int)(dev->MediaSize[1] * y_dpi / 72.0 + 0.5);
 }
 
 /* Set the MediaSize, updating width and height to remain consistent. */
@@ -560,8 +579,8 @@ gx_device_set_media_size(gx_device * dev, floatp media_width, floatp media_heigh
 {
     dev->MediaSize[0] = media_width;
     dev->MediaSize[1] = media_height;
-    dev->width = media_width * dev->HWResolution[0] / 72.0 + 0.499;
-    dev->height = media_height * dev->HWResolution[1] / 72.0 + 0.499;
+    dev->width = (int)(media_width * dev->HWResolution[0] / 72.0 + 0.499);
+    dev->height = (int)(media_height * dev->HWResolution[1] / 72.0 + 0.499);
 }
 
 /*
@@ -656,7 +675,6 @@ gx_device_copy_params(gx_device *dev, const gx_device *target)
 private int
 gx_parse_output_format(gs_parsed_file_name_t *pfn, const char **pfmt)
 {
-    int code;
     bool have_format = false, field = 0;
     int width[2], int_width = sizeof(int) * 3, w = 0;
     uint i;
@@ -714,7 +732,6 @@ gx_parse_output_file_name(gs_parsed_file_name_t *pfn, const char **pfmt,
 			  const char *fname, uint fnlen)
 {
     int code;
-    int i;
 
     *pfmt = 0;
     pfn->memory = 0;
@@ -797,14 +814,18 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
 	if (!parsed.fname)
 	    return_error(gs_error_undefinedfilename);
 	strcpy(fmode, gp_fmode_wb);
-	if (positionable)
-	    strcat(fmode, "+");
-	return parsed.iodev->procs.fopen(parsed.iodev, parsed.fname, fmode,
-					 pfile, NULL, 0);
+  	if (positionable && strcmp(parsed.iodev->dname, "%pipe%"))
+  	    strcat(fmode, "+");
+ 	code = parsed.iodev->procs.fopen(parsed.iodev, parsed.fname, fmode,
+  					 pfile, NULL, 0);
+ 	if (code)
+     	    eprintf1("**** Could not open the file %s .\n", parsed.fname);
+ 	return code;
     }
     *pfile = gp_open_printer((fmt ? pfname : fname), binary);
     if (*pfile)
-	return 0;
+  	return 0;
+    eprintf1("**** Could not open the file %s .\n", (fmt ? pfname : fname));
     return_error(gs_error_invalidfileaccess);
 }
 
