@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsdparam.c,v 1.5 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gsdparam.c,v 1.12.2.1 2004/08/27 21:36:36 dan Exp $ */
 /* Default device parameters for Ghostscript library */
 #include "memory_.h"		/* for memcpy */
 #include "string_.h"		/* for strlen */
@@ -32,7 +32,7 @@
 /* ================ Getting parameters ================ */
 
 /* Forward references */
-private bool param_HWColorMap(P2(gx_device *, byte *));
+private bool param_HWColorMap(gx_device *, byte *);
 
 /* Get the device parameters. */
 int
@@ -65,12 +65,6 @@ gs_get_device_or_hw_params(gx_device * orig_dev, gs_param_list * plist,
     return code;
 }
 
-/* Standard ProcessColorModel values. */
-static const char *const pcmsa[] =
-{
-    "", "DeviceGray", "", "DeviceRGB", "DeviceCMYK"
-};
-
 /* Get standard parameters. */
 int
 gx_default_get_params(gx_device * dev, gs_param_list * plist)
@@ -101,11 +95,11 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     param_string_from_string(dns, dev->dname);
     {
-	const char *cms = pcmsa[colors];
+	const char *cms = get_process_color_model_name(dev);
 
 	/* We might have an uninitialized device with */
 	/* color_info.num_components = 0.... */
-	if (*cms != 0)
+	if ((cms != NULL) && (*cms != '\0'))
 	    param_string_from_string(pcms, cms);
 	else
 	    pcms.data = 0;
@@ -175,7 +169,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     if (colors > 1) {
 	int RGBValues = dev->color_info.max_color + 1;
-	long ColorValues = 1L << depth;
+	long ColorValues = (depth >= 32 ? -1 : 1L << depth);
 
 	if ((code = param_write_int(plist, "RedValues", &RGBValues)) < 0 ||
 	    (code = param_write_int(plist, "GreenValues", &RGBValues)) < 0 ||
@@ -325,8 +319,8 @@ gdev_write_input_page_size(int index, gs_param_dict * pdict,
 {
     gdev_input_media_t media;
 
-    media.PageSize[0] = media.PageSize[2] = width_points;
-    media.PageSize[1] = media.PageSize[3] = height_points;
+    media.PageSize[0] = media.PageSize[2] = (float) width_points;
+    media.PageSize[1] = media.PageSize[3] = (float) height_points;
     media.MediaColor = 0;
     media.MediaWeight = 0;
     media.MediaType = 0;
@@ -382,19 +376,19 @@ gdev_end_output_media(gs_param_list * mlist, gs_param_dict * pdict)
 /* ================ Putting parameters ================ */
 
 /* Forward references */
-private int param_anti_alias_bits(P3(gs_param_list *, gs_param_name, int *));
-private int param_MediaSize(P4(gs_param_list *, gs_param_name,
-			       const float *, gs_param_float_array *));
+private int param_anti_alias_bits(gs_param_list *, gs_param_name, int *);
+private int param_MediaSize(gs_param_list *, gs_param_name,
+			    const float *, gs_param_float_array *);
 
-private int param_check_bool(P4(gs_param_list *, gs_param_name, bool, bool));
-private int param_check_long(P4(gs_param_list *, gs_param_name, long, bool));
-
-#define param_check_int(plist, pname, ival, defined)\
-  param_check_long(plist, pname, (long)(ival), defined)
-private int param_check_bytes(P5(gs_param_list *, gs_param_name, const byte *, uint, bool));
-
-#define param_check_string(plist, pname, str, defined)\
-  param_check_bytes(plist, pname, (const byte *)str, strlen(str), defined)
+private int param_check_bool(gs_param_list *, gs_param_name, bool, bool);
+private int param_check_long(gs_param_list *, gs_param_name, long, bool);
+#define param_check_int(plist, pname, ival, is_defined)\
+  param_check_long(plist, pname, (long)(ival), is_defined)
+private int param_check_bytes(gs_param_list *, gs_param_name, const byte *,
+			      uint, bool);
+#define param_check_string(plist, pname, str, is_defined)\
+  param_check_bytes(plist, pname, (const byte *)(str), \
+                    (is_defined) ? strlen(str) : 0, is_defined)
 
 /* Set the device parameters. */
 /* If the device was open and the put_params procedure closed it, */
@@ -441,7 +435,7 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     int depth = dev->color_info.depth;
     int GrayValues = dev->color_info.max_gray + 1;
     int RGBValues = dev->color_info.max_color + 1;
-    long ColorValues = 1L << depth;
+    long ColorValues = (depth >= 32 ? -1 : 1L << depth);
     int tab = dev->color_info.anti_alias.text_bits;
     int gab = dev->color_info.anti_alias.graphics_bits;
     gs_param_string cms;
@@ -638,18 +632,25 @@ nce:
 	    break;
     }
 
+    /* Separation, DeviceN Color, and ProcessColorModel related parameters. */
+    {
+	const char * pcms = get_process_color_model_name(dev);
+        /* the device should have set a process model name at this point */
+	if ((code = param_check_string(plist, "ProcessColorModel", pcms, (pcms != NULL))) < 0)
+	    ecode = code;
+    }
+    IGNORE_INT_PARAM("MaxSeparations")
+    if ((code = param_check_bool(plist, "Separations", false, true)) < 0)
+	ecode = code;
+
+    BEGIN_ARRAY_PARAM(param_read_name_array, "SeparationColorNames", scna, scna.size, scne) {
+	break;
+    } END_ARRAY_PARAM(scna, scne);
+
+
     /* Now check nominally read-only parameters. */
     if ((code = param_check_string(plist, "OutputDevice", dev->dname, true)) < 0)
 	ecode = code;
-    if ((code = param_check_string(plist, "ProcessColorModel", pcmsa[colors], colors != 0)) < 0)
-	ecode = code;
-    if ((code = param_check_int(plist, "MaxSeparations", 1, true)) < 0)
-	ecode = code;
-    if ((code = param_check_bool(plist, "Separations", false, true)) < 0)
-	ecode = code;
-    BEGIN_ARRAY_PARAM(param_read_name_array, "SeparationColorNames", scna, 0, scne) {
-	break;
-    } END_ARRAY_PARAM(scna, scne);
     if ((code = param_check_string(plist, "Name", dev->dname, true)) < 0)
 	ecode = code;
     if ((code = param_check_int(plist, "Colors", colors, true)) < 0)
@@ -802,14 +803,14 @@ param_MediaSize(gs_param_list * plist, gs_param_name pname,
 /* its existing value. */
 private int
 param_check_bool(gs_param_list * plist, gs_param_name pname, bool value,
-		 bool defined)
+		 bool is_defined)
 {
     int code;
     bool new_value;
 
     switch (code = param_read_bool(plist, pname, &new_value)) {
 	case 0:
-	    if (defined && new_value == value)
+	    if (is_defined && new_value == value)
 		break;
 	    code = gs_note_error(gs_error_rangecheck);
 	    goto e;
@@ -824,14 +825,14 @@ param_check_bool(gs_param_list * plist, gs_param_name pname, bool value,
 }
 private int
 param_check_long(gs_param_list * plist, gs_param_name pname, long value,
-		 bool defined)
+		 bool is_defined)
 {
     int code;
     long new_value;
 
     switch (code = param_read_long(plist, pname, &new_value)) {
 	case 0:
-	    if (defined && new_value == value)
+	    if (is_defined && new_value == value)
 		break;
 	    code = gs_note_error(gs_error_rangecheck);
 	    goto e;
@@ -846,14 +847,14 @@ param_check_long(gs_param_list * plist, gs_param_name pname, long value,
 }
 private int
 param_check_bytes(gs_param_list * plist, gs_param_name pname, const byte * str,
-		  uint size, bool defined)
+		  uint size, bool is_defined)
 {
     int code;
     gs_param_string new_value;
 
     switch (code = param_read_string(plist, pname, &new_value)) {
 	case 0:
-	    if (defined && new_value.size == size &&
+	    if (is_defined && new_value.size == size &&
 		!memcmp((const char *)str, (const char *)new_value.data,
 			size)
 		)
