@@ -14,14 +14,16 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsfunc3.c,v 1.3 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gsfunc3.c,v 1.13.2.2 2004/12/14 13:19:49 igor Exp $ */
 /* Implementation of LL3 Functions */
 #include "math_.h"
+#include "memory_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gsfunc3.h"
 #include "gsparam.h"
 #include "gxfunc.h"
+#include "stream.h"
 
 /* ---------------- Utilities ---------------- */
 
@@ -45,6 +47,37 @@ fn_free_functions(const gs_function_t *const * Functions, int count,
     gs_free_const_object(mem, Functions, "Functions");
 }
 
+/*
+ * Scale an array of subsidiary functions.  Note that the scale may either
+ * be propagated unchanged (step_ranges = false) or divided among the
+ * (1-output) subfunctions (step_ranges = true).
+ */
+private int
+fn_scale_functions(gs_function_t ***ppsfns, const gs_function_t *const *pfns,
+		   int count, const gs_range_t *pranges, bool step_ranges,
+		   gs_memory_t *mem)
+{
+    gs_function_t **psfns;
+    int code = alloc_function_array(count, &psfns, mem);
+    const gs_range_t *ranges = pranges;
+    int i;
+    
+    if (code < 0)
+	return code;
+    for (i = 0; i < count; ++i) {
+	int code = gs_function_make_scaled(pfns[i], &psfns[i], ranges, mem);
+
+	if (code < 0) {
+	    fn_free_functions((const gs_function_t *const *)psfns, count, mem);
+	    return code;
+	}
+	if (step_ranges)
+	    ++ranges;
+    }
+    *ppsfns = psfns;
+    return 0;
+}
+
 /* ---------------- Exponential Interpolation functions ---------------- */
 
 typedef struct gs_function_ElIn_s {
@@ -144,6 +177,45 @@ fn_ElIn_get_params(const gs_function_t *pfn_common, gs_param_list *plist)
     return ecode;
 }
 
+/* Make a scaled copy of an Exponential Interpolation function. */
+private int
+fn_ElIn_make_scaled(const gs_function_ElIn_t *pfn,
+		     gs_function_ElIn_t **ppsfn,
+		     const gs_range_t *pranges, gs_memory_t *mem)
+{
+    gs_function_ElIn_t *psfn =
+	gs_alloc_struct(mem, gs_function_ElIn_t, &st_function_ElIn,
+			"fn_ElIn_make_scaled");
+    float *c0;
+    float *c1;
+    int code, i;
+
+    if (psfn == 0)
+	return_error(gs_error_VMerror);
+    psfn->params = pfn->params;
+    psfn->params.C0 = c0 =
+	fn_copy_values(pfn->params.C0, pfn->params.n, sizeof(float), mem);
+    psfn->params.C1 = c1 =
+	fn_copy_values(pfn->params.C1, pfn->params.n, sizeof(float), mem);
+    if ((code = ((c0 == 0 && pfn->params.C0 != 0) ||
+		 (c1 == 0 && pfn->params.C1 != 0) ?
+		 gs_note_error(gs_error_VMerror) : 0)) < 0 ||
+	(code = fn_common_scale((gs_function_t *)psfn,
+				(const gs_function_t *)pfn,
+				pranges, mem)) < 0) {
+	gs_function_free((gs_function_t *)psfn, true, mem);
+	return code;
+    }
+    for (i = 0; i < pfn->params.n; ++i) {
+	double base = pranges[i].rmin, factor = pranges[i].rmax - base;
+
+	c1[i] = c1[i] * factor + base;
+	c0[i] = c0[i] * factor + base;
+    }
+    *ppsfn = psfn;
+    return 0;
+}
+
 /* Free the parameters of an Exponential Interpolation function. */
 void
 gs_function_ElIn_free_params(gs_function_ElIn_params_t * params,
@@ -154,6 +226,25 @@ gs_function_ElIn_free_params(gs_function_ElIn_params_t * params,
     fn_common_free_params((gs_function_params_t *) params, mem);
 }
 
+/* Serialize. */
+private int
+gs_function_ElIn_serialize(const gs_function_t * pfn, stream *s)
+{
+    uint n;
+    const gs_function_ElIn_params_t * p = (const gs_function_ElIn_params_t *)&pfn->params;
+    int code = fn_common_serialize(pfn, s);
+
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->C0[0], sizeof(p->C0[0]) * p->n, &n);
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->C1[0], sizeof(p->C1[0]) * p->n, &n);
+    if (code < 0)
+	return code;
+    return sputs(s, (const byte *)&p->N, sizeof(p->N), &n);
+}
+
 /* Allocate and initialize an Exponential Interpolation function. */
 int
 gs_function_ElIn_init(gs_function_t ** ppfn,
@@ -167,8 +258,10 @@ gs_function_ElIn_init(gs_function_t ** ppfn,
 	    (fn_is_monotonic_proc_t) fn_ElIn_is_monotonic,
 	    gs_function_get_info_default,
 	    (fn_get_params_proc_t) fn_ElIn_get_params,
+	    (fn_make_scaled_proc_t) fn_ElIn_make_scaled,
 	    (fn_free_params_proc_t) gs_function_ElIn_free_params,
-	    fn_common_free
+	    fn_common_free,
+	    (fn_serialize_proc_t) gs_function_ElIn_serialize,
 	}
     };
     int code;
@@ -238,8 +331,11 @@ fn_1ItSg_evaluate(const gs_function_t * pfn_common, const float *in, float *out)
     b0 = (i == 0 ? pfn->params.Domain[0] : pfn->params.Bounds[i - 1]);
     b1 = (i == k - 1 ? pfn->params.Domain[1] : pfn->params.Bounds[i]);
     e0 = pfn->params.Encode[2 * i];
-    encoded =
-	(arg - b0) * (pfn->params.Encode[2 * i + 1] - e0) / (b1 - b0) + e0;
+    if (b1 == b0)
+	encoded = e0;
+    else
+	encoded =
+	    (arg - b0) * (pfn->params.Encode[2 * i + 1] - e0) / (b1 - b0) + e0;
     if_debug3('~', "[~]1ItSg %g in %d => %g\n", arg, i, encoded);
     return gs_function_evaluate(pfn->params.Functions[i], &encoded, out);
 }
@@ -267,17 +363,43 @@ fn_1ItSg_is_monotonic(const gs_function_t * pfn_common,
     for (i = 0; i < pfn->params.k; ++i) {
 	float b0 = (i == 0 ? d0 : pfn->params.Bounds[i - 1]);
 	float b1 = (i == k - 1 ? d1 : pfn->params.Bounds[i]);
+	const float small = 0.0000001 * (b1 - b0);
 	float e0, e1;
 	float w0, w1;
 	int code;
+	float vv0, vv1;
+	double vb0, vb1;
 
-	if (v0 >= b1 || v1 <= b0)
+	if (v0 >= b1)
 	    continue;
+	if (v0 >= b1 - small)
+	    continue; /* Ignore a small noize */
+	vv0 = max(b0, v0);
+	vv1 = v1;
+	if (vv1 > b1 && v1 < b1 + small)
+	    vv1 = b1; /* Ignore a small noize */
+	if (vv0 == vv1)
+	    return 1;
+	if (vv0 < b1 && vv1 > b1)
+	    return 0; /* Consider stitches as monotonity beraks. */
 	e0 = pfn->params.Encode[2 * i];
 	e1 = pfn->params.Encode[2 * i + 1];
-	w0 = (max(v0, b0) - b0) * (e1 - e0) / (b1 - b0) + e0;
-	w1 = (min(v1, b1) - b0) * (e1 - e0) / (b1 - b0) + e0;
+	vb0 = max(vv0, b0);
+	vb1 = min(vv1, b1);
+	w0 = (float)(vb0 - b0) * (e1 - e0) / (b1 - b0) + e0;
+	w1 = (float)(vb1 - b0) * (e1 - e0) / (b1 - b0) + e0;
 	/* Note that w0 > w1 is now possible if e0 > e1. */
+	if (e0 > e1) {
+	    if (w0 > e0 && w0 - small <= e0)
+		w0 = e0; /* Suppress a small noize */
+	    if (w1 < e1 && w1 + small >= e1)
+		w1 = e1; /* Suppress a small noize */
+	} else {
+	    if (w0 < e0 && w0 + small >= e0)
+		w0 = e0; /* Suppress a small noize */
+	    if (w1 > e1 && w1 - small <= e1)
+		w1 = e1; /* Suppress a small noize */
+	}
 	if (w0 > w1) {
 	    code = gs_function_is_monotonic(pfn->params.Functions[i],
 					    &w1, &w0, effort);
@@ -335,6 +457,42 @@ fn_1ItSg_get_params(const gs_function_t *pfn_common, gs_param_list *plist)
     return ecode;
 }
 
+/* Make a scaled copy of a 1-Input Stitching function. */
+private int
+fn_1ItSg_make_scaled(const gs_function_1ItSg_t *pfn,
+		     gs_function_1ItSg_t **ppsfn,
+		     const gs_range_t *pranges, gs_memory_t *mem)
+{
+    gs_function_1ItSg_t *psfn =
+	gs_alloc_struct(mem, gs_function_1ItSg_t, &st_function_1ItSg,
+			"fn_1ItSg_make_scaled");
+    int code;
+
+    if (psfn == 0)
+	return_error(gs_error_VMerror);
+    psfn->params = pfn->params;
+    psfn->params.Functions = 0;	/* in case of failure */
+    psfn->params.Bounds =
+	fn_copy_values(pfn->params.Bounds, pfn->params.k - 1, sizeof(float),
+		       mem);
+    psfn->params.Encode =
+	fn_copy_values(pfn->params.Encode, 2 * pfn->params.k, sizeof(float),
+		       mem);
+    if ((code = (psfn->params.Bounds == 0 || psfn->params.Encode == 0 ?
+		 gs_note_error(gs_error_VMerror) : 0)) < 0 ||
+	(code = fn_common_scale((gs_function_t *)psfn,
+				(const gs_function_t *)pfn,
+				pranges, mem)) < 0 ||
+	(code = fn_scale_functions((gs_function_t ***)&psfn->params.Functions,
+				   pfn->params.Functions,
+				   pfn->params.n, pranges, false, mem)) < 0) {
+	gs_function_free((gs_function_t *)psfn, true, mem);
+	return code;
+    }
+    *ppsfn = psfn;
+    return 0;
+}
+
 /* Free the parameters of a 1-Input Stitching function. */
 void
 gs_function_1ItSg_free_params(gs_function_1ItSg_params_t * params,
@@ -346,6 +504,31 @@ gs_function_1ItSg_free_params(gs_function_1ItSg_params_t * params,
     fn_common_free_params((gs_function_params_t *) params, mem);
 }
 
+/* Serialize. */
+private int
+gs_function_1ItSg_serialize(const gs_function_t * pfn, stream *s)
+{
+    uint n;
+    const gs_function_1ItSg_params_t * p = (const gs_function_1ItSg_params_t *)&pfn->params;
+    int code = fn_common_serialize(pfn, s);
+    int k;
+
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->k, sizeof(p->k), &n);
+    if (code < 0)
+	return code;
+
+    for (k = 0; k < p->k && code >= 0; k++) 
+	code = gs_function_serialize(p->Functions[k], s);
+    if (code < 0)
+	return code;
+    code = sputs(s, (const byte *)&p->Bounds[0], sizeof(p->Bounds[0]) * (p->k - 1), &n);
+    if (code < 0)
+	return code;
+    return sputs(s, (const byte *)&p->Encode[0], sizeof(p->Encode[0]) * (p->k * 2), &n);
+}
+
 /* Allocate and initialize a 1-Input Stitching function. */
 int
 gs_function_1ItSg_init(gs_function_t ** ppfn,
@@ -358,8 +541,10 @@ gs_function_1ItSg_init(gs_function_t ** ppfn,
 	    (fn_is_monotonic_proc_t) fn_1ItSg_is_monotonic,
 	    (fn_get_info_proc_t) fn_1ItSg_get_info,
 	    (fn_get_params_proc_t) fn_1ItSg_get_params,
+	    (fn_make_scaled_proc_t) fn_1ItSg_make_scaled,
 	    (fn_free_params_proc_t) gs_function_1ItSg_free_params,
-	    fn_common_free
+	    fn_common_free,
+	    (fn_serialize_proc_t) gs_function_1ItSg_serialize,
 	}
     };
     int n = (params->Range == 0 ? 0 : params->n);
@@ -378,7 +563,7 @@ gs_function_1ItSg_init(gs_function_t ** ppfn,
 	    return_error(gs_error_rangecheck);
 	/* There are only k - 1 Bounds, not k. */
 	if (i < params->k - 1) {
-	    if (params->Bounds[i] <= prev)
+	    if (params->Bounds[i] < prev)
 		return_error(gs_error_rangecheck);
 	    prev = params->Bounds[i];
 	}
@@ -415,12 +600,26 @@ private_st_function_AdOt();
 
 /* Evaluate an Arrayed Output function. */
 private int
-fn_AdOt_evaluate(const gs_function_t * pfn_common, const float *in, float *out)
+fn_AdOt_evaluate(const gs_function_t *pfn_common, const float *in0, float *out)
 {
     const gs_function_AdOt_t *const pfn =
 	(const gs_function_AdOt_t *)pfn_common;
+    const float *in = in0;
+#define MAX_ADOT_IN 16
+    float in_buf[MAX_ADOT_IN];
     int i;
 
+    /*
+     * We have to take special care to handle the case where in and out
+     * overlap.  For the moment, handle it only for a limited number of
+     * input values.
+     */
+    if (in <= out + (pfn->params.n - 1) && out <= in + (pfn->params.m - 1)) {
+	if (pfn->params.m > MAX_ADOT_IN)
+	    return_error(gs_error_rangecheck);
+	memcpy(in_buf, in, pfn->params.m * sizeof(*in));
+	in = in_buf;
+    }
     for (i = 0; i < pfn->params.n; ++i) {
 	int code =
 	    gs_function_evaluate(pfn->params.Functions[i], in, out + i);
@@ -429,6 +628,7 @@ fn_AdOt_evaluate(const gs_function_t * pfn_common, const float *in, float *out)
 	    return code;
     }
     return 0;
+#undef MAX_ADOT_IN
 }
 
 /* Test whether an Arrayed Output function is monotonic. */
@@ -453,6 +653,45 @@ fn_AdOt_is_monotonic(const gs_function_t * pfn_common,
     return result;
 }
 
+/* Return Arrayed Output function information. */
+private void
+fn_AdOt_get_info(const gs_function_t *pfn_common, gs_function_info_t *pfi)
+{
+    const gs_function_AdOt_t *const pfn =
+	(const gs_function_AdOt_t *)pfn_common;
+
+    gs_function_get_info_default(pfn_common, pfi);
+    pfi->Functions = pfn->params.Functions;
+    pfi->num_Functions = pfn->params.n;
+}
+
+/* Make a scaled copy of an Arrayed Output function. */
+private int
+fn_AdOt_make_scaled(const gs_function_AdOt_t *pfn, gs_function_AdOt_t **ppsfn,
+		    const gs_range_t *pranges, gs_memory_t *mem)
+{
+    gs_function_AdOt_t *psfn =
+	gs_alloc_struct(mem, gs_function_AdOt_t, &st_function_AdOt,
+			"fn_AdOt_make_scaled");
+    int code;
+
+    if (psfn == 0)
+	return_error(gs_error_VMerror);
+    psfn->params = pfn->params;
+    psfn->params.Functions = 0;	/* in case of failure */
+    if ((code = fn_common_scale((gs_function_t *)psfn,
+				(const gs_function_t *)pfn,
+				pranges, mem)) < 0 ||
+	(code = fn_scale_functions((gs_function_t ***)&psfn->params.Functions,
+				   pfn->params.Functions,
+				   pfn->params.n, pranges, true, mem)) < 0) {
+	gs_function_free((gs_function_t *)psfn, true, mem);
+	return code;
+    }
+    *ppsfn = psfn;
+    return 0;
+}
+
 /* Free the parameters of an Arrayed Output function. */
 void
 gs_function_AdOt_free_params(gs_function_AdOt_params_t * params,
@@ -462,6 +701,21 @@ gs_function_AdOt_free_params(gs_function_AdOt_params_t * params,
     fn_common_free_params((gs_function_params_t *) params, mem);
 }
 
+/* Serialize. */
+private int
+gs_function_AdOt_serialize(const gs_function_t * pfn, stream *s)
+{
+    const gs_function_AdOt_params_t * p = (const gs_function_AdOt_params_t *)&pfn->params;
+    int code = fn_common_serialize(pfn, s);
+    int k;
+
+    if (code < 0)
+	return code;
+    for (k = 0; k < p->n && code >= 0; k++) 
+	code = gs_function_serialize(p->Functions[k], s);
+    return code;
+}
+
 /* Allocate and initialize an Arrayed Output function. */
 int
 gs_function_AdOt_init(gs_function_t ** ppfn,
@@ -472,10 +726,12 @@ gs_function_AdOt_init(gs_function_t ** ppfn,
 	{
 	    (fn_evaluate_proc_t) fn_AdOt_evaluate,
 	    (fn_is_monotonic_proc_t) fn_AdOt_is_monotonic,
-	    gs_function_get_info_default, /****** WRONG ******/
+	    (fn_get_info_proc_t) fn_AdOt_get_info,
 	    fn_common_get_params,	/****** WHAT TO DO ABOUT THIS? ******/
+	    (fn_make_scaled_proc_t) fn_AdOt_make_scaled,
 	    (fn_free_params_proc_t) gs_function_AdOt_free_params,
-	    fn_common_free
+	    fn_common_free,
+	    (fn_serialize_proc_t) gs_function_AdOt_serialize,
 	}
     };
     int m = params->m, n = params->n;
@@ -501,14 +757,41 @@ gs_function_AdOt_init(gs_function_t ** ppfn,
 	gs_function_AdOt_t *pfn =
 	    gs_alloc_struct(mem, gs_function_AdOt_t, &st_function_AdOt,
 			    "gs_function_AdOt_init");
+	float *domain = (float *)
+	    gs_alloc_byte_array(mem, 2 * m, sizeof(float),
+				"gs_function_AdOt_init(Domain)");
+	int i, j;
 
 	if (pfn == 0)
 	    return_error(gs_error_VMerror);
 	pfn->params = *params;
-	pfn->params.Domain = 0;
+	pfn->params.Domain = domain;
 	pfn->params.Range = 0;
 	pfn->head = function_AdOt_head;
 	pfn->head.is_monotonic = is_monotonic;
+	if (domain == 0) {
+	    gs_function_free((gs_function_t *)pfn, true, mem);
+	    return_error(gs_error_VMerror);
+	}
+	/*
+	 * We compute the Domain as the intersection of the Domains of
+	 * the individual subfunctions.  This isn't quite right: some
+	 * subfunction might actually make use of a larger domain of
+	 * input values.  However, the only place that Arrayed Output
+	 * functions are used is in Shading and similar dictionaries,
+	 * where the input values are clamped to the intersection of
+	 * the individual Domains anyway.
+	 */
+	memcpy(domain, params->Functions[0]->params.Domain,
+	       2 * sizeof(float) * m);
+	for (i = 1; i < n; ++i) {
+	    const float *dom = params->Functions[i]->params.Domain;
+
+	    for (j = 0; j < 2 * m; j += 2, dom += 2) {
+		domain[j] = max(domain[j], dom[0]);
+		domain[j + 1] = min(domain[j + 1], dom[1]);
+	    }
+	}
 	*ppfn = (gs_function_t *) pfn;
     }
     return 0;
