@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsmisc.c,v 1.7 2002/05/17 20:29:29 easysw Exp $ */
+/* $Id: gsmisc.c,v 1.15.2.1 2004/08/31 04:03:30 ray Exp $ */
 /* Miscellaneous utilities for Ghostscript library */
 
 /*
@@ -48,7 +48,6 @@ orig_sqrt(double x)
 #include "gconfigv.h"		/* for USE_ASM */
 #include "gxfarith.h"
 #include "gxfixed.h"
-#include "gscdefs.h"
 
 /* Define private replacements for stdin, stdout, and stderr. */
 FILE *gs_stdio[3];
@@ -159,14 +158,14 @@ dprintf_file_only(const char *file)
 }
 #endif
 void
-printf_program_ident(const char *program_name, long revision_number, long revision_patch)
+printf_program_ident(const char *program_name, long revision_number)
 {
     if (program_name)
 	outprintf((revision_number ? "%s " : "%s"), program_name);
     if (revision_number) {
-	int fpart = revision_number % 100;
-
-	outprintf("%d.%02d.%d", (int)(revision_number / 100), fpart, revision_patch);
+	outprintf("%d.%d.%d", (int)(revision_number / 10000),
+	          (int)((revision_number / 100) % 100),
+		  (int)(revision_number % 100));
     }
 }
 void
@@ -178,7 +177,7 @@ eprintf_program_ident(const char *program_name,
 	if (revision_number) {
 	    int fpart = revision_number % 100;
 
-	    epf("%d.%02d.%d", (int)(revision_number / 100), fpart, gs_revisionpatch);
+	    epf("%d.%02d", (int)(revision_number / 100), fpart);
 	}
 	epf(": ");
     }
@@ -536,7 +535,7 @@ ilog2(int n)
     while (m >= 16)
 	m >>= 4, l += 4;
     return
-	(m <= 1 ? 0 :
+	(m <= 1 ? l :
 	 "\000\000\001\001\002\002\002\002\003\003\003\003\003\003\003\003"[m] + l);
 }
 
@@ -1142,10 +1141,16 @@ gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 static const int isincos[5] =
 {0, 1, 0, -1, 0};
 
+/* GCC with -ffast-math compiles ang/90. as ang*(1/90.), losing precission.
+ * This doesn't happen when the numeral is replaced with a non-const variable.
+ * So we define the variable to work around the GCC problem. 
+ */
+static double const_90_degrees = 90.;
+
 double
 gs_sin_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/*
@@ -1160,7 +1165,7 @@ gs_sin_degrees(double ang)
 double
 gs_cos_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
@@ -1172,7 +1177,7 @@ gs_cos_degrees(double ang)
 void
 gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
