@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gspath2.h,v 1.3 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gspath2.h,v 1.5 2002/06/16 08:45:42 lpd Exp $ */
 /* Level 2 graphics state path procedures */
 /* Requires gsmatrix.h */
 
@@ -22,12 +22,12 @@
 #  define gspath2_INCLUDED
 
 /* Miscellaneous */
-int gs_setbbox(P5(gs_state *, floatp, floatp, floatp, floatp));
+int gs_setbbox(gs_state *, floatp, floatp, floatp, floatp);
 
 /* Rectangles */
-int gs_rectappend(P3(gs_state *, const gs_rect *, uint));
-int gs_rectclip(P3(gs_state *, const gs_rect *, uint));
-int gs_rectfill(P3(gs_state *, const gs_rect *, uint));
-int gs_rectstroke(P4(gs_state *, const gs_rect *, uint, const gs_matrix *));
+int gs_rectappend(gs_state *, const gs_rect *, uint);
+int gs_rectclip(gs_state *, const gs_rect *, uint);
+int gs_rectfill(gs_state *, const gs_rect *, uint);
+int gs_rectstroke(gs_state *, const gs_rect *, uint, const gs_matrix *);
 
 #endif /* gspath2_INCLUDED */
