@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gstype2.c,v 1.3 2002/04/23 11:58:44 easysw Exp $ */
+/* $Id: gstype2.c,v 1.25.2.3 2004/10/28 08:54:35 igor Exp $ */
 /* Adobe Type 2 charstring interpreter */
 #include "math_.h"
 #include "memory_.h"
@@ -26,10 +26,11 @@
 #include "gxmatrix.h"
 #include "gxcoord.h"
 #include "gxistate.h"
-#include "gzpath.h"
+#include "gxpath.h"
 #include "gxfont.h"
 #include "gxfont1.h"
 #include "gxtype1.h"
+#include "gxhintn.h"
 
 /* NOTE: The following are not yet implemented:
  *	Registry items other than 0
@@ -56,7 +57,9 @@ private int
 type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state_t * ipsp,
 	  bool explicit_width)
 {
-    fixed wx;
+    t1_hinter *h = &pcis->h;
+    fixed sbx = fixed_0, sby = fixed_0, wx, wy = fixed_0;
+    int code;
 
     if (explicit_width) {
 	wx = cstack[0] + pcis->pfont->data.nominalWidthX;
@@ -64,6 +67,19 @@ type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state_t * ipsp,
 	--csp;
     } else
 	wx = pcis->pfont->data.defaultWidthX;
+    if (pcis->seac_accent < 0) {
+	if (pcis->sb_set) {
+	    sbx = pcis->lsb.x;
+	    sby = pcis->lsb.y;
+	}
+	if (pcis->width_set) {
+	    wx = pcis->width.x;
+	    wy = pcis->width.y;
+	}
+    }
+    code = t1_hinter__sbw(h, sbx, sby, wx, wy);
+    if (code < 0)
+	return code;
     gs_type1_sbw(pcis, fixed_0, fixed_0, wx, fixed_0);
     /* Back up the interpretation pointer. */
     ipsp->ip--;
@@ -82,30 +98,18 @@ type2_vstem(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack)
 {
     fixed x = 0;
     cs_ptr ap;
+    t1_hinter *h = &pcis->h;
+    int code;
 
-    apply_path_hints(pcis, false);
-    for (ap = cstack; ap + 1 <= csp; x += ap[1], ap += 2)
-	type1_vstem(pcis, x += ap[0], ap[1]);
+    for (ap = cstack; ap + 1 <= csp; x += ap[1], ap += 2) {
+        code = t1_hinter__vstem(h, x += ap[0], ap[1]);
+	if (code < 0)
+	    return code;
+    }
     pcis->num_hints += (csp + 1 - cstack) >> 1;
     return 0;
 }
 
-/* Enable only the hints selected by a mask. */
-private void
-enable_hints(stem_hint_table * psht, const byte * mask)
-{
-    stem_hint *table = &psht->data[0];
-    stem_hint *ph = table + psht->current;
-
-    for (ph = &table[psht->count]; --ph >= table;) {
-	ph->active = (mask[ph->index >> 3] & (0x80 >> (ph->index & 7))) != 0;
-	if_debug6('1', "[1]  %s %u: %g(%g),%g(%g)\n",
-		  (ph->active ? "enable" : "disable"), ph->index,
-		  fixed2float(ph->v0), fixed2float(ph->dv0),
-		  fixed2float(ph->v1), fixed2float(ph->dv1));
-    }
-}
-
 /* ------ Main interpreter ------ */
 
 /*
@@ -115,13 +119,13 @@ enable_hints(stem_hint_table * psht, const byte * mask)
  * argument is only for compatibility with the Type 1 charstring interpreter.
  */
 int
-gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
+gs_type2_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
 		   int *ignore_pindex)
 {
     gs_font_type1 *pfont = pcis->pfont;
     gs_type1_data *pdata = &pfont->data;
+    t1_hinter *h = &pcis->h;
     bool encrypted = pdata->lenIV >= 0;
-    gs_op1_state s;
     fixed cstack[ostack_size];
     cs_ptr csp;
 #define clear CLEAR_CSTACK(cstack, csp)
@@ -143,24 +147,32 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 
     switch (pcis->init_done) {
 	case -1:
+	    t1_hinter__init(h, pcis->path);
 	    break;
 	case 0:
-	    gs_type1_finish_init(pcis, &s);	/* sets sfc, ptx, pty, origin */
+	    gs_type1_finish_init(pcis);	/* sets origin */
+            code = t1_hinter__set_mapping(h, &pcis->pis->ctm,
+			    &pfont->FontMatrix, &pfont->base->FontMatrix,
+			    pcis->scale.x.log2_unit, pcis->scale.x.log2_unit,
+			    pcis->scale.x.log2_unit - pcis->log2_subpixels.x,
+			    pcis->scale.y.log2_unit - pcis->log2_subpixels.y,
+			    pcis->origin.x, pcis->origin.y, 
+			    gs_currentaligntopixels(pfont->dir));
+	    if (code < 0)
+	    	return code;
+	    code = t1_hinter__set_font_data(h, 2, pdata, pcis->no_grid_fitting);
+	    if (code < 0)
+	    	return code;
 	    break;
 	default /*case 1 */ :
-	    ptx = pcis->position.x;
-	    pty = pcis->position.y;
-	    sfc = pcis->fc;
+	    break;
     }
-    sppath = pcis->path;
-    s.pcis = pcis;
     INIT_CSTACK(cstack, csp, pcis);
 
-    if (str == 0)
+    if (pgd == 0)
 	goto cont;
-    ipsp->char_string = *str;
-    ipsp->free_char_string = 0;	/* don't free caller-supplied strings */
-    cip = str->data;
+    ipsp->cs_data = *pgd;
+    cip = pgd->bits.data;
   call:state = crypt_charstring_seed;
     if (encrypted) {
 	int skip = pdata->lenIV;
@@ -180,14 +192,15 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 	    /* This is a number, decode it and push it on the stack. */
 
 	    if (c < c_pos2_0) {	/* 1-byte number */
-		decode_push_num1(csp, c);
+		decode_push_num1(csp, cstack, c);
 	    } else if (c < cx_num4) {	/* 2-byte number */
-		decode_push_num2(csp, c, cip, state, encrypted);
+		decode_push_num2(csp, cstack, c, cip, state, encrypted);
 	    } else if (c == cx_num4) {	/* 4-byte number */
 		long lw;
 
 		decode_num4(lw, cip, state, encrypted);
 		/* 32-bit numbers are 16:16. */
+		CS_CHECK_PUSH(csp, cstack);
 		*++csp = arith_rshift(lw, 16 - _fixed_shift);
 	    } else		/* not possible */
 		return_error(gs_error_invalidfont);
@@ -220,7 +233,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 	    case c_callsubr:
 		c = fixed2int_var(*csp) + pdata->subroutineNumberBias;
 		code = pdata->procs.subr_data
-		    (pfont, c, false, &ipsp[1].char_string);
+		    (pfont, c, false, &ipsp[1].cs_data);
 	      subr:if (code < 0) {
 	            /* Calling a Subr with an out-of-range index is clearly a error:
 	             * the Adobe documentation says the results of doing this are
@@ -235,15 +248,10 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		--csp;
 		ipsp->ip = cip, ipsp->dstate = state;
 		++ipsp;
-		ipsp->free_char_string = code;
-		cip = ipsp->char_string.data;
+		cip = ipsp->cs_data.bits.data;
 		goto call;
 	    case c_return:
-		if (ipsp->free_char_string > 0)
-		    gs_free_const_string(pfont->memory,
-					 ipsp->char_string.data,
-					 ipsp->char_string.size,
-					 "gs_type2_interpret");
+		gs_glyph_data_free(&ipsp->cs_data, "gs_type2_interpret");
 		--ipsp;
 		goto cont;
 	    case c_undoc15:
@@ -258,35 +266,20 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 	    case cx_vstem:
 		goto vstem;
 	    case cx_vmoveto:
-		/*
-		 * Type 2 CharStrings, unlike Type 1, insert an explicit
-		 * closepath before a moveto rather than an implicit one.
-		 * (This makes a difference for charpath.)  Note that if
-		 * we are just getting the glyph info, sppath may be 0:
-		 * this is OK, because check_first_operator will return.
-		 */
-		if (pfont->PaintType != 1 && sppath != 0) {
-		    code = gx_path_close_subpath(sppath);
-		    if (code < 0)
-			return code;
-		}
 		check_first_operator(csp > cstack);
-		accum_y(*csp);
-	      move:if ((pcis->hint_next != 0 || path_is_drawing(sppath)))
-		    apply_path_hints(pcis, true);
-		code = gx_path_add_point(sppath, ptx, pty);
-	      cc:if (code < 0)
+                code = t1_hinter__rmoveto(h, 0, *csp);
+	      move:
+	      cc:
+		if (code < 0)
 		    return code;
 		goto pp;
 	    case cx_rlineto:
 		for (ap = cstack; ap + 1 <= csp; ap += 2) {
-		    accum_xy(ap[0], ap[1]);
-		    code = gx_path_add_line(sppath, ptx, pty);
+		    code = t1_hinter__rlineto(h, ap[0], ap[1]);
 		    if (code < 0)
 			return code;
 		}
-	      pp:if_debug2('1', "[1]pt=(%g,%g)\n",
-			  fixed2float(ptx), fixed2float(pty));
+	      pp:
 		cnext;
 	    case cx_hlineto:
 		vertical = false;
@@ -294,47 +287,41 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 	    case cx_vlineto:
 		vertical = true;
 	      hvl:for (ap = cstack; ap <= csp; vertical = !vertical, ++ap) {
-		    if (vertical)
-			accum_y(*ap);
-		    else
-			accum_x(*ap);
-		    code = gx_path_add_line(sppath, ptx, pty);
+		    if (vertical) {
+			code = t1_hinter__rlineto(h, 0, ap[0]);
+		    } else {
+			code = t1_hinter__rlineto(h, ap[0], 0);
+		    }
 		    if (code < 0)
 			return code;
 		}
 		goto pp;
 	    case cx_rrcurveto:
 		for (ap = cstack; ap + 5 <= csp; ap += 6) {
-		    code = gs_op1_rrcurveto(&s, ap[0], ap[1], ap[2],
+		    code = t1_hinter__rcurveto(h, ap[0], ap[1], ap[2],
 					    ap[3], ap[4], ap[5]);
 		    if (code < 0)
 			return code;
 		}
 		goto pp;
 	    case cx_endchar:
-		/*
-		 * See vmoveto above re closing the subpath.  Note that
-		 * sppath may be 0 if we are just getting the glyph info.
-		 */
-		if (pfont->PaintType != 1 && sppath != 0) {
-		    code = gx_path_close_subpath(sppath);
-		    if (code < 0)
-			return code;
-		}
-		/*
-		 * It is an undocumented (!) feature of Type 2 CharStrings
-		 * that if endchar is invoked with 4 or 5 operands, it is
-		 * equivalent to the Type 1 seac operator!  In this case,
-		 * the asb operand of seac is missing: we assume it is
-		 * the same as the l.s.b. of the accented character.
+  		/*
+		 * It is a feature of Type 2 CharStrings that if endchar is
+		 * invoked with 4 or 5 operands, it is equivalent to the
+		 * Type 1 seac operator. In this case, the asb operand of
+		 * seac is missing: we assume it is the same as the
+		 * l.s.b. of the accented character.  This feature was
+		 * undocumented until the 16 March 2000 version of the Type
+		 * 2 Charstring Format specification, but, thankfully, is
+		 * described in that revision.
 		 */
 		if (csp >= cstack + 3) {
 		    check_first_operator(csp > cstack + 3);
-		    code = gs_type1_seac(pcis, cstack, pcis->lsb.x, ipsp);
+		    code = gs_type1_seac(pcis, cstack, 0, ipsp);
 		    if (code < 0)
 			return code;
 		    clear;
-		    cip = ipsp->char_string.data;
+		    cip = ipsp->cs_data.bits.data;
 		    goto call;
 		}
 		/*
@@ -342,6 +329,15 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		 * In this case, there might be a width on the stack.
 		 */
 		check_first_operator(csp >= cstack);
+		code = t1_hinter__endchar(h, (pcis->seac_accent >= 0));
+		if (code < 0)
+		    return code;
+		if (pcis->seac_accent < 0) {
+		    code = t1_hinter__endglyph(h);
+		    if (code < 0)
+			return code;
+		} else
+		    t1_hinter__setcurrentpoint(h, pcis->save_adxy.x, pcis->save_adxy.y);
 		code = gs_type1_endchar(pcis);
 		if (code == 1) {
 		    /*
@@ -352,31 +348,30 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		     */
 		    pcis->num_hints = 0;
 		    /* do accent of seac */
-		    spt = pcis->position;
 		    ipsp = &pcis->ipstack[pcis->ips_count - 1];
-		    cip = ipsp->char_string.data;
+		    cip = ipsp->cs_data.bits.data;
 		    goto call;
 		}
 		return code;
 	    case cx_rmoveto:
 		/* See vmoveto above re closing the subpath. */
-		if (pfont->PaintType != 1 && sppath != 0) {
-		    code = gx_path_close_subpath(sppath);
-		    if (code < 0)
-			return code;
+		check_first_operator(!((csp - cstack) & 1));
+		if (csp > cstack + 1) {
+		  /* Some Type 2 charstrings omit the vstemhm operator before rmoveto, 
+		     even though this is only allowed before hintmask and cntrmask.
+		     Thanks to Felix Pahl.
+		   */
+		  type2_vstem(pcis, csp - 2, cstack);
+		  cstack [0] = csp [-1];
+		  cstack [1] = csp [ 0];
+		  csp = cstack + 1;
 		}
-		check_first_operator(csp > cstack + 1);
-		accum_xy(csp[-1], *csp);
+                code = t1_hinter__rmoveto(h, csp[-1], *csp);
 		goto move;
 	    case cx_hmoveto:
 		/* See vmoveto above re closing the subpath. */
-		if (pfont->PaintType != 1 && sppath != 0) {
-		    code = gx_path_close_subpath(sppath);
-		    if (code < 0)
-			return code;
-		}
 		check_first_operator(csp > cstack);
-		accum_x(*csp);
+                code = t1_hinter__rmoveto(h, *csp, 0);
 		goto move;
 	    case cx_vhcurveto:
 		vertical = true;
@@ -384,30 +379,19 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 	    case cx_hvcurveto:
 		vertical = false;
 	      hvc:for (ap = cstack; ap + 3 <= csp; vertical = !vertical, ap += 4) {
-		    gs_fixed_point pt1, pt2;
-		    fixed ax0 = sppath->position.x - ptx;
-		    fixed ay0 = sppath->position.y - pty;
-
-		    if (vertical)
-			accum_y(ap[0]);
-		    else
-			accum_x(ap[0]);
-		    pt1.x = ptx + ax0, pt1.y = pty + ay0;
-		    accum_xy(ap[1], ap[2]);
-		    pt2.x = ptx, pt2.y = pty;
+		    gs_fixed_point pt[2] = {{0, 0}, {0, 0}};
 		    if (vertical) {
+			pt[0].y = ap[0];
+			pt[1].x = ap[3];
 			if (ap + 4 == csp)
-			    accum_xy(ap[3], ap[4]);
-			else
-			    accum_x(ap[3]);
+			    pt[1].y = ap[4];
 		    } else {
+			pt[0].x = ap[0];
 			if (ap + 4 == csp)
-			    accum_xy(ap[4], ap[3]);
-			else
-			    accum_y(ap[3]);
+			    pt[1].x = ap[4];
+			pt[1].y = ap[3];
 		    }
-		    code = gx_path_add_curve(sppath, pt1.x, pt1.y,
-					     pt2.x, pt2.y, ptx, pty);
+		    code = t1_hinter__rcurveto(h, pt[0].x, pt[0].y, ap[1], ap[2], pt[1].x, pt[1].y);
 		    if (code < 0)
 			return code;
 		}
@@ -430,18 +414,20 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		    deltas = base + n - 1;
 		    for (j = 0; j < n; j++, base++, deltas += k - 1)
 			for (i = 1; i < k; i++)
-			    *base += deltas[i] * pfont->data.WeightVector.values[i];
+			    *base += (fixed)(deltas[i] * 
+				pfont->data.WeightVector.values[i]);
 		}
 		cnext;
 	    case c2_hstemhm:
-		pcis->have_hintmask = true;
 	      hstem:check_first_operator(!((csp - cstack) & 1));
-		apply_path_hints(pcis, false);
 		{
 		    fixed x = 0;
 
-		    for (ap = cstack; ap + 1 <= csp; x += ap[1], ap += 2)
-			type1_hstem(pcis, x += ap[0], ap[1]);
+		    for (ap = cstack; ap + 1 <= csp; x += ap[1], ap += 2) {
+			    code = t1_hinter__hstem(h, x += ap[0], ap[1]);
+			    if (code < 0)
+				return code;
+		    }
 		}
 		pcis->num_hints += (csp + 1 - cstack) >> 1;
 		cnext;
@@ -450,12 +436,8 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		 * A hintmask at the beginning of the CharString is
 		 * equivalent to vstemhm + hintmask.  For simplicity, we use
 		 * this interpretation everywhere.
-		 *
-		 * Even though the Adobe documentation doesn't say this,
-		 * it appears that the same holds true for cntrmask.
 		 */
 	    case c2_cntrmask:
-		pcis->have_hintmask = true;
 		check_first_operator(!((csp - cstack) & 1));
 		type2_vstem(pcis, csp, cstack);
 		/*
@@ -469,8 +451,6 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		    byte mask[max_total_stem_hints / 8];
 		    int i;
 
-		    if_debug3('1', "[1]mask[%d:%dh,%dv]", pcis->num_hints,
-			  pcis->hstem_hints.count, pcis->vstem_hints.count);
 		    for (i = 0; i < pcis->num_hints; ++cip, i += 8) {
 			charstring_next(*cip, state, mask[i >> 3], encrypted);
 			if_debug1('1', " 0x%02x", mask[i >> 3]);
@@ -482,35 +462,33 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 			/****** NYI ******/
 		    } else {	/* hintmask or equivalent */
 			if_debug0('1', "[1]hstem hints:\n");
-			enable_hints(&pcis->hstem_hints, mask);
 			if_debug0('1', "[1]vstem hints:\n");
-			enable_hints(&pcis->vstem_hints, mask);
+			code = t1_hinter__hint_mask(h, mask);
+			if (code < 0)
+			    return code;
 		    }
 		}
 		break;
 	    case c2_vstemhm:
-		pcis->have_hintmask = true;
 	      vstem:check_first_operator(!((csp - cstack) & 1));
 		type2_vstem(pcis, csp, cstack);
 		cnext;
 	    case c2_rcurveline:
 		for (ap = cstack; ap + 5 <= csp; ap += 6) {
-		    code = gs_op1_rrcurveto(&s, ap[0], ap[1], ap[2], ap[3],
+		    code = t1_hinter__rcurveto(h, ap[0], ap[1], ap[2], ap[3],
 					    ap[4], ap[5]);
 		    if (code < 0)
 			return code;
 		}
-		accum_xy(ap[0], ap[1]);
-		code = gx_path_add_line(sppath, ptx, pty);
+		code = t1_hinter__rlineto(h, ap[0], ap[1]);
 		goto cc;
 	    case c2_rlinecurve:
 		for (ap = cstack; ap + 7 <= csp; ap += 2) {
-		    accum_xy(ap[0], ap[1]);
-		    code = gx_path_add_line(sppath, ptx, pty);
+		    code = t1_hinter__rlineto(h, ap[0], ap[1]);
 		    if (code < 0)
 			return code;
 		}
-		code = gs_op1_rrcurveto(&s, ap[0], ap[1], ap[2], ap[3],
+		code = t1_hinter__rcurveto(h, ap[0], ap[1], ap[2], ap[3],
 					ap[4], ap[5]);
 		goto cc;
 	    case c2_vvcurveto:
@@ -520,7 +498,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		    fixed dxa = (n & 1 ? *ap++ : 0);
 
 		    for (; ap + 3 <= csp; ap += 4) {
-			code = gs_op1_rrcurveto(&s, dxa, ap[0], ap[1], ap[2],
+			code = t1_hinter__rcurveto(h, dxa, ap[0], ap[1], ap[2],
 						fixed_0, ap[3]);
 			if (code < 0)
 			    return code;
@@ -535,7 +513,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		    fixed dya = (n & 1 ? *ap++ : 0);
 
 		    for (; ap + 3 <= csp; ap += 4) {
-			code = gs_op1_rrcurveto(&s, ap[0], dya, ap[1], ap[2],
+			code = t1_hinter__rcurveto(h, ap[0], dya, ap[1], ap[2],
 						ap[3], fixed_0);
 			if (code < 0)
 			    return code;
@@ -551,13 +529,14 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		    ++cip;
 		    charstring_next(*cip, state, c2, encrypted);
 		    ++cip;
+		    CS_CHECK_PUSH(csp, cstack);
 		    *++csp = int2fixed((((c1 ^ 0x80) - 0x80) << 8) + c2);
 		}
 		goto pushed;
 	    case c2_callgsubr:
 		c = fixed2int_var(*csp) + pdata->gsubrNumberBias;
 		code = pdata->procs.subr_data
-		    (pfont, c, true, &ipsp[1].char_string);
+		    (pfont, c, true, &ipsp[1].cs_data);
 		goto subr;
 	    case cx_escape:
 		charstring_next(*cip, state, c, encrypted);
@@ -652,6 +631,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 			csp -= 3;
 			break;
 		    case ce2_random:
+			CS_CHECK_PUSH(csp, cstack);
 			++csp;
 			/****** NYI ******/
 			break;
@@ -659,9 +639,9 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 			{
 			    double prod = fixed2float(csp[-1]) * *csp;
 
-			    csp[-1] =
+			    csp[-1] = 
 				(prod > max_fixed ? max_fixed :
-				 prod < min_fixed ? min_fixed : prod);
+				 prod < min_fixed ? min_fixed : (fixed)prod);
 			}
 			--csp;
 			break;
@@ -670,6 +650,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 			    *csp = float2fixed(sqrt(fixed2float(*csp)));
 			break;
 		    case ce2_dup:
+			CS_CHECK_PUSH(csp, cstack);
 			csp[1] = *csp;
 			++csp;
 			break;
@@ -750,20 +731,19 @@ flex:			{
 				flex_depth = join.y;
 			    if (fabs(flex_depth) < fixed2float(*csp)) {
 				/* Do flex as line. */
-				accum_xy(x_end, y_end);
-				code = gx_path_add_line(sppath, ptx, pty);
+				code = t1_hinter__rlineto(h, x_end, y_end);
 			    } else {
 				/*
 				 * Do flex as curve.  We can't jump to rrc,
 				 * because the flex operators don't clear
 				 * the stack (!).
 				 */
-				code = gs_op1_rrcurveto(&s,
+				code = t1_hinter__rcurveto(h, 
 					csp[-12], csp[-11], csp[-10],
 					csp[-9], csp[-8], csp[-7]);
 				if (code < 0)
 				    return code;
-				code = gs_op1_rrcurveto(&s,
+				code = t1_hinter__rcurveto(h, 
 					csp[-6], csp[-5], csp[-4],
 					csp[-3], csp[-2], csp[-1]);
 			    }
