@@ -14,8 +14,9 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxcht.c,v 1.3 2002/04/23 11:58:45 easysw Exp $ */
+/*$Id: gxcht.c,v 1.11.2.1 2004/04/03 10:55:14 igor Exp $ */
 /* Color halftone rendering for Ghostscript imaging library */
+#include <assert.h>
 #include "memory_.h"
 #include "gx.h"
 #include "gserrors.h"
@@ -28,6 +29,7 @@
 #include "gxdcolor.h"
 #include "gxistate.h"
 #include "gzht.h"
+#include "gsserial.h"
 
 /* Define whether to force use of the slow code, for testing. */
 #define USE_SLOW_CODE 0
@@ -51,43 +53,457 @@
 /* Define the colored halftone device color type. */
 gs_private_st_ptrs1(st_dc_ht_colored, gx_device_color, "dc_ht_colored",
     dc_ht_colored_enum_ptrs, dc_ht_colored_reloc_ptrs, colors.colored.c_ht);
+private dev_color_proc_save_dc(gx_dc_ht_colored_save_dc);
+private dev_color_proc_get_dev_halftone(gx_dc_ht_colored_get_dev_halftone);
 private dev_color_proc_load(gx_dc_ht_colored_load);
 private dev_color_proc_fill_rectangle(gx_dc_ht_colored_fill_rectangle);
 private dev_color_proc_equal(gx_dc_ht_colored_equal);
+private dev_color_proc_write(gx_dc_ht_colored_write);
+private dev_color_proc_read(gx_dc_ht_colored_read);
 const gx_device_color_type_t gx_dc_type_data_ht_colored = {
     &st_dc_ht_colored,
+    gx_dc_ht_colored_save_dc, gx_dc_ht_colored_get_dev_halftone,
+    gx_dc_ht_get_phase,
     gx_dc_ht_colored_load, gx_dc_ht_colored_fill_rectangle,
-    gx_dc_default_fill_masked, gx_dc_ht_colored_equal
+    gx_dc_default_fill_masked, gx_dc_ht_colored_equal,
+    gx_dc_ht_colored_write, gx_dc_ht_colored_read,
+    gx_dc_ht_colored_get_nonzero_comps
 };
 #undef gx_dc_type_ht_colored
 const gx_device_color_type_t *const gx_dc_type_ht_colored =
     &gx_dc_type_data_ht_colored;
 #define gx_dc_type_ht_colored (&gx_dc_type_data_ht_colored)
 
+/* save information about the operand device color */
+private void
+gx_dc_ht_colored_save_dc(const gx_device_color * pdevc,
+			 gx_device_color_saved * psdc)
+{
+    psdc->type = pdevc->type;
+    memcpy( psdc->colors.colored.c_base,
+            pdevc->colors.colored.c_base,
+            sizeof(psdc->colors.colored.c_base) );
+    memcpy( psdc->colors.colored.c_level,
+            pdevc->colors.colored.c_level,
+            sizeof(psdc->colors.colored.c_base) );
+    psdc->colors.colored.alpha = pdevc->colors.colored.alpha;
+    psdc->phase = pdevc->phase;
+}
+
+/* get the halftone used for the operand device color */
+private const gx_device_halftone *
+gx_dc_ht_colored_get_dev_halftone(const gx_device_color * pdevc)
+{
+    return pdevc->colors.colored.c_ht;
+}
+
 /* Compare two colored halftones for equality. */
 private bool
 gx_dc_ht_colored_equal(const gx_device_color * pdevc1,
 		       const gx_device_color * pdevc2)
 {
-    uint num_comp, m, i;
+    uint num_comp = pdevc1->colors.colored.num_components;
 
     if (pdevc2->type != pdevc1->type ||
 	pdevc1->colors.colored.c_ht != pdevc2->colors.colored.c_ht ||
 	pdevc1->colors.colored.alpha != pdevc2->colors.colored.alpha ||
-	pdevc1->colors.colored.plane_mask != pdevc2->colors.colored.plane_mask ||
 	pdevc1->phase.x != pdevc2->phase.x ||
-	pdevc1->phase.y != pdevc2->phase.y
+	pdevc1->phase.y != pdevc2->phase.y ||
+        num_comp != pdevc2->colors.colored.num_components
 	)
 	return false;
-    for (m = pdevc1->colors.colored.plane_mask, i = 0; m != 0; m >>=1, i++)
-        if(m & 1)
-            if (pdevc1->colors.colored.c_base[i]  != pdevc2->colors.colored.c_base[i] ||
-                pdevc1->colors.colored.c_level[i] != pdevc2->colors.colored.c_level[i])
-                return false;
+    return
+	!memcmp(pdevc1->colors.colored.c_base,
+		pdevc2->colors.colored.c_base,
+		num_comp * sizeof(pdevc1->colors.colored.c_base[0])) &&
+	!memcmp(pdevc1->colors.colored.c_level,
+		pdevc2->colors.colored.c_level,
+		num_comp * sizeof(pdevc1->colors.colored.c_level[0]));
 }
 
-/* Define an abbreviation for a heavily used value. */
+
+/*
+ * Flags to indicate the pieces of a colored halftone that are included
+ * in its string representation. The first byte of the string holds this
+ * set of flags.
+ *
+ * The case alpha = gx_max_color_value is by far the most common, so
+ * special treatment is provided for this case.
+ *
+ * The halftone is never transmitted as part of a device color, so there
+ * is no flag for it.
+ */
+private const int   dc_ht_colored_has_base = 0x01;
+private const int   dc_ht_colored_has_level = 0x02;
+private const int   dc_ht_colored_has_alpha = 0x04;
+private const int   dc_ht_colored_alpha_is_max = 0x08;
+
+/*
+ * Serialize a device color that uses a traditional colored halftone.
+ *
+ * The number of components of a device color must match that of the
+ * process color model, so it is not transmitted.
+ *
+ * The most common situation in which this routine is used is for 1-bit
+ * per component devices. In that case, base[i] will always be 0 or 1,
+ * and thus may be fit in a single bit.
+ *
+ * In many situations, one or more of the color component intensity
+ * levels will be 0. The plane_mask field identifies those components
+ * where this is not the case. By tansmitting the plane_mask, only those
+ * components with non-zero levels need be transmitted.
+ *
+ * The case alpha = gx_max_color_value is by far the most common, so
+ * special treatment is provided for this case.
+ *
+ *
+ * Operands:
+ *
+ *  pdevc       pointer to device color to be serialized
+ *
+ *  psdc        pointer ot saved version of last serialized color (for
+ *              this band)
+ *  
+ *  dev         pointer to the current device, used to retrieve process
+ *              color model information
+ *
+ *  pdata       pointer to buffer in which to write the data
+ *
+ *  psize       pointer to a location that, on entry, contains the size of
+ *              the buffer pointed to by pdata; on return, the size of
+ *              the data required or actually used will be written here.
+ *
+ * Returns:
+ *  1, with *psize set to 0, if *psdc and *pdevc represent the same color
+ *
+ *  0, with *psize set to the amount of data written, if everything OK
+ *
+ *  gs_error_rangecheck, with *psize set to the size of buffer required,
+ *  if *psize was not large enough
+ *
+ *  < 0, != gs_error_rangecheck, in the event of some other error
+ *  (currently none); in this case *psize is not changed.
+ */
+private int
+gx_dc_ht_colored_write(
+    const gx_device_color *         pdevc,
+    const gx_device_color_saved *   psdc0,
+    const gx_device *               dev,
+    byte *                          pdata,
+    uint *                          psize )
+{
+    int                             req_size = 1;   
+    int                             flag_bits = 0;
+    int                             num_comps = dev->color_info.num_components;
+    int                             depth = dev->color_info.depth;
+    gx_color_index                  plane_mask = pdevc->colors.colored.plane_mask;
+    gx_color_value                  alpha = pdevc->colors.colored.alpha;
+    const gx_device_color_saved *   psdc = psdc0;
+    byte *                          pdata0 = pdata;
+
+    /* sanity check */
+    assert(pdevc->colors.colored.num_components == num_comps);
+
+    /* check if saved color is of the same type */
+    if (psdc != 0 && psdc->type != pdevc->type)
+        psdc = 0;
+
+    /* calculate the size required */
+    if ( psdc == 0                                                         ||
+         memcmp( pdevc->colors.colored.c_base,
+                 psdc->colors.colored.c_base,
+                 num_comps * sizeof(pdevc->colors.colored.c_base[0]) ) != 0  ) {
+        flag_bits |= dc_ht_colored_has_base;
+        if (num_comps == depth)     /* 1 bit / component */
+            req_size += (num_comps + 7) >> 3;
+        else
+            req_size += num_comps * sizeof(pdevc->colors.colored.c_base[0]);
+    }
+
+    plane_mask = pdevc->colors.colored.plane_mask;
+    if ( psdc == 0                                                          ||
+         memcmp( pdevc->colors.colored.c_level,
+                 psdc->colors.colored.c_level,
+                 num_comps * sizeof(pdevc->colors.colored.c_level[0]) ) != 0  ) {
+        gx_color_index  comp_bit;
+        int             i;
+        uint            tmp_mask;
+
+        flag_bits |= dc_ht_colored_has_level;
+        if (num_comps > 8 * sizeof(uint)) {
+            tmp_mask = (uint)plane_mask;
+            req_size += enc_u_sizew(tmp_mask);
+            tmp_mask = (uint)(plane_mask >> (8 * sizeof(uint)));
+            req_size += enc_u_sizew(tmp_mask);
+        } else {
+            tmp_mask = (uint)plane_mask;
+            req_size += enc_u_sizew(tmp_mask);
+        }
+        for (i = 0, comp_bit = 0x1; i < num_comps; i++, comp_bit <<= 1) {
+            if ((plane_mask & comp_bit) != 0)
+                req_size += enc_u_sizew(pdevc->colors.colored.c_level[i]);
+        }
+    }
+
+    if (psdc == 0 || alpha != psdc->colors.colored.alpha) {
+        if (alpha == gx_max_color_value)
+            flag_bits |= dc_ht_colored_alpha_is_max;
+        else {
+            flag_bits |= dc_ht_colored_has_alpha;
+            req_size += enc_u_sizew(alpha);
+        }
+    }
+
+    /* see if there is anything to do */
+    if (flag_bits == 0) {
+        *psize = 0;
+        return 1;
+    }
+
+    /* see if enough space is available */
+    if (req_size > *psize) {
+        *psize = req_size;
+        return gs_error_rangecheck;
+    }
+
+    /* write out the flag byte */
+    *pdata++ = (byte)flag_bits;
+
+    /* write out such other parts of the device color as required */
+    if ((flag_bits & dc_ht_colored_has_base) != 0) {
+        if (num_comps == depth) {
+            gx_color_index  base_mask = 0;
+            int             num_bytes = (num_comps + 7) >> 3;
+            int             i;
+
+            for (i = 0; i < num_comps; i++) {
+                if (pdevc->colors.colored.c_base[i] != 0)
+                    base_mask |= (gx_color_index)1 << i;
+            }
+            for (i = 0; i < num_bytes; i++, base_mask >>= 8)
+                *pdata++ = (byte)base_mask;
+        } else {
+            memcpy( pdata,
+                    pdevc->colors.colored.c_base,
+                    num_comps * sizeof(pdevc->colors.colored.c_base[0]) );
+            pdata += num_comps * sizeof(pdevc->colors.colored.c_base[0]);
+        }
+    }
+
+    if ((flag_bits & dc_ht_colored_has_level) != 0) {
+        gx_color_index  code_bit;
+        int             i;
+        uint            tmp_mask;
+
+        if (num_comps > 8 * sizeof(uint)) {
+            tmp_mask = (uint)plane_mask;
+            enc_u_putw(tmp_mask, pdata);
+            tmp_mask = (uint)(plane_mask >> (8 * sizeof(uint)));
+            enc_u_putw(tmp_mask, pdata);
+        } else {
+            tmp_mask = (uint)plane_mask;
+            enc_u_putw(tmp_mask, pdata);
+        }
+        for (i = 0, code_bit = 0x1; i < num_comps; i++, code_bit <<= 1) {
+            if ((plane_mask & code_bit) != 0)
+                enc_u_putw(pdevc->colors.colored.c_level[i], pdata);
+        }
+    }
+
+    if ((flag_bits & dc_ht_colored_has_alpha) != 0)
+        enc_u_putw(alpha, pdata);
+
+    *psize = pdata - pdata0;
+    return 0;
+}
+
+/*
+ * Reconstruct a device color from its serial representation.
+ *
+ * Operands:
+ *
+ *  pdevc       pointer to the location in which to write the
+ *              reconstructed device color
+ *
+ *  pis         pointer to the current imager state (to access the
+ *              current halftone)
+ *
+ *  prior_devc  pointer to the current device color (this is provided
+ *              separately because the device color is not part of the
+ *              imager state)
+ *
+ *  dev         pointer to the current device, used to retrieve process
+ *              color model information
+ *
+ *  pdata       pointer to the buffer to be read
+ *
+ *  size        size of the buffer to be read; this should be large
+ *              enough to hold the entire color description
+ *
+ *  mem         pointer to the memory to be used for allocations
+ *              (ignored here)
+ *
+ * Returns:
+ *
+ *  # of bytes read if everthing OK, < 0 in the event of an error
+ */
+private int
+gx_dc_ht_colored_read(
+    gx_device_color *       pdevc,
+    const gs_imager_state * pis,
+    const gx_device_color * prior_devc,
+    const gx_device *       dev,
+    const byte *            pdata,
+    uint                    size,
+    gs_memory_t *           mem )       /* ignored */
+{
+    gx_device_color         devc;
+    int                     num_comps = dev->color_info.num_components;
+    int                     depth = dev->color_info.depth;
+    const byte *            pdata0 = pdata;
+    int                     flag_bits;
+
+    /* if prior information is available, use it */
+    if (prior_devc != 0 && prior_devc->type == gx_dc_type_ht_colored)
+        devc = *prior_devc;
+    else
+        memset(&devc, 0, sizeof(devc));   /* clear pointers */
+    devc.type = gx_dc_type_ht_colored;
+
+    /* the number of components is determined by the color model */
+    devc.colors.colored.num_components = num_comps;
+    devc.colors.colored.c_ht = pis->dev_ht;
+
+    /*
+     * Verify that we have at least the flag bits. For performance
+     * reasons, the routines that convert serialized representations
+     * of integers do not check buffer size. Hence, in many cases below,
+     * only a very rough check is made to verify that we have not
+     * exhausted the buffer. This should not cause a problem in
+     * practice.
+     */
+    if (size == 0)
+        return_error(gs_error_rangecheck);
+    size--;
+    flag_bits = *pdata++;
+
+    /* read the other components provided */
+    if ((flag_bits & dc_ht_colored_has_base) != 0) {
+        if (depth == num_comps) {
+            gx_color_index  base_mask = 0;
+            int             num_bytes = (num_comps + 7) >> 3;
+            int             i, shift = 0;
+
+            if (size < num_bytes)
+                return_error(gs_error_rangecheck);
+	    size -= num_bytes;
+            for (i = 0; i < num_bytes; i++, shift += 8)
+                base_mask |= (gx_color_index)(*pdata++) << shift;
+            for (i = 0; i < num_comps; i++, base_mask >>= 1)
+                devc.colors.colored.c_base[i] = base_mask & 0x1;
+        } else {
+            if (size < num_comps)
+                return_error(gs_error_rangecheck);
+	    size -= num_comps;
+            memcpy(devc.colors.colored.c_base, pdata, num_comps);
+            pdata += num_comps;
+        }
+    }
+
+    if ((flag_bits & dc_ht_colored_has_level) != 0) {
+        const byte *    pdata_start = pdata;
+        gx_color_index  plane_mask;
+        uint            tmp_mask;
+        int             i;
+
+        if (size < 1)
+            return_error(gs_error_rangecheck);
+
+        if (num_comps > 8 * sizeof(uint)) {
+            enc_u_getw(tmp_mask, pdata);
+            plane_mask = (gx_color_index)tmp_mask;
+            enc_u_getw(tmp_mask, pdata);
+            plane_mask = (gx_color_index)tmp_mask << (8 * sizeof(uint));
+        } else {
+            enc_u_getw(tmp_mask, pdata);
+            plane_mask = (gx_color_index)tmp_mask;
+        }
+        devc.colors.colored.plane_mask = plane_mask;
+        for (i = 0; i < num_comps; i++, plane_mask >>= 1) {
+            if ((plane_mask & 0x1) != 0) {
+                if (size - (pdata - pdata_start) < 1)
+                    return_error(gs_error_rangecheck);
+                enc_u_getw(devc.colors.colored.c_level[i], pdata);
+            } else
+                devc.colors.colored.c_level[i] = 0;
+        }
+        size -= pdata - pdata_start;
+    }
+
+    if ((flag_bits & dc_ht_colored_alpha_is_max) != 0)
+        devc.colors.colored.alpha = gx_max_color_value;
+    else if ((flag_bits & dc_ht_colored_has_alpha) != 0) {
+        const byte *    pdata_start = pdata;
+
+        if (size < 1)
+            return_error(gs_error_rangecheck);
+        enc_u_getw(devc.colors.colored.alpha, pdata);
+        size -= pdata - pdata_start;
+    }
+
+    /* set the phase as required (select value is arbitrary) */
+    color_set_phase_mod( &devc,
+                         pis->screen_phase[0].x,
+                         pis->screen_phase[0].y,
+                         pis->dev_ht->lcm_width,
+                         pis->dev_ht->lcm_height );
+
+    /* everything looks OK */
+    *pdevc = devc;
+    return pdata - pdata0;
+}
+
+
+/*
+ * Get the nonzero components of a coloredhalftone. This is used to
+ * distinguish components that are given zero intensity due to halftoning
+ * from those for which the original color intensity was in fact zero.
+ *
+ * An original component intensity of zero will yield a c_base value of
+ * 0 and a c_level of 0. The plane_mask field already contains the latter
+ * information, so we need only add those components for which c_base is
+ * non-zero.
+ */
+int
+gx_dc_ht_colored_get_nonzero_comps(
+    const gx_device_color * pdevc,
+    const gx_device *       dev_ignored,
+    gx_color_index *        pcomp_bits )
+{
+    int                     i, ncomps =  pdevc->colors.colored.num_components;
+    gx_color_index          comp_bits = pdevc->colors.colored.plane_mask;
+
+    for (i = 0; i < ncomps; i++) {
+        if (pdevc->colors.colored.c_base[i] != 0)
+            comp_bits |= ((gx_color_index)1) << i;
+    }
+    *pcomp_bits = comp_bits;
+
+    return 0;
+}
+
+/*
+ * Define an abbreviation for a heavily used value: the maximum number of
+ * of device colorants (device colors).
+ */
 #define MAX_DCC GX_DEVICE_COLOR_MAX_COMPONENTS
+/*
+ * Define a size for the "colors" array.  For less than 5 colors, there are
+ * 2**n values stored (for a maximum of 16 values).  For 5 or more colors, we
+ * only store 2 values per color so the array size can be 2 * MAX_DCC.  Use which
+ * ever is larger for the array size.
+ */
+#define MAX_DCC_16 (2 * MAX_DCC < 16 ? 16 : 2 * MAX_DCC)
 
 /* Forward references. */
 /* Use a typedef to attempt to work around overly picky compilers. */
@@ -96,7 +512,7 @@ typedef struct color_values_pair_s {
     gx_color_value_array values[2];
 } color_values_pair_t;
 #define SET_HT_COLORS_PROC(proc)\
-  int proc(P7(\
+  int proc(\
 	      color_values_pair_t *pvp,\
 	      gx_color_index colors[1 << MAX_DCC],\
 	      const gx_const_strip_bitmap *sbits[MAX_DCC],\
@@ -104,14 +520,14 @@ typedef struct color_values_pair_s {
 	      gx_device *dev,\
 	      gx_ht_cache *caches[MAX_DCC],\
 	      int nplanes\
-	      ))
+	      )
 
 private SET_HT_COLORS_PROC(set_ht_colors_le_4);
 private SET_HT_COLORS_PROC(set_cmyk_1bit_colors);
 private SET_HT_COLORS_PROC(set_ht_colors_gt_4);
 
 #define SET_COLOR_HT_PROC(proc)\
-  void proc(P14(\
+  void proc(\
 	        byte *dest_data, /* the output tile */\
 		uint dest_raster, /* ibid. */\
 		int px,	/* the initial phase of the output tile */\
@@ -124,11 +540,11 @@ private SET_HT_COLORS_PROC(set_ht_colors_gt_4);
 		gx_color_index plane_mask,	/* which planes are halftoned */\
 		gx_device *dev,		/* in case we are mapping lazily */\
 		const color_values_pair_t *pvp,	/* color values ditto */\
-		gx_color_index colors[1 << MAX_DCC],	/* the actual colors for the tile, */\
-				/* actually [1 << nplanes] */\
+		gx_color_index colors[MAX_DCC],	/* the actual colors for the tile, */\
+				/* actually [nplanes] */\
 		const gx_const_strip_bitmap * sbits[MAX_DCC]	/* the bitmaps for the planes, */\
 				/* actually [nplanes] */\
-		))
+		)
 
 private SET_COLOR_HT_PROC(set_color_ht_le_4);
 private SET_COLOR_HT_PROC(set_color_ht_gt_4);
@@ -138,15 +554,8 @@ private int
 gx_dc_ht_colored_load(gx_device_color * pdevc, const gs_imager_state * pis,
 		      gx_device * ignore_dev, gs_color_select_t select)
 {
-    gx_device_halftone *pdht = pdevc->colors.colored.c_ht;
-    gx_ht_cache *pcache = pis->ht_cache;
-    gx_ht_order *porder =
-	(pdht->components ? &pdht->components[0].corder : &pdht->order);
-
-    if (pcache->order.bit_data != porder->bit_data)
-	gx_ht_init_cache(pcache, porder);
-    /* Set the cache pointers in the default order. */
-    pdht->order.cache = porder->cache = pcache;
+    /* TO_DO_DEVICEN */
+ 
     return 0;
 }
 
@@ -165,15 +574,17 @@ gx_dc_ht_colored_fill_rectangle(const gx_device_color * pdevc,
     const gx_device_halftone *pdht = pdevc->colors.colored.c_ht;
     int depth = dev->color_info.depth;
     int nplanes = dev->color_info.num_components;
+
     SET_HT_COLORS_PROC((*set_ht_colors)) =
 	(
 #if USE_SLOW_CODE
 	 set_ht_colors_gt_4
 #else
-	 dev_proc(dev, map_cmyk_color) == cmyk_1bit_map_cmyk_color ?
-	   set_cmyk_1bit_colors :
-	 nplanes < 4 ? set_ht_colors_le_4 :
-	 set_ht_colors_gt_4
+	 (dev_proc(dev, map_cmyk_color) == gx_default_encode_color &&
+	  dev->color_info.depth == 4) ?
+	    set_cmyk_1bit_colors :
+	    nplanes <= 4 ? set_ht_colors_le_4 :
+			  set_ht_colors_gt_4
 #endif
 	 );
     SET_COLOR_HT_PROC((*set_color_ht)) =
@@ -185,7 +596,7 @@ gx_dc_ht_colored_fill_rectangle(const gx_device_color * pdevc,
 #endif
 	 set_color_ht_gt_4);
     color_values_pair_t vp;
-    gx_color_index colors[1 << MAX_DCC];
+    gx_color_index colors[MAX_DCC_16];
     const gx_const_strip_bitmap *sbits[MAX_DCC];
     gx_ht_cache *caches[MAX_DCC];
     int special;
@@ -231,12 +642,9 @@ gx_dc_ht_colored_fill_rectangle(const gx_device_color * pdevc,
     } else {
 	gx_ht_order_component *pocs = pdht->components;
 
-	caches[0] = pocs[pdht->color_indices[0]].corder.cache;
-	caches[1] = pocs[pdht->color_indices[1]].corder.cache;
-	caches[2] = pocs[pdht->color_indices[2]].corder.cache;
-	caches[3] = pocs[pdht->color_indices[3]].corder.cache;
-	for (i = 4; i < nplanes; ++i)
-	    caches[i] = pocs[pdht->color_indices[i]].corder.cache;
+
+	for (i = 0; i < nplanes; ++i)
+	    caches[i] = pocs[i].corder.cache;
     }
     special = set_ht_colors(&vp, colors, sbits, pdevc, dev, caches, nplanes);
     no_rop = source == NULL && lop_no_S_is_T(lop);
@@ -387,10 +795,11 @@ private const gx_const_strip_bitmap ht_no_bitmap = {
 
 /* Set the color value(s) and halftone mask for one plane. */
 
-/* Free variables: pvp, pdc, sbits */
+/* Free variables: pvp, pdc, sbits, max_color */
 #define SET_PLANE_COLOR_CONSTANT(i)\
   BEGIN\
-    pvp->values[1][i] = pvp->values[0][i] = pdc->colors.colored.c_base[i];\
+    pvp->values[1][i] = pvp->values[0][i] = \
+	fractional_color(pdc->colors.colored.c_base[i], max_color);\
     sbits[i] = &ht_no_bitmap;\
   END
 
@@ -411,7 +820,7 @@ private const gx_const_strip_bitmap ht_no_bitmap = {
 	const gx_device_halftone *pdht = pdc->colors.colored.c_ht;  \
 	int nlevels =\
 	    (pdht->components ?\
-	     pdht->components[pdht->color_indices[i]].corder.num_levels :\
+	     pdht->components[i].corder.num_levels :\
 	     pdht->order.num_levels);\
 \
 	pvp->values[1][i] = pvp->values[0][i];                   \
@@ -424,12 +833,13 @@ private const gx_const_strip_bitmap ht_no_bitmap = {
 /* Set up the colors and the individual plane halftone bitmaps. */
 private int
 set_ht_colors_le_4(color_values_pair_t *pvp /* only used internally */,
-		   gx_color_index colors[1 << MAX_DCC],
+		   gx_color_index colors[MAX_DCC_16],
 		   const gx_const_strip_bitmap * sbits[MAX_DCC],
 		   const gx_device_color * pdc, gx_device * dev,
 		   gx_ht_cache * caches[MAX_DCC], int nplanes)
 {
     gx_color_value max_color = dev->color_info.dither_colors - 1;
+    gx_color_value cvalues[4];
     /*
      * NB: the halftone orders are all set up for an additive color space.
      *     To make these work with a subtractive device space such as CMYK,
@@ -438,46 +848,36 @@ set_ht_colors_le_4(color_values_pair_t *pvp /* only used internally */,
      *     space, this will reverse (in an approximate sense) the color
      *     conversion performed to express the color in the device space.
      */
-    bool invert = dev->color_info.num_components >= 4;  /****** HACK ******/
+    bool invert = dev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE;
 
     SET_PLANE_COLOR(0);
-    SET_PLANE_COLOR(1);
-    SET_PLANE_COLOR(2);
+    if (nplanes >= 2) {
+        SET_PLANE_COLOR(1);
+    }
+    if (nplanes >= 3) {
+        SET_PLANE_COLOR(2);
+    }
     if (nplanes == 3) {
 	gx_color_value alpha = pdc->colors.colored.alpha;
 
 	if (alpha == gx_max_color_value) {
-#ifndef DEBUG
-#  undef gx_map_rgb_color
-	    dev_proc_map_rgb_color((*gx_map_rgb_color)) =
-		dev_proc(dev, map_rgb_color);
-#endif
 #define M(i)\
-  colors[i] = gx_map_rgb_color(dev, pvp->values[(i) & 1][0],\
-			       pvp->values[((i) & 2) >> 1][1],\
-			       pvp->values[(i) >> 2][2])
+  cvalues[0] = pvp->values[(i) & 1][0];\
+  cvalues[1] = pvp->values[((i) & 2) >> 1][1];\
+  cvalues[2] = pvp->values[(i) >> 2][2];\
+  colors[i] = dev_proc(dev, encode_color)(dev, cvalues);
+
 	    M(0); M(1); M(2); M(3); M(4); M(5); M(6); M(7);
 #undef M
 	} else {
-#ifndef DEBUG
-#  undef gx_map_rgb_alpha_color
-	    dev_proc_map_rgb_alpha_color((*gx_map_rgb_alpha_color)) =
-		dev_proc(dev, map_rgb_alpha_color);
-#endif
 #define M(i)\
-  colors[i] = gx_map_rgb_alpha_color(dev, pvp->values[(i) & 1][0],\
+  colors[i] = dev_proc(dev, map_rgb_alpha_color)(dev, pvp->values[(i) & 1][0],\
 				     pvp->values[((i) & 2) >> 1][1],\
 				     pvp->values[(i) >> 2][2], alpha)
 	    M(0); M(1); M(2); M(3); M(4); M(5); M(6); M(7);
 #undef M
 	}
-    } else {
-#ifdef DEBUG
-#  define do_map_cmyk_color(dev, c, m, y, k) gx_map_cmyk_color(dev, c, m, y, k)
-#else
-	dev_proc_map_cmyk_color((*do_map_cmyk_color)) =
-	    dev_proc(dev, map_cmyk_color);
-#endif
+    } else if (nplanes > 3){
 	SET_PLANE_COLOR(3);
 	if (nplanes > 4) {
 	    /*
@@ -497,12 +897,12 @@ set_ht_colors_le_4(color_values_pair_t *pvp /* only used internally */,
 	 * Each zero component can cut the cost of color mapping in
 	 * half, so it's worth doing a little checking here.
 	 */
-
 #define M(i)\
-  colors[i] = do_map_cmyk_color(dev, pvp->values[(i) & 1][0],\
-				pvp->values[((i) & 2) >> 1][1],\
-				pvp->values[((i) & 4) >> 2][2],\
-				pvp->values[(i) >> 3][3])
+    cvalues[0] = pvp->values[(i) & 1][0];\
+    cvalues[1] = pvp->values[((i) & 2) >> 1][1];\
+    cvalues[2] = pvp->values[((i) & 4) >> 2][2];\
+    cvalues[3] = pvp->values[(i) >> 3][3];\
+    colors[i] = dev_proc(dev, encode_color)(dev, cvalues)
 
       /* We know that plane_mask <= 15. */
 	switch ((int)pdc->colors.colored.plane_mask) {
@@ -550,7 +950,6 @@ c2:	    case 2:
 	M(0);
 
 #undef M
-
     }
     return 0;
 }
@@ -558,7 +957,7 @@ c2:	    case 2:
 /* Set up colors using the standard 1-bit CMYK mapping. */
 private int
 set_cmyk_1bit_colors(color_values_pair_t *ignore_pvp,
-		     gx_color_index colors[1 << MAX_DCC /*16 used*/],
+		     gx_color_index colors[MAX_DCC_16 /*16 used*/],
 		     const gx_const_strip_bitmap * sbits[MAX_DCC /*4 used*/],
 		     const gx_device_color * pdc, gx_device * dev,
 		     gx_ht_cache * caches[MAX_DCC /*4 used*/],
@@ -588,7 +987,7 @@ set_cmyk_1bit_colors(color_values_pair_t *ignore_pvp,
     } else {\
 	int nlevels =\
 	    (pdht->components ?\
-	     pdht->components[pdht->color_indices[i]].corder.num_levels :\
+	     pdht->components[i].corder.num_levels :\
 	     pdht->order.num_levels);\
 \
 	mask0 |= mask;\
@@ -601,6 +1000,7 @@ set_cmyk_1bit_colors(color_values_pair_t *ignore_pvp,
     SET_PLANE_COLOR_CMYK(1, 0x44444444);
     SET_PLANE_COLOR_CMYK(2, 0x22222222);
     SET_PLANE_COLOR_CMYK(3, 0x11111111);
+
 #undef SET_PLANE_COLOR_CMYK
     {
 	gx_ht_cache *ctemp;
@@ -614,39 +1014,48 @@ set_cmyk_1bit_colors(color_values_pair_t *ignore_pvp,
 }
 
 /*
- * Set up colors for >4 planes.  In this case, we compute the colors
- * themselves lazily.
+ * Set up colors for >4 planes.  In this case, we assume that the color
+ * component values are "separable".  (That we can form a gx_color_index value
+ * for a color by a bit wise or of the gx_color_index values of the individual
+ * components.)
  */
 private int
 set_ht_colors_gt_4(color_values_pair_t *pvp,
-		   gx_color_index colors[1 << MAX_DCC],
+		   gx_color_index colors[MAX_DCC_16],
 		   const gx_const_strip_bitmap * sbits[MAX_DCC],
 		   const gx_device_color * pdc, gx_device * dev,
 		   gx_ht_cache * caches[MAX_DCC], int nplanes)
 {
     gx_color_value max_color = dev->color_info.dither_colors - 1;
-    /* See set_ht_color_le_4 for invert. */
-    bool invert = true;  /****** HACK ******/
+    bool invert = dev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE;
     gx_color_index plane_mask = pdc->colors.colored.plane_mask;
     int i;
-    gx_color_index ci;
+    gx_color_value cv[MAX_DCC] = {0};
 
     /* Set the color values and halftone caches. */
     for (i = 0; i < nplanes; ++i)
 	if ((plane_mask >> i) & 1)
 	    SET_PLANE_COLOR(i);
-        else
+	else
 	    SET_PLANE_COLOR_CONSTANT(i);
 
     /*
-     * Clear the color table entries that will actually be used, namely,
-     * those whose indices are either 0 or 1 in bits selected by plane_mask
-     * and 0 in all other bits.
+     * Determine a gs_color_index value for each pair of component values.
+     * We assume that an overall index value can be formed from the
+     * bitwise or of each component.  We calculate a value for both
+     * the high and low value of each component.  These are stored
+     * in adjacent locations in 'colors'.
      */
-    ci = 0;
-    do {
-	colors[ci] = gx_no_color_index;
-    } while ((ci = ((ci | ~plane_mask) + 1) & plane_mask) != 0);
+    for (i = 0; i < nplanes; i++ ) {
+	cv[i] = pvp->values[0][i];
+	colors[2 * i] = dev_proc(dev, encode_color)(dev, cv);
+	/* We only need both values for components being halftoned */
+	if ((plane_mask >> i) & 1) {
+	    cv[i] = pvp->values[1][i];
+	    colors[2 * i + 1] = dev_proc(dev, encode_color)(dev, cv);
+	}
+	cv[i] = 0;
+    }
 
     return 0;
 }
@@ -689,7 +1098,7 @@ init_tile_cursor(int i, tile_cursor_t *ptc, const gx_const_strip_bitmap *btile,
     ptc->row = ptc->tdata + by * ptc->raster;
     ptc->data = ptc->row + ptc->xoffset;
     ptc->bit_shift = ptc->xshift;
-    if_debug6('h', "[h]plane %d: size=%d,%d shift=%d bx=%d by=%dn",
+    if_debug6('h', "[h]plane %d: size=%d,%d shift=%d bx=%d by=%d\n",
 	      i, tw, btile->size.y, btile->shift, bx, by);
 }
 
@@ -743,7 +1152,7 @@ set_color_ht_le_4(byte *dest_data, uint dest_raster, int px, int py,
 		  int w, int h, int depth, int special, int nplanes,
 		  gx_color_index plane_mask, gx_device *ignore_dev,
 		  const color_values_pair_t *ignore_pvp,
-		  gx_color_index colors[1 << MAX_DCC],
+		  gx_color_index colors[MAX_DCC_16],
 		  const gx_const_strip_bitmap * sbits[MAX_DCC])
 {
     /*
@@ -763,7 +1172,7 @@ set_color_ht_le_4(byte *dest_data, uint dest_raster, int px, int py,
 	    "\000\010\004\014\002\012\006\016\001\011\005\015\003\013\007\017"[plane_mask];
     }
     if_debug6('h',
-	      "[h]color_ht: x=%d y=%d w=%d h=%d plane_mask=0x%lu depth=%d\n",
+	      "[h]color_ht_le_4: x=%d y=%d w=%d h=%d plane_mask=0x%lu depth=%d\n",
 	      px, py, w, h, (ulong)plane_mask, depth);
 
     /* Do one-time cursor initialization. */
@@ -925,15 +1334,16 @@ set_color_ht_le_4(byte *dest_data, uint dest_raster, int px, int py,
 }
 
 /*
- * Render the combined halftone for nplanes > 4.  We haven't made any
- * attempt to optimize this code.
+ * Render the combined halftone for nplanes > 4.  This routine assumes
+ * that we can form a gx_color_index value by the bitwise or index values
+ * for each of the individual components.
  */
 private void
 set_color_ht_gt_4(byte *dest_data, uint dest_raster, int px, int py,
 		  int w, int h, int depth, int special, int num_planes,
 		  gx_color_index plane_mask, gx_device *dev,
 		  const color_values_pair_t *pvp,
-		  gx_color_index colors[1 << MAX_DCC],
+		  gx_color_index colors[MAX_DCC_16],
 		  const gx_const_strip_bitmap * sbits[MAX_DCC])
 {
     int x, y;
@@ -942,7 +1352,7 @@ set_color_ht_gt_4(byte *dest_data, uint dest_raster, int px, int py,
     byte *dest_row =
 	dest_data + dest_raster * (h - 1) + (w * depth) / 8;
     int pmin, pmax;
-    gx_color_value cv[MAX_DCC];
+    gx_color_index base_color = 0;
 
     /* Compute the range of active planes. */
     if (plane_mask == 0)
@@ -953,6 +1363,9 @@ set_color_ht_gt_4(byte *dest_data, uint dest_raster, int px, int py,
 	for (pmax = 0; (plane_mask >> pmax) > 1; )
 	    ++pmax;
     }
+    if_debug6('h',
+	      "[h]color_ht_gt_4: x=%d y=%d w=%d h=%d plane_mask=0x%lu depth=%d\n",
+	      px, py, w, h, (ulong)plane_mask, depth);
 
     /* Do one-time cursor initialization. */
     {
@@ -965,14 +1378,13 @@ set_color_ht_gt_4(byte *dest_data, uint dest_raster, int px, int py,
 		init_tile_cursor(i, &cursor[i], sbits[i], endx, lasty);
     }
 
-    /* Pre-load the color value for the unchanging planes. */
+    /* Pre-load the color value for the non halftoning planes. */
     {
 	int i;
 
-	for (i = 0; i < pmin; ++i)
-	    cv[i] = pvp->values[0][i];
-	for (i = pmax + 1; i < num_planes; ++i)
-	    cv[i] = pvp->values[0][i];
+	for (i = 0; i < num_planes; ++i)
+	    if ((~plane_mask >> i) & 1)
+		base_color |= colors[2 * i];
     }
 
     /* Now compute the actual tile. */
@@ -982,8 +1394,7 @@ set_color_ht_gt_4(byte *dest_data, uint dest_raster, int px, int py,
 
 	--y;
 	for (x = w; x > 0;) {
-	    gx_color_index index = 0;
-	    gx_color_index tcolor;
+	    gx_color_index tcolor = base_color;
 
 	    for (i = pmin; i <= pmax; ++i)
 		if ((plane_mask >> i) & 1) {
@@ -1002,19 +1413,8 @@ b:		    if (ptc->bit_shift < 8)
 			ptc->bit_shift = 8 - ptc->xbits;
 			goto b;
 		    }
-		    index |= (gx_color_index)(tile_bit & 1) << i;
+		    tcolor |= colors[2 * i + (tile_bit & 1)];
 		}
-	    tcolor = colors[index];
-	    if (tcolor == gx_no_color_index) {
-		/* Map the color value now. */
-		int i;
-
-		for (i = pmin; i <= pmax; ++i)
-		    cv[i] = pvp->values[(index >> i) & 1][i];
-		/****** HACK -- NO WAY TO MAP GENERAL COLORS ******/
-		tcolor = colors[index] =
-		    gx_map_cmyk_color(dev, cv[0], cv[1], cv[2], cv[3]);
-	    }
 	    --x;
 	    switch (dbytes) {
 		case 0:	/* 4 -- might be 2, but we don't support this */
