@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxclist.c,v 1.5 2002/06/28 15:54:46 easysw Exp $ */
+/*$Id: gxclist.c,v 1.9.2.1 2003/12/12 22:08:03 giles Exp $ */
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -27,6 +27,7 @@
 #include "gxcldev.h"
 #include "gxclpath.h"
 #include "gsparams.h"
+#include "gxdcolor.h"
 
 /* GC information */
 #define CLIST_IS_WRITER(cdev) ((cdev)->common.ymin < 0)
@@ -74,7 +75,7 @@ private dev_proc_get_band(clist_get_band);
 /* Driver procedures defined in other files are declared in gxcldev.h. */
 
 /* Other forward declarations */
-private int clist_put_current_params(P1(gx_device_clist_writer *cldev));
+private int clist_put_current_params(gx_device_clist_writer *cldev);
 
 /* The device procedures */
 const gx_device_procs gs_clist_device_procs = {
@@ -121,7 +122,19 @@ const gx_device_procs gs_clist_device_procs = {
     clist_create_compositor,
     gx_forward_get_hardware_params,
     gx_default_text_begin,
-    gx_default_finish_copydevice
+    gx_default_finish_copydevice,
+    NULL,			/* begin_transparency_group */
+    NULL,			/* end_transparency_group */
+    NULL,			/* begin_transparency_mask */
+    NULL,			/* end_transparency_mask */
+    NULL,			/* discard_transparency_layer */
+    gx_forward_get_color_mapping_procs,
+    gx_forward_get_color_comp_index,
+    gx_forward_encode_color,
+    gx_forward_decode_color,
+    gx_default_pattern_manage,
+    gx_default_fill_rectangle_hl_color,
+    gx_default_include_color_space
 };
 
 /* ------ Define the command set and syntax ------ */
@@ -159,15 +172,7 @@ clist_tile_cache_size(const gx_device * target, uint data_size)
     if (bits_size < min_bits_size)
 	bits_size = min_bits_size;
 #undef min_bits_size
-
-   /*
-    * Make sure that the bits_size value is a nice round number, both here
-    * and below (and everywhere, really), since Ghostscript uses a lot of
-    * FORTRAN-like buffers rather than using real data structures or
-    * pointers.
-    */
-
-    return (bits_size + align_cached_bits_mod) & -align_cached_bits_mod;
+    return bits_size;
 }
 
 /*
@@ -295,6 +300,10 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
 
     /* Call create_buf_device to get the memory planarity set up. */
     cdev->buf_procs.create_buf_device(&pbdev, target, NULL, NULL, true);
+    /* HACK - if the buffer device can't do copy_alpha, disallow */
+    /* copy_alpha in the commmand list device as well. */
+    if (dev_proc(pbdev, copy_alpha) == gx_no_copy_alpha)
+	cdev->disable_mask |= clist_disable_copy_alpha;
     if (band_height) {
 	/*
 	 * The band height is fixed, so the band buffer requirement
@@ -322,7 +331,6 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     if (code < 0)
 	return code;
     cdev->page_tile_cache_size = bits_size;
-    bits_size = (bits_size + align_cached_bits_mod) & -align_cached_bits_mod;
     data += bits_size;
     size -= bits_size;
     code = clist_init_bands(dev, &bdev, size, band_width, band_height);
