@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxclrect.c,v 1.3 2002/04/23 11:58:45 easysw Exp $ */
+/* $Id: gxclrect.c,v 1.5.2.1 2003/12/12 22:08:03 giles Exp $ */
 /* Rectangle-oriented command writing for command list */
 #include "gx.h"
 #include "gserrors.h"
@@ -429,6 +429,10 @@ clist_copy_alpha(gx_device * dev, const byte * data, int data_x,
     int y0;
     int data_x_bit;
 
+    /* If the target can't perform copy_alpha, exit now */
+    if (depth > 1 && (cdev->disable_mask & clist_disable_copy_alpha) != 0)
+	return_error(gs_error_unknownerror);
+
     fit_copy(dev, data, data_x, raster, id, x, y, width, height);
     y0 = y;
     data_x_bit = data_x << log2_depth;
@@ -579,8 +583,10 @@ clist_strip_copy_rop(gx_device * dev,
 	gx_color_index D = pcls->colors_used.or;
 	int code;
 
+	/* Reducing D, S, T to rop_operand (which apparently is 32 bit) appears safe
+	   due to 'all' a has smaller snumber of significant bits. */
 	pcls->colors_used.or =
-	    ((rop_proc_table[color_rop])(D, S, T) & all) | D;
+	    ((rop_proc_table[color_rop])((rop_operand)D, (rop_operand)S, (rop_operand)T) & all) | D;
 	pcls->colors_used.slow_rop |= slow_rop;
 	if (rop3_uses_T(rop)) {
 	    if (tcolors == 0 || tcolors[0] != tcolors[1]) {
