@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: iddstack.h,v 1.3 2002/04/23 11:58:46 easysw Exp $ */
+/* $Id: iddstack.h,v 1.5 2002/06/16 04:47:10 lpd Exp $ */
 /* Dictionary stack API subset needed by idict.h */
 
 #ifndef iddstack_INCLUDED
@@ -30,9 +30,9 @@ typedef struct dict_stack_s dict_stack_t;
  * dictionary stack (including changing the protection or size of the
  * top dictionary on the stack) must call this.
  */
-void dstack_set_top(P1(dict_stack_t *));
+void dstack_set_top(dict_stack_t *);
 
 /* Check whether a dictionary is one of the permanent ones on the d-stack. */
-bool dstack_dict_is_permanent(P2(const dict_stack_t *, const ref *));
+bool dstack_dict_is_permanent(const dict_stack_t *, const ref *);
 
 #endif /* iddstack_INCLUDED */
