@@ -14,15 +14,12 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: ifont2.h,v 1.3 2002/04/23 11:58:46 easysw Exp $ */
+/* $Id: ifont2.h,v 1.5.2.1 2003/12/12 22:08:04 giles Exp $ */
 /* Type 2 font utilities 2 */
 
 #ifndef ifont2_INCLUDED
 #  define ifont2_INCLUDED
 
-/* Declare the Type 2 interpreter. */
-extern charstring_interpret_proc(gs_type2_interpret);
-
 /* Default value of lenIV */
 #define DEFAULT_LENIV_2 (-1)
 
@@ -31,7 +28,7 @@ extern charstring_interpret_proc(gs_type2_interpret);
  * entry in a CIDFontType 0 font), beyond those common to Type 1 and Type 2
  * fonts.
  */
-int type2_font_params(P3(const_os_ptr op, charstring_font_refs_t *pfr,
-			 gs_type1_data *pdata1));
+int type2_font_params(const_os_ptr op, charstring_font_refs_t *pfr,
+		      gs_type1_data *pdata1);
 
 #endif /* ifont2_INCLUDED */
