@@ -14,14 +14,14 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: iinit.c,v 1.5 2003/08/19 15:24:41 tillkamppeter Exp $ */
+/* $Id: iinit.c,v 1.8.2.1 2003/12/12 22:08:04 giles Exp $ */
 /* Initialize internally known objects for Ghostscript interpreter */
 #include "string_.h"
 #include "ghost.h"
 #include "gscdefs.h"
 #include "gsexit.h"
 #include "gsstruct.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "ialloc.h"
 #include "iddict.h"
 #include "dstack.h"
@@ -47,29 +47,29 @@
  */
 /* The size of systemdict can be set in the makefile. */
 #ifndef SYSTEMDICT_SIZE
-#  define SYSTEMDICT_SIZE 941
+#  define SYSTEMDICT_SIZE 631
 #endif
 #ifndef SYSTEMDICT_LEVEL2_SIZE
-#  define SYSTEMDICT_LEVEL2_SIZE 1241
+#  define SYSTEMDICT_LEVEL2_SIZE 983
 #endif
 #ifndef SYSTEMDICT_LL3_SIZE
-#  define SYSTEMDICT_LL3_SIZE 1417
+#  define SYSTEMDICT_LL3_SIZE 1123
 #endif
 /* The size of level2dict, if applicable, can be set in the makefile. */
 #ifndef LEVEL2DICT_SIZE
-#  define LEVEL2DICT_SIZE 433
+#  define LEVEL2DICT_SIZE 251
 #endif
 /* Ditto the size of ll3dict. */
 #ifndef LL3DICT_SIZE
-#  define LL3DICT_SIZE 83
+#  define LL3DICT_SIZE 43
 #endif
 /* Ditto the size of filterdict. */
 #ifndef FILTERDICT_SIZE
-#  define FILTERDICT_SIZE 83
+#  define FILTERDICT_SIZE 43
 #endif
 /* Define an arbitrary size for the operator procedure tables. */
 #ifndef OP_ARRAY_TABLE_SIZE
-#  define OP_ARRAY_TABLE_SIZE 180
+#  define OP_ARRAY_TABLE_SIZE 300
 #endif
 #ifndef OP_ARRAY_TABLE_GLOBAL_SIZE
 #  define OP_ARRAY_TABLE_GLOBAL_SIZE OP_ARRAY_TABLE_SIZE
@@ -461,8 +461,10 @@ op_init(i_ctx_t *i_ctx_p)
 		uint opidx = (tptr - op_defs_all) * OP_DEFS_MAX_SIZE +
 		    index_in_table;
 
-		if (index_in_table >= OP_DEFS_MAX_SIZE)
-		    dprintf1("opdef overrun: %s\n", def->oname);
+		if (index_in_table >= OP_DEFS_MAX_SIZE) {
+		    lprintf1("opdef overrun! %s\n", def->oname);
+		    return_error(e_Fatal);
+		}
 		gs_interp_make_oper(&oper, def->proc, opidx);
 		/* The first character of the name is a digit */
 		/* giving the minimum acceptable number of operands. */
