@@ -1,4 +1,4 @@
-/* Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1996-2003 artofcode LLC.  All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,11 +14,13 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: imainarg.c,v 1.10 2003/08/19 15:24:41 tillkamppeter Exp $ */
+/* $Id: imainarg.c,v 1.29.2.2 2004/11/30 20:32:47 ghostgum Exp $ */
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
 #include "string_.h"
+#include <stdlib.h>	/* for qsort */
+
 #include "ghost.h"
 #include "gp.h"
 #include "gsargs.h"
@@ -29,7 +31,7 @@
 #include "gxdevmem.h"
 #include "gsdevice.h"
 #include "stream.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "estack.h"
 #include "ialloc.h"
 #include "strimpl.h"		/* for sfilter.h */
@@ -47,10 +49,11 @@
 #include "interp.h"
 #include "iutil.h"
 #include "ivmspace.h"
+#include "vdtrace.h"
 
 /* Import operator procedures */
-extern int zflush(P1(i_ctx_t *));
-extern int zflushpage(P1(i_ctx_t *));
+extern int zflush(i_ctx_t *);
+extern int zflushpage(i_ctx_t *);
 
 #ifndef GS_LIB
 #  define GS_LIB "GS_LIB"
@@ -82,26 +85,26 @@ extern int zflushpage(P1(i_ctx_t *));
 #define runInit 1
 #define runFlush 2
 #define runBuffer 4
-private int swproc(P3(gs_main_instance *, const char *, arg_list *));
-private int argproc(P2(gs_main_instance *, const char *));
-private int run_buffered(P2(gs_main_instance *, const char *));
-private int esc_strlen(P1(const char *));
-private void esc_strcat(P2(char *, const char *));
-private int runarg(P5(gs_main_instance *, const char *, const char *, const char *, int));
-private int run_string(P3(gs_main_instance *, const char *, int));
-private int run_finish(P4(gs_main_instance *, int, int, ref *));
-private int try_stdout_redirect(P3(gs_main_instance * minst, 
-    const char *command, const char *filename));
+private int swproc(gs_main_instance *, const char *, arg_list *);
+private int argproc(gs_main_instance *, const char *);
+private int run_buffered(gs_main_instance *, const char *);
+private int esc_strlen(const char *);
+private void esc_strcat(char *, const char *);
+private int runarg(gs_main_instance *, const char *, const char *, const char *, int);
+private int run_string(gs_main_instance *, const char *, int);
+private int run_finish(gs_main_instance *, int, int, ref *);
+private int try_stdout_redirect(gs_main_instance * minst, 
+				const char *command, const char *filename);
 
 /* Forward references for help printout */
-private void print_help(P1(gs_main_instance *));
-private void print_revision(P1(const gs_main_instance *));
-private void print_version(P1(const gs_main_instance *));
-private void print_usage(P1(const gs_main_instance *));
-private void print_devices(P1(const gs_main_instance *));
-private void print_emulators(P1(const gs_main_instance *));
-private void print_paths(P1(gs_main_instance *));
-private void print_help_trailer(P1(const gs_main_instance *));
+private void print_help(gs_main_instance *);
+private void print_revision(const gs_main_instance *);
+private void print_version(const gs_main_instance *);
+private void print_usage(const gs_main_instance *);
+private void print_devices(const gs_main_instance *);
+private void print_emulators(const gs_main_instance *);
+private void print_paths(gs_main_instance *);
+private void print_help_trailer(const gs_main_instance *);
 
 /* ------ Main program ------ */
 
@@ -112,6 +115,14 @@ gs_main_arg_fopen(const char *fname, void *vminst)
     gs_main_set_lib_paths((gs_main_instance *) vminst);
     return lib_fopen(fname);
 }
+private void
+set_debug_flags(const char *arg, char *flags)
+{
+    byte value = (*arg == '-' ? (++arg, 0) : 0xff);
+
+    while (*arg)
+	flags[*arg++ & 127] = value;
+}
 #define arg_heap_copy(str) arg_copy(str, &gs_memory_default)
 int
 gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
@@ -338,21 +349,6 @@ run_stdin:
 		if (code < 0)
 		    return code;
 		pal->expand_ats = false;
-
-		if (arg[0]) {
-		    char *sarg;
-
-		    if (arg[0] == '@' ||
-			(arg[0] == '-' && !isdigit(arg[1]))
-			)
-			break;
-		    sarg = arg_heap_copy(arg);
-		    if (sarg == NULL)
-			return e_Fatal;
-		    code = runarg(minst, "", sarg, ".runstring", 0);
-		    if (code < 0)
-			return code;
-		}
 		while ((arg = arg_next(pal, &code)) != 0) {
 		    char *sarg;
 
@@ -376,8 +372,8 @@ run_stdin:
 		    arg_push_string(pal, p);
 		}
 		pal->expand_ats = ats;
+		break;
 	    }
-	    break;
 	case 'E':		/* log errors */
 	    switch (*arg) {
 		case 0:
@@ -392,8 +388,11 @@ run_stdin:
 	    }
 	    break;
 	case 'f':		/* run file of arbitrary name */
-	    if (*arg != 0)
-		argproc(minst, arg);
+	    if (*arg != 0) {
+		code = argproc(minst, arg);
+		if (code < 0)
+		    return code;
+	    }
 	    break;
 	case 'F':		/* run file with buffer_size = 1 */
 	    if (!*arg) {
@@ -403,8 +402,10 @@ run_stdin:
 		uint bsize = minst->run_buffer_size;
 
 		minst->run_buffer_size = 1;
-		argproc(minst, arg);
+		code = argproc(minst, arg);
 		minst->run_buffer_size = bsize;
+		if (code < 0)
+		    return code;
 	    }
 	    break;
 	case 'g':		/* define device geometry */
@@ -412,7 +413,8 @@ run_stdin:
 		long width, height;
 		ref value;
 
-		gs_main_init1(minst);
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
 		if (sscanf((const char *)arg, "%ldx%ld", &width, &height) != 2) {
 		    puts("-g must be followed by <width>x<height>");
 		    return e_Fatal;
@@ -488,7 +490,8 @@ run_stdin:
 	    }
 	    break;
 	case 'q':		/* quiet startup */
-	    gs_main_init1(minst);
+	    if ((code = gs_main_init1(minst)) < 0)
+		return code;
 	    initial_enter_name("QUIET", &vtrue);
 	    break;
 	case 'r':		/* define device resolution */
@@ -496,7 +499,8 @@ run_stdin:
 		float xres, yres;
 		ref value;
 
-		gs_main_init1(minst);
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
 		switch (sscanf((const char *)arg, "%fx%f", &xres, &yres)) {
 		    default:
 			puts("-r must be followed by <res> or <xres>x<yres>");
@@ -530,7 +534,8 @@ run_stdin:
 		    eqp = strchr(adef, '#');
 		/* Initialize the object memory, scanner, and */
 		/* name table now if needed. */
-		gs_main_init1(minst);
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
 		if (eqp == adef) {
 		    puts("Usage: -dname, -dname=token, -sname=string");
 		    return e_Fatal;
@@ -603,12 +608,16 @@ run_stdin:
 		initial_enter_name(adef, &value);
 		break;
 	    }
+	case 'T':
+            set_debug_flags(arg, vd_flags);
+	    break;
 	case 'u':		/* undefine name */
 	    if (!*arg) {
 		puts("-u requires a name to undefine.");
 		return e_Fatal;
 	    }
-	    gs_main_init1(minst);
+	    if ((code = gs_main_init1(minst)) < 0)
+		return code;
 	    i_initial_remove_name(minst->i_ctx_p, arg);
 	    break;
 	case 'v':		/* print revision */
@@ -641,12 +650,7 @@ run_stdin:
 	    return e_Quit;
 /*#endif */
 	case 'Z':
-	    {
-		byte value = (*arg == '-' ? (++arg, 0) : 0xff);
-
-		while (*arg)
-		    gs_debug[*arg++ & 127] = value;
-	    }
+            set_debug_flags(arg, gs_debug);
 	    break;
     }
     return 0;
@@ -685,17 +689,18 @@ argproc(gs_main_instance * minst, const char *arg)
 {
     int code = gs_main_init1(minst);		/* need i_ctx_p to proceed */
     char *filearg;
-    
+
     if (code < 0)
-    	return code;
+        return code;
     filearg = arg_heap_copy(arg);
-    minst->i_ctx_p->filearg = filearg;	/* allow reading this file if SAFER set */
+    if (filearg == NULL)
+        return e_Fatal;
     if (minst->run_buffer_size) {
 	/* Run file with run_string. */
-	return run_buffered(minst, arg);
+	return run_buffered(minst, filearg);
     } else {
 	/* Run file directly in the normal way. */
-	return runarg(minst, "", arg, ".runfile", runInit | runFlush);
+	return runarg(minst, "", filearg, ".runfile", runInit | runFlush);
     }
 }
 private int
@@ -742,10 +747,11 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
        const char *post, int options)
 {
     int len = strlen(pre) + esc_strlen(arg) + strlen(post) + 1;
+    int code;
     char *line;
 
     if (options & runInit) {
-	int code = gs_main_init2(minst);	/* Finish initialization */
+	code = gs_main_init2(minst);	/* Finish initialization */
 
 	if (code < 0)
 	    return code;
@@ -758,7 +764,10 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
     strcpy(line, pre);
     esc_strcat(line, arg);
     strcat(line, post);
-    return run_string(minst, line, options);
+    minst->i_ctx_p->starting_arg_file = true;
+    code = run_string(minst, line, options);
+    minst->i_ctx_p->starting_arg_file = false;
+    return code;
 }
 private int
 run_string(gs_main_instance * minst, const char *str, int options)
@@ -847,8 +856,8 @@ private const char help_usage2[] = "\
  -sOutputFile=<file> select output file: - for stdout, |command for pipe,\n\
                                          embed %d or %ld for page #\n";
 private const char help_trailer[] = "\
-For more information, see %s%sUse.htm.\n\
-Report bugs using the form at \"http://www.cups.org/str.php\".\n";
+For more information, see %s.\n\
+Report bugs to %s, using the form in Bug-form.htm.\n";
 private const char help_devices[] = "Available devices:";
 private const char help_default_device[] = "Default output device:";
 private const char help_emulators[] = "Input formats:";
@@ -873,7 +882,7 @@ print_help(gs_main_instance * minst)
 private void
 print_revision(const gs_main_instance *minst)
 {
-    printf_program_ident(gs_product, gs_revision, gs_revisionpatch);
+    printf_program_ident(gs_product, gs_revision);
     outprintf(" (%d-%02d-%02d)\n%s\n",
 	    (int)(gs_revisiondate / 10000),
 	    (int)(gs_revisiondate / 100 % 100),
@@ -885,7 +894,7 @@ print_revision(const gs_main_instance *minst)
 private void
 print_version(const gs_main_instance *minst)
 {
-    printf_program_ident(NULL, gs_revision, gs_revisionpatch);
+    printf_program_ident(NULL, gs_revision);
 }
 
 /* Print usage information. */
@@ -896,7 +905,7 @@ print_usage(const gs_main_instance *minst)
     outprintf("%s", help_usage2);
 }
 
-/* compare function for device name qsort */
+/* compare function for qsort */
 private int
 cmpstr(const void *v1, const void *v2)
 {
@@ -932,7 +941,7 @@ print_devices(const gs_main_instance *minst)
 		pos += 1 + len;
 	    }
 	}
-	else {                          /* new-style sorted device list */
+	else {				/* new-style sorted device list */
 	    for (i = 0; (pdev = gs_getdevice(i)) != 0; i++)
 		names[i] = gs_devicename(pdev);
 	    qsort((void*)names, ndev, sizeof(const char*), cmpstr);
@@ -1018,8 +1027,12 @@ print_paths(gs_main_instance * minst)
 private void
 print_help_trailer(const gs_main_instance *minst)
 {
-    outprintf(help_trailer, gs_doc_directory,
-	    gp_file_name_concat_string(gs_doc_directory,
-				       strlen(gs_doc_directory)),
-	    GS_BUG_MAILBOX);
+    char buffer[gp_file_name_sizeof];
+    const char *use_htm = "Use.htm", *p = buffer;
+    uint blen = sizeof(buffer);
+
+    if (gp_file_name_combine(gs_doc_directory, strlen(gs_doc_directory), 
+	    use_htm, strlen(use_htm), false, buffer, &blen) != gp_combine_success)
+	p = use_htm;
+    outprintf(help_trailer, p, GS_BUG_MAILBOX);
 }
