@@ -14,14 +14,14 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: interp.c,v 1.4 2003/07/13 04:43:18 easysw Exp $ */
+/* $Id: interp.c,v 1.14.2.1 2003/12/12 22:08:04 giles Exp $ */
 /* Ghostscript language interpreter */
 #include "memory_.h"
 #include "string_.h"
 #include "ghost.h"
 #include "gsstruct.h"		/* for iastruct.h */
 #include "stream.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "estack.h"
 #include "ialloc.h"
 #include "iastruct.h"
@@ -46,6 +46,7 @@
 #include "files.h"		/* for file_check_read */
 #include "oper.h"
 #include "store.h"
+#include "gpcheck.h"
 
 /*
  * We may or may not optimize the handling of the special fast operators
@@ -79,13 +80,13 @@ no_reschedule(i_ctx_t **pi_ctx_p)
 {
     return_error(e_invalidcontext);
 }
-int (*gs_interp_reschedule_proc)(P1(i_ctx_t **)) = no_reschedule;
+int (*gs_interp_reschedule_proc)(i_ctx_t **) = no_reschedule;
 
 /*
  * The procedure to call for time-slicing.
  * This is a no-op unless the context machinery has been installed.
  */
-int (*gs_interp_time_slice_proc)(P1(i_ctx_t **)) = 0;
+int (*gs_interp_time_slice_proc)(i_ctx_t **) = 0;
 
 /*
  * The number of interpreter "ticks" between calls on the time_slice_proc.
@@ -128,15 +129,15 @@ struct stats_interp_s {
 #endif
 
 /* Forward references */
-private int estack_underflow(P1(i_ctx_t *));
-private int interp(P3(i_ctx_t **, const ref *, ref *));
-private int interp_exit(P1(i_ctx_t *));
-private void set_gc_signal(P3(i_ctx_t *, int *, int));
-private int copy_stack(P3(i_ctx_t *, const ref_stack_t *, ref *));
-private int oparray_pop(P1(i_ctx_t *));
-private int oparray_cleanup(P1(i_ctx_t *));
-private int zsetstackprotect(P1(i_ctx_t *));
-private int zcurrentstackprotect(P1(i_ctx_t *));
+private int estack_underflow(i_ctx_t *);
+private int interp(i_ctx_t **, const ref *, ref *);
+private int interp_exit(i_ctx_t *);
+private void set_gc_signal(i_ctx_t *, int *, int);
+private int copy_stack(i_ctx_t *, const ref_stack_t *, ref *);
+private int oparray_pop(i_ctx_t *);
+private int oparray_cleanup(i_ctx_t *);
+private int zsetstackprotect(i_ctx_t *);
+private int zcurrentstackprotect(i_ctx_t *);
 
 /* Stack sizes */
 
@@ -405,7 +406,7 @@ gs_interp_make_oper(ref * opref, op_proc_t proc, int idx)
 /*
  * Call the garbage collector, updating the context pointer properly.
  */
-private int
+int
 interp_reclaim(i_ctx_t **pi_ctx_p, int space)
 {
     i_ctx_t *i_ctx_p = *pi_ctx_p;
@@ -430,7 +431,7 @@ interp_reclaim(i_ctx_t **pi_ctx_p, int space)
  * In case of a quit or a fatal error, also store the exit code.
  * Set *perror_object to null or the error object.
  */
-private int gs_call_interp(P5(i_ctx_t **, ref *, int, int *, ref *));
+private int gs_call_interp(i_ctx_t **, ref *, int, int *, ref *);
 int
 gs_interpret(i_ctx_t **pi_ctx_p, ref * pref, int user_errors, int *pexit_code,
 	     ref * perror_object)
@@ -868,7 +869,7 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
 	  r_packed_is_name(iref_packed) :
 	  r_has_type(IREF, t_name)))
 	) {
-	void debug_print_ref(P1(const ref *));
+	void debug_print_ref(const ref *);
 	os_ptr save_osp = osp;	/* avoid side-effects */
 	es_ptr save_esp = esp;
 
@@ -1649,6 +1650,7 @@ res:
     } else
 	code = 0;
     ticks_left = gs_interp_time_slice_ticks;
+    set_code_on_interrupt(&code);
     goto sched;
 
     /* Error exits. */
