@@ -14,13 +14,13 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: istack.c,v 1.3 2002/04/23 11:58:47 easysw Exp $ */
+/* $Id: istack.c,v 1.5.2.1 2003/12/12 22:08:04 giles Exp $ */
 /* Manager for expandable stacks of refs */
 #include "memory_.h"
 #include "ghost.h"
 #include "gsstruct.h"
 #include "gsutil.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "ialloc.h"
 #include "istack.h"
 #include "istkparm.h"
@@ -30,9 +30,9 @@
 #include "store.h"
 
 /* Forward references */
-private void init_block(P3(ref_stack_t *pstack, const ref *pblock_array,
-			   uint used));
-private int ref_stack_push_block(P3(ref_stack_t *pstack, uint keep, uint add));
+private void init_block(ref_stack_t *pstack, const ref *pblock_array,
+			uint used);
+private int ref_stack_push_block(ref_stack_t *pstack, uint keep, uint add);
 
 /* GC descriptors and procedures */
 private_st_ref_stack_params();
