@@ -14,14 +14,14 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: ivmem2.h,v 1.3 2002/04/23 11:58:47 easysw Exp $ */
+/* $Id: ivmem2.h,v 1.5 2002/06/16 04:47:10 lpd Exp $ */
 /* VM control user parameter procedures */
 
 #ifndef ivmem2_INCLUDED
 #  define ivmem2_INCLUDED
 
 /* Exported by zvmem2.c for zusparam.c */
-int set_vm_reclaim(P2(i_ctx_t *, long));
-int set_vm_threshold(P2(i_ctx_t *, long));
+int set_vm_reclaim(i_ctx_t *, long);
+int set_vm_threshold(i_ctx_t *, long);
 
 #endif /* ivmem2_INCLUDED */
