@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: sdcparam.c,v 1.3 2002/04/23 11:58:47 easysw Exp $ */
+/* $Id: sdcparam.c,v 1.5.2.1 2003/12/12 22:08:05 giles Exp $ */
 /* DCT filter parameter setting and reading */
 #include "memory_.h"
 #include "jpeglib_.h"
@@ -155,7 +155,7 @@ s_DCT_get_quantization_tables(gs_param_list * plist,
 	    default_table_ptrs = defaults->data.compress->cinfo.quant_tbl_ptrs;
 	}
     } else {
-	/**************** quant_tables.size NOT INITIALIZED ****************/
+	quant_tables.size = count_of(d_comp_info);
 	num_in_tables = quant_tables.size;
 	for (i = 0; i < num_in_tables; ++i)
 	    d_comp_info[i].quant_tbl_no = i;
