@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: ziodev.c,v 1.4 2003/07/13 04:43:18 easysw Exp $ */
+/* $Id: ziodev.c,v 1.12.2.1 2003/12/12 22:08:06 giles Exp $ */
 /* Standard IODevice implementation */
 #include "memory_.h"
 #include "stdio_.h"
@@ -33,7 +33,7 @@
 #include "files.h"
 #include "scanchar.h"		/* for char_EOL */
 #include "store.h"
-#include "errors.h"
+#include "ierrors.h"
 
 /* Import the dtype of the stdio IODevices. */
 extern const char iodev_dtype_stdio[];
@@ -57,12 +57,12 @@ extern const char iodev_dtype_stdio[];
  */
 
 #define LINEEDIT_BUF_SIZE 20	/* initial size, not fixed size */
-private iodev_proc_open_device(lineedit_open);
+/*private iodev_proc_open_device(lineedit_open);*/ /* no longer used */
 const gx_io_device gs_iodev_lineedit =
     iodev_special("%lineedit%", iodev_no_init, iodev_no_open_device);
 
 #define STATEMENTEDIT_BUF_SIZE 50	/* initial size, not fixed size */
-private iodev_proc_open_device(statementedit_open);
+/*private iodev_proc_open_device(statementedit_open);*/ /* no longer used */
 const gx_io_device gs_iodev_statementedit =
     iodev_special("%statementedit%", iodev_no_init, iodev_no_open_device);
 
