@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: zmedia2.c,v 1.4 2002/06/28 17:05:38 easysw Exp $ */
+/* $Id: zmedia2.c,v 1.9.2.1 2004/02/13 19:18:35 ray Exp $ */
 /* Media matching for setpagedevice */
 #include "math_.h"
 #include "memory_.h"
@@ -29,10 +29,10 @@
 /* <pagedict> <attrdict> <policydict> <keys> .matchmedia <key> true */
 /* <pagedict> <attrdict> <policydict> <keys> .matchmedia false */
 /* <pagedict> null <policydict> <keys> .matchmedia null true */
-private int zmatch_page_size(P8(const ref * pvreq, const ref * pvmed,
-				int policy, int orient, bool roll,
-				float *best_mismatch, gs_matrix * pmat,
-				gs_point * pmsize));
+private int zmatch_page_size(const ref * pvreq, const ref * pvmed,
+			     int policy, int orient, bool roll,
+			     float *best_mismatch, gs_matrix * pmat,
+			     gs_point * pmsize);
 typedef struct match_record_s {
     ref best_key, match_key;
     uint priority, no_match_priority;
@@ -178,15 +178,24 @@ zmatchmedia(i_ctx_t *i_ctx_p)
 		} else if (!obj_eq(prvalue, pmvalue))
 		    goto no;
 	    }
-	    /* We have a match.  If it is a better match */
-	    /* than the current best one, it supersedes it */
-	    /* regardless of priority. */
-	    if (best_mismatch < mbest) {
-		mbest = best_mismatch;
+	    /* We have a match. Save the match in case no better match is found */
+	    if (r_has_type(&match.match_key, t_null)) 
+		match.match_key = aelt.key;
+	    /*
+	     * If it is a better match than the current best it supersedes it 
+	     * regardless of priority. If the match is the same, then update 
+	     * to the current only if the key value is lower.
+	     */
+	    if (best_mismatch <= mbest) {
+		if (best_mismatch < mbest  ||
+		    (r_has_type(&match.match_key, t_integer) &&
+		     match.match_key.value.intval > aelt.key.value.intval)) {
 		reset_match(&match);
+		    match.match_key = aelt.key;
+		    mbest = best_mismatch;
 	    }
-	    /* In case of a tie, see if the new match has */
-	    /* priority. */
+	    }
+	    /* In case of a tie, see if the new match has priority. */
 	    for (pi = match.priority; pi > 0;) {
 		ref pri;
 
@@ -198,8 +207,6 @@ zmatchmedia(i_ctx_t *i_ctx_p)
 		    break;
 		}
 	    }
-	    /* Save the match in case no match has priority. */
-	    match.match_key = aelt.key;
 no:;
 	}
     }
@@ -265,32 +272,37 @@ zmatchpagesize(i_ctx_t *i_ctx_p)
     return 0;
 }
 /* Match the PageSize.  See below for details. */
-private bool match_page_size(P8(const gs_point * request,
-				const gs_rect * medium,
-				int policy, int orient, bool roll,
-				float *best_mismatch, gs_matrix * pmat,
-				gs_point * pmsize));
+private int
+match_page_size(const gs_point * request,
+			     const gs_rect * medium,
+			     int policy, int orient, bool roll,
+			     float *best_mismatch, gs_matrix * pmat,
+			     gs_point * pmsize);
 private int
 zmatch_page_size(const ref * pvreq, const ref * pvmed,
 		 int policy, int orient, bool roll,
 		 float *best_mismatch, gs_matrix * pmat, gs_point * pmsize)
 {
     uint nr, nm;
+    int code;
+    ref rv[6];
 
-    check_array(*pvreq);
+    /* array_get checks array types and size. */
+    /* This allows normal or packed arrays to be used */
+    if ((code = array_get(pvreq, 1, &rv[1])) < 0)
+        return_error(code);
     nr = r_size(pvreq);
-    check_array(*pvmed);
+    if ((code = array_get(pvmed, 1, &rv[3])) < 0)
+        return_error(code);
     nm = r_size(pvmed);
     if (!((nm == 2 || nm == 4) && (nr == 2 || nr == nm)))
 	return_error(e_rangecheck);
     {
-	ref rv[6];
 	uint i;
 	double v[6];
 	int code;
 
 	array_get(pvreq, 0, &rv[0]);
-	array_get(pvreq, 1, &rv[1]);
 	for (i = 0; i < 4; ++i)
 	    array_get(pvmed, i % nm, &rv[i + 2]);
 	if ((code = num_params(rv + 5, 6, v)) < 0)
@@ -320,91 +332,38 @@ zmatch_page_size(const ref * pvreq, const ref * pvmed,
  * NOTE: The algorithm here doesn't work properly for variable-size media
  * when the match isn't exact.  We'll fix it if we ever need to.
  */
-private void make_adjustment_matrix(P5(const gs_point * request,
-				       const gs_rect * medium,
-				       gs_matrix * pmat,
-				       bool scale, int rotate));
-private bool
+private void make_adjustment_matrix(const gs_point * request,
+				    const gs_rect * medium,
+				    gs_matrix * pmat,
+				    bool scale, int rotate);
+private int
 match_page_size(const gs_point * request, const gs_rect * medium, int policy,
 		int orient, bool roll, float *best_mismatch, gs_matrix * pmat,
 		gs_point * pmsize)
 {
     double rx = request->x, ry = request->y;
 
-    if (policy == 7) {
-		/* (Adobe) hack: just impose requested values */
-	*best_mismatch = 0;
-	gs_make_identity(pmat);
-	*pmsize = *request;
-    } else {
-        int fit_direct  = rx - medium->p.x >= -5 && rx - medium->q.x <= 5 
-                       && ry - medium->p.y >= -5 && ry - medium->q.y <= 5;
-	int fit_rotated = rx - medium->p.y >= -5 && rx - medium->q.y <= 5 
-                       && ry - medium->p.x >= -5 && ry - medium->q.x <= 5;
-        
-        if ( fit_direct && fit_rotated) {
-	    *best_mismatch = 0;
-	    make_adjustment_matrix(request, medium, pmat, false, orient < 0 ? 0 : orient);
-        } else if ( fit_direct ) {
-            int rotate = orient < 0 ? 0 : orient;
-            *best_mismatch = rotate & 1 ? 0.1 : 0;
-	    make_adjustment_matrix(request, medium, pmat, false, (rotate + 1) & 2);
-        } else if ( fit_rotated ) {
-            int rotate = orient < 0 ? 1 : orient;
-            *best_mismatch = rotate & 1 ? 0 : 0.1;
-	    make_adjustment_matrix(request, medium, pmat, false, rotate | 1);
-        } else {
-	    int rotate = orient >= 0 ? orient : rx < ry ^ medium->q.x < medium->q.y;
-	    bool larger =
-	    (rotate & 1 ? medium->q.y >= rx && medium->q.x >= ry :
-	     medium->q.x >= rx && medium->q.y >= ry);
-	    bool adjust = false;
-	    float mismatch = medium->q.x * medium->q.y - rx * ry;
+    if ((rx <= 0) || (ry <= 0))
+	return_error(e_rangecheck);
 
-	    switch (policy) {
-	        default:		/* exact match only */
-		    return false;
-	        case 3:		/* nearest match, adjust */
-		    adjust = true;
-	        case 5:		/* nearest match, don't adjust */
-		    if (fabs(mismatch) >= fabs(*best_mismatch))
-		        return false;
-		    break;
-	        case 4:		/* next larger match, adjust */
-		    adjust = true;
-	        case 6:		/* next larger match, don't adjust */
-		    if (!larger || mismatch >= *best_mismatch)
-		        return false;
-		    break;
-	    }
-	    if (adjust)
-	        make_adjustment_matrix(request, medium, pmat, !larger, rotate);
-	    else {
-	        gs_rect req_rect;
-                if(rotate & 1) { 
-                    req_rect.p.x = ry;
-                    req_rect.p.y = rx;
-                } else {
-                    req_rect.p.x = rx;
-                    req_rect.p.y = ry;
-                }
-	        req_rect.q = req_rect.p;
-	        make_adjustment_matrix(request, &req_rect, pmat, false, rotate);
-	    }
-	    *best_mismatch = mismatch;
-        }
-        if (pmat->xx == 0) {	/* Swap request X and Y. */
-	    double temp = rx;
+    /*
+     * M. Sweet (Easy Software Products)
+     *
+     * Since no Ghostscript driver actually lists its supported page
+     * sizes, anything other than "impose" will yield the default page
+     * size that was compiled into Ghostscript, which is usually Letter.
+     *
+     * Since this is not the expected or intended behavior, we have
+     * dropped the media size matching code in favor of just imposing
+     * the selected size.
+     */
 
-	    rx = ry, ry = temp;
-        }
-#define ADJUST_INTO(req, mmin, mmax)\
-      (req < mmin ? mmin : req > mmax ? mmax : req)
-        pmsize->x = ADJUST_INTO(rx, medium->p.x, medium->q.x);
-        pmsize->y = ADJUST_INTO(ry, medium->p.y, medium->q.y);
-#undef ADJUST_INTO
-    }
-    return true;
+    /* (Adobe) hack: just impose requested values */
+    *best_mismatch = 0;
+    gs_make_identity(pmat);
+    *pmsize = *request;
+
+    return 1;
 }
 /*
  * Compute the adjustment matrix for scaling and/or rotating the page
@@ -430,16 +389,14 @@ make_adjustment_matrix(const gs_point * request, const gs_rect * medium,
     /* If 'medium' is flexible, adjust 'mx' and 'my' towards 'rx' and 'ry',
        respectively. Note that 'mx' and 'my' have just acquired the largest
        permissible value, medium->q. */
-    if (medium->p.x < mx) {	/* non-empty width range */
+    if (medium->p.x < mx)	/* non-empty width range */
       if (rx < medium->p.x) mx = medium->p.x;	/* minimum */
       else if (mx > rx) mx = rx;	/* fits */
       /* else use medium->q.x, i.e., the maximum */
-    }
-    if (medium->p.y < my) {	/* non-empty height range */
+    if (medium->p.y < my)	/* non-empty height range */
       if (ry < medium->p.y) my = medium->p.y;	/* minimum */
       else if (my > ry) my = ry;	/* fits */
       /* else use medium->q.y, i.e., the maximum */
-    }
 
     /* Translate to align the centers. */ 
     gs_make_translation(mx / 2, my / 2, pmat);
