@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: zmisc1.c,v 1.4 2003/02/03 22:33:37 tillkamppeter Exp $ */
+/* $Id: zmisc1.c,v 1.7 2002/06/16 03:43:51 lpd Exp $ */
 /* Miscellaneous Type 1 font operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -29,8 +29,8 @@
 
 /* <state> <from_string> <to_string> .type1encrypt <new_state> <substring> */
 /* <state> <from_string> <to_string> .type1decrypt <new_state> <substring> */
-private int type1crypt(P2(i_ctx_t *,
-			int (*)(P4(byte *, const byte *, uint, ushort *))));
+private int type1crypt(i_ctx_t *,
+		       int (*)(byte *, const byte *, uint, ushort *));
 private int
 ztype1encrypt(i_ctx_t *i_ctx_p)
 {
@@ -43,7 +43,7 @@ ztype1decrypt(i_ctx_t *i_ctx_p)
 }
 private int
 type1crypt(i_ctx_t *i_ctx_p,
-	   int (*proc)(P4(byte *, const byte *, uint, ushort *)))
+	   int (*proc)(byte *, const byte *, uint, ushort *))
 {
     os_ptr op = osp;
     crypt_state state;
@@ -109,17 +109,22 @@ zexD(i_ctx_t *i_ctx_p)
     (*s_exD_template.set_defaults)((stream_state *)&state);
     if (r_has_type(op, t_dictionary)) {
 	uint cstate;
+        bool is_eexec;
 
 	check_dict_read(*op);
 	if ((code = dict_uint_param(op, "seed", 0, 0xffff, 0x10000,
 				    &cstate)) < 0 ||
 	    (code = dict_int_param(op, "lenIV", 0, max_int, 4,
-				   &state.lenIV)) < 0
+				   &state.lenIV)) < 0 ||
+	    (code = dict_bool_param(op, "eexec", false,
+				   &is_eexec)) < 0
 	    )
 	    return code;
 	state.cstate = cstate;
+        state.binary = (is_eexec ? -1 : 1);
 	code = 1;
     } else {
+        state.binary = 1;
 	code = eexec_param(op, &state.cstate);
     }
     if (code < 0)
@@ -152,9 +157,8 @@ zexD(i_ctx_t *i_ctx_p)
 		}
 		pss->binary_to_hex = 0;
 	    }
-	    state.record_left = (s->cursor.r.limit - s->cursor.r.ptr)
-				+ pss->record_left;
-	}
+	    state.record_left = pss->record_left;
+	} 
     }
     return filter_read(i_ctx_p, code, &s_exD_template, (stream_state *)&state, 0);
 }
