@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: zshade.c,v 1.3 2002/04/23 11:58:48 easysw Exp $ */
+/* $Id: zshade.c,v 1.14 2002/10/31 16:19:48 alexcher Exp $ */
 /* PostScript language interface to shading */
 #include "memory_.h"
 #include "ghost.h"
@@ -38,7 +38,7 @@
 #include "store.h"
 
 /* Forward references */
-private int shading_param(P2(const_os_ptr op, const gs_shading_t ** ppsh));
+private int shading_param(const_os_ptr op, const gs_shading_t ** ppsh);
 
 /* ---------------- Standard operators ---------------- */
 
@@ -143,8 +143,8 @@ shading_param(const_os_ptr op, const gs_shading_t ** ppsh)
 extern_st(st_color_space);
 
 typedef int (*build_shading_proc_t)
-     (P5(i_ctx_t *i_ctx_p, const ref *op, const gs_shading_params_t *params,
-	 gs_shading_t **ppsh, gs_memory_t *mem));
+     (i_ctx_t *i_ctx_p, const ref *op, const gs_shading_params_t *params,
+      gs_shading_t **ppsh, gs_memory_t *mem);
 
 /* Operators */
 
@@ -224,10 +224,11 @@ fail:
 
 /* Collect a Function value. */
 private int
-build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn, int num_inputs,
-		       gs_memory_t *mem)
+build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
+		       int num_inputs, gs_memory_t *mem)
 {
     ref *pFunction;
+    int code;
 
     *ppfn = 0;
     if (dict_find_string(op, "Function", &pFunction) <= 0)
@@ -237,7 +238,6 @@ build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
 	gs_function_t **Functions;
 	uint i;
 	gs_function_AdOt_params_t params;
-	int code;
 
 	check_read(*pFunction);
 	if (size == 0)
@@ -253,7 +253,7 @@ build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
 	    if (code < 0)
 		break;
 	}
-	params.m = 1;
+	params.m = num_inputs;
 	params.Domain = 0;
 	params.n = size;
 	params.Range = 0;
@@ -262,9 +262,28 @@ build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
 	    code = gs_function_AdOt_init(ppfn, &params, mem);
 	if (code < 0)
 	    gs_function_AdOt_free_params(&params, mem);
-	return code;
-    } else
-	return fn_build_function(i_ctx_p, pFunction, ppfn, mem);
+    } else {
+	code = fn_build_function(i_ctx_p, pFunction, ppfn, mem);
+	if (code < 0)
+	    return code;
+	if ((*ppfn)->params.m != num_inputs) {
+	    gs_function_free(*ppfn, true, mem);
+	    return_error(e_rangecheck);
+	}
+    }
+    return code;
+}
+
+/* According to PLRM 3rd ed, p. 264  "indexed color space is not
+ * allowed in any shading whose color values are generated by a function;
+ * this applies to any shading dictionary that contains a Function entry."
+ * Adobe interpreters follow PLRM in this respect and we follow them.
+ */
+private int
+check_indexed_vs_function(const gs_color_space *pcs, const gs_function_t *foo)
+{ if (foo && gs_color_space_get_index(pcs) == gs_color_space_index_Indexed)
+    return_error(e_rangecheck);
+  return 0;
 }
 
 /* ------ Build shadings ------ */
@@ -277,14 +296,17 @@ build_shading_1(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     gs_shading_Fb_params_t params;
     int code;
     static const float default_Domain[4] = {0, 1, 0, 1};
+    ref *pmatrix;
 
     *(gs_shading_params_t *)&params = *pcommon;
     gs_make_identity(&params.Matrix);
     params.Function = 0;
     if ((code = dict_floats_param(op, "Domain", 4, params.Domain,
 				  default_Domain)) < 0 ||
-	(code = dict_matrix_param(op, "Matrix", &params.Matrix)) < 0 ||
+	(dict_find_string(op, "Matrix", &pmatrix) > 0 &&
+	 (code = read_matrix(pmatrix, &params.Matrix)) < 0) ||
 	(code = build_shading_function(i_ctx_p, op, &params.Function, 2, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = gs_shading_Fb_init(ppsh, &params, mem)) < 0
 	) {
 	gs_free_object(mem, params.Function, "Function");
@@ -316,6 +338,8 @@ build_directional_shading(i_ctx_t *i_ctx_p, const ref * op, float *Coords, int n
 	(code = build_shading_function(i_ctx_p, op, pFunction, 1, mem)) < 0
 	)
 	return code;
+    if (!*pFunction)
+	    return_error(e_undefined);
     if (dict_find_string(op, "Extend", &pExtend) <= 0)
 	Extend[0] = Extend[1] = false;
     else {
@@ -346,6 +370,7 @@ build_shading_2(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code = build_directional_shading(i_ctx_p, op, params.Coords, 4,
 					  params.Domain, &params.Function,
 					  params.Extend, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = gs_shading_A_init(ppsh, &params, mem)) < 0
 	) {
 	gs_free_object(mem, params.Function, "Function");
@@ -371,6 +396,7 @@ build_shading_3(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code = build_directional_shading(i_ctx_p, op, params.Coords, 6,
 					  params.Domain, &params.Function,
 					  params.Extend, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = gs_shading_R_init(ppsh, &params, mem)) < 0
 	) {
 	gs_free_object(mem, params.Function, "Function");
@@ -392,6 +418,7 @@ build_mesh_shading(i_ctx_t *i_ctx_p, const ref * op,
 		   gs_memory_t *mem)
 {
     int code;
+    float *data = 0;
     ref *pDataSource;
 
     *pDecode = 0;
@@ -400,14 +427,12 @@ build_mesh_shading(i_ctx_t *i_ctx_p, const ref * op,
 	return_error(e_rangecheck);
     if (r_is_array(pDataSource)) {
 	uint size = r_size(pDataSource);
-	float *data =
-	    (float *)gs_alloc_byte_array(mem, size, sizeof(float),
-					 "build_mesh_shading");
-	int code;
 
+	data = (float *)gs_alloc_byte_array(mem, size, sizeof(float),
+					    "build_mesh_shading");
 	if (data == 0)
 	    return_error(e_VMerror);
-	code = float_params(pDataSource->value.refs + size - 1, size, data);
+	code = process_float_array(pDataSource, size, data);
 	if (code < 0) {
 	    gs_free_object(mem, data, "build_mesh_shading");
 	    return code;
@@ -431,36 +456,44 @@ build_mesh_shading(i_ctx_t *i_ctx_p, const ref * op,
 	    default:
 		return_error(e_typecheck);
 	}
+    code = build_shading_function(i_ctx_p, op, pFunction, 1, mem);
+    if (code < 0) {
+	gs_free_object(mem, data, "build_mesh_shading");
+	return code;
+    }
     if (data_source_is_array(params->DataSource)) {
 	params->BitsPerCoordinate = 0;
 	params->BitsPerComponent = 0;
     } else {
-	int num_decode =
-	    4 + gs_color_space_num_components(params->ColorSpace) * 2;
+	int num_decode = 4 +
+	    (*pFunction != 0 ? 1 :
+	     gs_color_space_num_components(params->ColorSpace)) * 2;
 
-/****** FREE FLOAT ARRAY DATA ON ERROR ******/
 	if ((code = dict_int_param(op, "BitsPerCoordinate", 1, 32, 0,
-				   &params->BitsPerCoordinate)) < 0 ||
+				   &params->BitsPerCoordinate)) >= 0 &&
 	    (code = dict_int_param(op, "BitsPerComponent", 1, 16, 0,
-				   &params->BitsPerComponent)) < 0
-	    )
-	    return code;
-	*pDecode = (float *)
-	    gs_alloc_byte_array(mem, num_decode, sizeof(float),
-				"build_mesh_shading");
-	if (*pDecode == 0)
-	    return_error(e_VMerror);
-	code = dict_floats_param(op, "Decode", num_decode, *pDecode, NULL);
-	if (code < 0) {
-	    gs_free_object(mem, *pDecode, "build_mesh_shading");
-	    *pDecode = 0;
-	    return code;
+				   &params->BitsPerComponent)) >= 0
+	    ) {
+	    *pDecode = (float *)
+		gs_alloc_byte_array(mem, num_decode, sizeof(float),
+				    "build_mesh_shading");
+	    if (*pDecode == 0)
+		code = gs_note_error(e_VMerror);
+	    else {
+		code = dict_floats_param(op, "Decode", num_decode, *pDecode, NULL);
+		if (code < 0) {
+		    gs_free_object(mem, *pDecode, "build_mesh_shading");
+		    *pDecode = 0;
+		}
+	    }
 	}
     }
-    code = build_shading_function(i_ctx_p, op, pFunction, 1, mem);
     if (code < 0) {
-	gs_free_object(mem, *pDecode, "build_mesh_shading");
-	*pDecode = 0;
+	if (*pFunction != 0) {
+	    gs_function_free(*pFunction, true, mem);
+	    *pFunction = 0;
+	}
+	gs_free_object(mem, data, "build_mesh_shading");
     }
     return code;
 }
@@ -490,6 +523,7 @@ build_shading_4(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = flag_bits_param(op, (gs_shading_mesh_params_t *)&params,
 				&params.BitsPerFlag)) < 0 ||
 	(code = gs_shading_FfGt_init(ppsh, &params, mem)) < 0
@@ -518,6 +552,7 @@ build_shading_5(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = dict_int_param(op, "VerticesPerRow", 2, max_int, 0,
 			       &params.VerticesPerRow)) < 0 ||
 	(code = gs_shading_LfGt_init(ppsh, &params, mem)) < 0
@@ -546,6 +581,7 @@ build_shading_6(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = flag_bits_param(op, (gs_shading_mesh_params_t *)&params,
 				&params.BitsPerFlag)) < 0 ||
 	(code = gs_shading_Cp_init(ppsh, &params, mem)) < 0
@@ -574,6 +610,7 @@ build_shading_7(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
+	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
 	(code = flag_bits_param(op, (gs_shading_mesh_params_t *)&params,
 				&params.BitsPerFlag)) < 0 ||
 	(code = gs_shading_Tpp_init(ppsh, &params, mem)) < 0
